//
// FileScanner.cpp
// 

#include "EventInfoScanner.h"
#include "FindChange.h"

#include "Common/ParamList.h"
#include "GDLog/GDLog.h"
#include "GDLog/GDTrace.h"
#include "FileScanner/FileScanner.h"

using namespace std;


namespace InforGuard{
namespace FileScanner{


long CFileScanner::m_delay;
std::map<std::string, CFindChange*> CFileScanner::m_dirInfo;


CFileScanner::CFileScanner():
	CMonitorInterface("FileScanner", SCANNER)
{
}


CFileScanner::~CFileScanner()
{
}


bool CFileScanner::Initialize( const char * sParams )
{
	CGDTrace trace( LOG_L6, "CFileScanner::Initialize(%s)", sParams );

	// һĿ¼һ߳̾ò
	bool ret = true;

	CParamList paramList( sParams, "|" );
	m_delay = atoi( paramList.GetStringValue( "delay" ).c_str() );

	if ( m_delay > 60*60*24 )
	{
		m_delay = 60*60*24;
	}
	else if ( m_delay < 1 )
	{
		m_delay = 1;
	}

	CGDLog::WriteSysLog( "2038:CFileScanner Initialize OK" );

	return trace.SetRetResult( ret );
}


bool CFileScanner::Uninitialize()
{
	CGDTrace trace( LOG_L6, "CFileScanner::Uninitialize" );

	// һĿ¼һ߳̾ò
	bool ret = true;

	CGDLog::WriteSysLog( "2039:CFileScanner Uninitialize OK" );

	return trace.SetRetResult( ret );
}


bool CFileScanner::WatchDirectory( CMonitorInformation& moniInfo )
{
	CGDTrace trace( LOG_L6, "CFileScanner::WatchDirectory" );

	// dirȷ,߳
	std::map<std::string, CFindChange*>::iterator iter;
	iter = m_dirInfo.find( moniInfo.m_monitorPath );
	
	if (iter == m_dirInfo.end()) // ûĿ¼ض
	{
		CFindChange* dirObj = new CFindChange( moniInfo, m_delay );

		// ߳
		CGDLog::WriteSysLog( "2040:CFileScanner::WatchDirectory start thread..." );

		dirObj->Start();

		m_dirInfo.insert( std::make_pair( string( moniInfo.m_monitorPath.c_str() ), dirObj ) );
	}
    else
    {
        CFindChange* dirObj = iter->second;
        if( dirObj )
        {
		    dirObj->setPolicy( moniInfo );
        }
    }

	return trace.SetRetResult( true );
}



bool CFileScanner::UnwatchDirectory( const char * sDirId )
{
	CGDTrace trace( LOG_L6, "CFileScanner:UnwatchDirectory(%s)", sDirId );

	// dirȷ,ֹͣ߳
	std::map<std::string, CFindChange*>::iterator iter;
	iter = m_dirInfo.find( sDirId );
	
	if (iter == m_dirInfo.end()) // ûĿ¼ļ
	{
		return trace.SetRetResult( false );
	}
	else
	{
		// ֹͣ߳
		CGDLog::WriteSysLog( "2041:CFileScanner::WatchDirectory stop thread..." );

		iter->second->Stop();
		delete iter->second;
		m_dirInfo.erase( iter );
		return trace.SetRetResult( true );
	}
}


bool CFileScanner::IsWatched( const char * sDirId )
{
	CGDTrace trace( LOG_L6, "CFileScanner::IsWatched(%s)", sDirId );
	// dirȷ,ȡ״̬
	std::map<std::string, CFindChange*>::iterator iter;
	iter = m_dirInfo.find( sDirId );
	
	if (iter == m_dirInfo.end()) // ûĿ¼ļ
	{
		return trace.SetRetResult( false );
	}
	else
	{
		// ״̬
		InforGuard::GDThreadPool::enumThreadStatus status = iter->second->GetStatus();
		bool bRet = ( InforGuard::GDThreadPool::TS_STARTING == status || InforGuard::GDThreadPool::TS_STARTED == status );
		return trace.SetRetResult( bRet );
	}
}


bool CFileScanner::ReadEvent( EVENT_S& event )
{
	static bool bHasEvent = false;

	// ¼ȡ¼
	if ( CEventInfoScanner::Read( event ) )
	{
		if ( !bHasEvent )
		{
			CGDTrace trace( LOG_L6, "CFileScanner::ReadEvent(%d,%s)", event.event_type, event.o_filepath );
			bHasEvent = true;
		}
	}
	else
	{
		if ( bHasEvent )
		{
			CGDTrace trace( LOG_L6, "CFileScanner::ReadEvent()=false" );
			bHasEvent = false;
		}
	}

	return bHasEvent;
}


}	// namespace FileScanner
}	// namespace InforGuard







