//======================================================================
// 
// Irp.c
//
// Copyright (C) CVIC Software Engineering CO.,LTD.
//
//======================================================================
#include "Define.h"
#include "GuardFileDriver.h"
#include "FastIo.h"
#include "Disk.h"
#include "Filter.h"
#include "../include/Driver/Ioctlcmd.h"
#include "Event.h"


// Irpݵĳʼ
VOID InitIrp()
{
	KeInitializeSpinLock( &gSequenceMutex );
}


VOID UnInitIrp()
{
}


//----------------------------------------------------------------------
//      P A T T E R N   M A T C H I N G   R O U T I N E S
//----------------------------------------------------------------------

ULONG MyInterlockedIncrement( PULONG Number )
{
    if( *NtBuildNumber < NT4FINAL ) {

        //
        // Just use the filter spin lock
        //
        return ExInterlockedAddUlong( Number, 1, &gSequenceMutex );

    } else {

        return InterlockedIncrement( Number );
    }
}


//----------------------------------------------------------------------
//
// GuardFileDriverHookRoutine
//
// This routine is the main hook routine where we figure out what
// calls are being sent to the file system.
//
//----------------------------------------------------------------------
NTSTATUS GuardFileDriverHookRoutine( PDEVICE_OBJECT HookDevice, IN PIRP Irp )
{
    PIO_STACK_LOCATION  currentIrpStack = IoGetCurrentIrpStackLocation(Irp);
    PFILE_OBJECT        fileObject;
    PHOOK_EXTENSION     hookExt;
    PCHAR               fullPathName;
    ULONG               seq = MyInterlockedIncrement(&gSequence);
    
    //
    // ϱ־FALSE, УTRUE, 
    //
    BOOLEAN				bBlockFlag = FALSE;
	ULONG				eventType = 0;
    ULONG				disposition;

    //
    // Allocate a buffer
    //
    fullPathName = ExAllocatePool( PagedPool, MAXPATHLEN );
    fullPathName[0] = 0;

    //
    // Extract the file object from the IRP
    //
    fileObject    = currentIrpStack->FileObject;

    //
    // Point at the device extension, which contains information on which
    // file system this IRP is headed for
    //
    hookExt = HookDevice->DeviceExtension;

    //
    // If a GUI is up there, get the canonical pathname
    //
	if( gFilterOn )
	{
        if( currentIrpStack->MajorFunction == IRP_MJ_CREATE )
		{
            // Clear any existing fileobject/name association stored in the hash table
            // 
            FreeHashEntry( fileObject );
        }

        GetFullPath( (BOOLEAN) (currentIrpStack->MajorFunction == IRP_MJ_CREATE), 
                            fileObject, hookExt, fullPathName );
	}

    //
    // Determine what function we're dealing with
    //
    switch( currentIrpStack->MajorFunction ) {

    case IRP_MJ_CREATE:

		if( gFilterOn ) 
		{
			MYULONG processID = (MYULONG)PsGetCurrentProcessId();
			//
			// ļĲѡ
			//
			disposition = (currentIrpStack->Parameters.Create.Options >> 24) & 0xFF;
			if( disposition == FILE_CREATE 
				|| disposition == FILE_OVERWRITE_IF 
				|| disposition == FILE_OPEN_IF )	// FILE_OPEN_IF Ǵڼ򿪣粻򴴽򿪣
			{
				if ( !IsSuperProcess() && IsProtectFile( fullPathName, MONITOR_EVENT_DRIVER_CREATE ) )
				{
					bBlockFlag = TRUE;
					DbgPrint(("GuardFileDriver: GuardFileDriverHookRoutine-Create\n"));
					AddEvent( processID, MONITOR_EVENT_DRIVER_CREATE, fullPathName );
					eventType = MONITOR_EVENT_DRIVER_CREATE;
				}
			}
			else if( disposition == FILE_OPEN )
			{
				ACCESS_MASK access = currentIrpStack->Parameters.Create.SecurityContext->DesiredAccess;
				
				if( (access & FILE_GENERIC_WRITE) == FILE_GENERIC_WRITE )	// ļ򿪷ʽǷΪдʽ
				{
					if ( !IsSuperProcess() && IsProtectFile( fullPathName, MONITOR_EVENT_DRIVER_CHANGE ) )
					{
						bBlockFlag = TRUE;
						DbgPrint(("GuardFileDriver: GuardFileDriverHookRoutine-Change\n"));
    					AddEvent( processID, MONITOR_EVENT_DRIVER_CHANGE, fullPathName );
						eventType = MONITOR_EVENT_DRIVER_CHANGE;
					}
				}
			}
		}
		else
		{
			FreeHashEntry( fileObject );
		}
        break;

    case IRP_MJ_CLOSE:

        //
        // This fileobject/name association can be discarded now.
        //
        FreeHashEntry( fileObject );
        break;

    case IRP_MJ_SET_INFORMATION:

		if ( gFilterOn )
		{
			if ( currentIrpStack->Parameters.SetFile.FileInformationClass == FileDispositionInformation )		// ɾ
			{
				if ( !IsSuperProcess() && IsProtectFile( fullPathName, MONITOR_EVENT_DRIVER_DELETE ) )
				{
					MYULONG processID = (MYULONG)PsGetCurrentProcessId();
					bBlockFlag = TRUE;
					DbgPrint(("GuardFileDriver: GuardFileDriverHookRoutine-Delete\n"));
      				AddEvent( processID, MONITOR_EVENT_DRIVER_DELETE, fullPathName );
					eventType = MONITOR_EVENT_DRIVER_DELETE;
				}
			}
			else if ( currentIrpStack->Parameters.SetFile.FileInformationClass==FileRenameInformation )	// 
			{
				if ( !IsSuperProcess() && IsProtectFile( fullPathName, MONITOR_EVENT_DRIVER_RENAME ) )
				{
					MYULONG processID = (MYULONG)PsGetCurrentProcessId();
					bBlockFlag = TRUE;
					DbgPrint(("GuardFileDriver: GuardFileDriverHookRoutine-Rename\n"));
      				AddEvent( processID, MONITOR_EVENT_DRIVER_RENAME, fullPathName );
					eventType = MONITOR_EVENT_DRIVER_RENAME;
				}
			}
		}
        break;

    case IRP_MJ_SET_SECURITY:
		if( gFilterOn ) 
		{
			if ( !IsSuperProcess() && IsProtectFile( fullPathName, MONITOR_EVENT_DRIVER_CHANGE ) )
			{
				// ޸ļȫԵΪ

				MYULONG processID = (MYULONG)PsGetCurrentProcessId();
				bBlockFlag = TRUE;
				DbgPrint(("GuardFileDriver: GuardFileDriverHookRoutine-Change\n"));
    			AddEvent( processID, MONITOR_EVENT_DRIVER_CHANGE, fullPathName );
				eventType = MONITOR_EVENT_DRIVER_CHANGE;
			}
		}
		break;

    default:

        break;
    }
    
    //
    // Free the buffer
    //
    if ( fullPathName )
	{
        ExFreePool( fullPathName );
    }

    //
	// ϱ־л
	//
    if( bBlockFlag )
    {
		//
		// ǷӦIrpԱֵд֤
		//
		Irp->IoStatus.Information = 0;
		if ( eventType == MONITOR_EVENT_DRIVER_CREATE )		// ½
		{
			Irp->IoStatus.Status      = STATUS_SUCCESS;
			IoCompleteRequest( Irp, IO_NO_INCREMENT );
			return STATUS_SUCCESS;
		}
		else if ( eventType == MONITOR_EVENT_DRIVER_DELETE )	// ɾ
		{
			Irp->IoStatus.Status      = STATUS_ACCESS_DENIED;
			IoCompleteRequest( Irp, IO_NO_INCREMENT );
			return STATUS_ACCESS_DENIED;
		}
		else if ( eventType == MONITOR_EVENT_DRIVER_CHANGE )	// ޸
		{
			Irp->IoStatus.Status      = STATUS_ACCESS_DENIED;
			IoCompleteRequest( Irp, IO_NO_INCREMENT );
			return STATUS_ACCESS_DENIED;
		}
		else	// 
		{
			Irp->IoStatus.Status      = STATUS_ACCESS_DENIED;
			IoCompleteRequest( Irp, IO_NO_INCREMENT );
			return STATUS_ACCESS_DENIED;
		}
    }
	
    //
    // ֱ´IRP
    //
	IoSkipCurrentIrpStackLocation( Irp );

    return IoCallDriver( hookExt->FileSystem, Irp );
}


//----------------------------------------------------------------------
//
// GuardFileDriverDeviceRoutine
//
// In this routine we handle requests to our own device. The only 
// requests we care about handling explicitely are IOCTL commands that
// we will get from the GUI. We also expect to get Create and Close 
// commands when the GUI opens and closes communications with us.
//
//----------------------------------------------------------------------
NTSTATUS GuardFileDriverDeviceRoutine( IN PDEVICE_OBJECT DeviceObject, IN PIRP Irp )
{
    PIO_STACK_LOCATION  irpStack;
    PVOID               inputBuffer;
    PVOID               outputBuffer;
    ULONG               inputBufferLength;
    ULONG               outputBufferLength;
    ULONG               ioControlCode;

    //
    // Go ahead and set the request up as successful
    //
    Irp->IoStatus.Status      = STATUS_SUCCESS;
    Irp->IoStatus.Information = 0;

    //
    // Get a pointer to the current location in the Irp. This is where
    // the function codes and parameters are located.
    //
    irpStack = IoGetCurrentIrpStackLocation (Irp);

    //
    // Get the pointer to the input/output buffer and its length
    //
    inputBuffer     = Irp->AssociatedIrp.SystemBuffer;
    inputBufferLength   = irpStack->Parameters.DeviceIoControl.InputBufferLength;
    outputBuffer    = Irp->AssociatedIrp.SystemBuffer;
    outputBufferLength  = irpStack->Parameters.DeviceIoControl.OutputBufferLength;
    ioControlCode   = irpStack->Parameters.DeviceIoControl.IoControlCode;

    switch (irpStack->MajorFunction) {
    case IRP_MJ_CREATE:

        DbgPrint(("GuardFileDriver: IRP_MJ_CREATE\n"));

        // 
        // Start the sequence number at 0
        // 
        gSequence = 0;
        break;

    case IRP_MJ_CLOSE:

        DbgPrint(("GuardFileDriver: IRP_MJ_CLOSE\n"));

        //
        // A GUI is closing communication
        //
        gFilterOn  = FALSE;

        //
        // If the GUI has no more references to us, reset the output
        // buffers and hash table.
        //
        //GuardFileDriverResetStore();
        HashCleanup();

        //
        // Disconnect from file systems
        //
        HookDriveSet( 0, DeviceObject->DriverObject );
        break;

    case IRP_MJ_DEVICE_CONTROL:

        //
        // This path will never execute because we have registered a 
        // fast I/O path for device control. That means that the fast I/O entry 
        // point will ALWAYS be called for Device Control operations
        //
        DbgPrint (("GuardFileDriver: IRP_MJ_DEVICE_CONTROL\n"));

        //
        // Get output buffer if its passed as an MDL
        //
        if( Irp->MdlAddress ) {

            outputBuffer = MmGetSystemAddressForMdl( Irp->MdlAddress );
        }

        //
        // Its a request from the GUI. Simply call our fast handler.
        //
        GuardFileDriverFastIoDeviceControl( irpStack->FileObject, TRUE,
                                    inputBuffer, inputBufferLength, 
                                    outputBuffer, outputBufferLength,
                                    ioControlCode, &Irp->IoStatus, DeviceObject );
        break;
    }

    //
    // Complete the IRP
    //
    IoCompleteRequest( Irp, IO_NO_INCREMENT );
    return STATUS_SUCCESS;   
}


//----------------------------------------------------------------------
//
// GuardFileDriverDispatch
//
// Based on which device the Irp is destined for we call either the
// filesystem filter function, or our own device handling routine.
//
//----------------------------------------------------------------------
NTSTATUS GuardFileDriverDispatch( IN PDEVICE_OBJECT DeviceObject, IN PIRP Irp )
{
    //
    // Determine if its a request from the ManagerApp to us, or one that is
    // directed at a file system driver that we've hooked
    //
    if ( gUIDevice != DeviceObject )
	{
		if ( gGuardFileDriverDriver == DeviceObject->DriverObject )
		{
			return( GuardFileDriverHookRoutine( DeviceObject, Irp ));
		}
		else	// 
		{
			Irp->IoStatus.Status      = STATUS_ACCESS_DENIED;
			IoCompleteRequest( Irp, IO_NO_INCREMENT );
			return STATUS_ACCESS_DENIED;
		}
    }
	else
	{

        return( GuardFileDriverDeviceRoutine( DeviceObject, Irp ));
    }
}
