/***********************************************************************
*
* ļ  Nofier.cpp
* ģĿģڻȡļ¼
* ߼ڣ  wang_jin 2009-08-26
* 汾 1.0
* ȨϢCVICSE Middleware
* ļ¼wang_jin at 2009-08-29
*
***********************************************************************/

#include "NofierWin32.h"

#include "Common/Common.h"
#include "Common/ExceptionError.h"
#include "Common/ParamList.h"
#include "Common/DateTime.h"
#include "Common/GDTimer.h"
#include "Common/ConvertString.h"
#include "FileOpt/FileOpt.h"
#include "GDLog/GDLog.h"
#include "GDLog/GDTrace.h"
#include "GDLog/RunLog.h"
#include "GDLog/LogFile.h"
#include "GDLog/ThreadData.h"

#include <assert.h>
#ifdef WIN32
#	include <process.h> // for _beginthreadex
#	include <windows.h>
#	include <algorithm> // for std::sort
#	include <sstream>
#else
#	define InterlockedIncrement atomic_inc
#endif

using namespace std;


UINT CNofier::m_dwDefaultFlags = 0;
unsigned int CNofier::CDirInfo::ReadDirectoryChanges()
{
	// ִ첽I/O
	// UINT nBytes = 0;
	unsigned long nBytes = 0;

#ifdef WIN32	
	BOOL bRet = ReadDirectoryChangesW(
		m_hDir, 
		m_evtBuf.buffer + m_evtBuf.offset, 
		MAX_BUFFER - m_evtBuf.offset, 
		m_bSubDir, 
		m_dwFlags, 
		&nBytes, 
		&m_ol, 
		NULL );
	if ( bRet )
	{
		return 0;
	}
	else
	{
		GDPrint("read() false\n");
		CGDLog::WriteErrLog( "CNofier:CompletionRoutine() ReadDirectoryChangesW 1 error[%d]", GetLastError() );
		return -1;
	}
#else
	int nRet = read( 
		m_fd, 
		m_evtBuf.buffer + m_evtBuf.offset, 
		MAX_BUFFER - m_evtBuf.offset );
	if ( nRet < 0 )
	{
		ErrCode dwErr = GetLastError();
		GDPrint( "CNofier:CompletionRoutine() read() error[%d]", dwErr );
		CGDLog::WriteErrLog( "CNofier:CompletionRoutine() read() error[%d]", dwErr );
		return -1;
	}
	else
	{
		return nRet;
	}
#endif
}


CNofier::CNofier()
{
	m_dwDefaultFlags = 
#ifdef WIN32
		FILE_NOTIFY_CHANGE_FILE_NAME |	// ļƱ仯
		FILE_NOTIFY_CHANGE_DIR_NAME |	// Ŀ¼Ʊ仯
	//	FILE_NOTIFY_CHANGE_ATTRIBUTES |	// ļԱ仯ֻ||鵵
	//	FILE_NOTIFY_CHANGE_SIZE |		// ļС仯
		FILE_NOTIFY_CHANGE_LAST_WRITE |	// ޸ʱ
	//	FILE_NOTIFY_CHANGE_LAST_ACCESS| // ʱ
	//	FILE_NOTIFY_CHANGE_CREATION |	// ¼
	//	FILE_NOTIFY_CHANGE_SECURITY |	// ȫԱ仯
#else
	//	IN_ACCESS | // File was accessed
		IN_MODIFY | // File was modified
	//	IN_ATTRIB | // Metadata changed
		IN_CLOSE_WRITE | // Writable file closed
		IN_CLOSE_NOWRITE | // Unwritable file closed
	//	IN_OPEN | // File was opened
		IN_MOVED_FROM | // File was moved from X
		IN_MOVED_TO | // File was moved to Y
		IN_CREATE | // Subfile was created
		IN_DELETE | // Subfile was deleted
#endif
		0;

	m_nMilliSeconds = 500; // ¼󣬹500Ϊ¼
}

void CNofier::SetEventFlag( UINT nFlags )
{
	CGDTrace trace( LOG_L6, "CNofier::SetEventFlag(%d)", nFlags );
	
	m_dwDefaultFlags = nFlags;
}

void CNofier::SetEventDelay( int ms )
{
	CGDTrace trace( LOG_L6, "CNofier::SetEventFlag(%d)", ms );
	
	if ( ms <= 0 ) ms = 100;
	else if ( ms > 30000 ) ms = 30000;

	m_nMilliSeconds = ms;
}

CNofier::~CNofier()
{
	if ( m_nThreads > 0 )
	{
		Uninitialize();
	}

	// ɾ
	map < string, CEventInfo* >::iterator iter;
	for ( iter = m_evtmap.begin(); iter != m_evtmap.end(); )
	{
		delete (CEventInfo*) ( *iter ).second;
		m_evtmap.erase( iter++ );
	}
	m_filter.clear();
}

// ʼʧ쳣һ˳
bool CNofier::Initialize( const char * sParams )
{
	CGDTrace trace( LOG_L6, "CNofier:Initialize(%s)", sParams );
	
	// Բ
	CParamList pl( sParams, "|" );
	if ( sParams && *sParams )
	{		
		this->SetEventDelay( pl.GetNumValue( "delay" ) );
	}
	SetEventFlag( m_dwDefaultFlags );

	// ̸߳(˫)
#ifdef WIN32
	SYSTEM_INFO sysInfo = {0};
	GetSystemInfo( &sysInfo );
	m_nThreads = sysInfo.dwNumberOfProcessors;
#else
	m_nThreads = sysconf( _SC_NPROCESSORS_ONLN );
#endif
	GDPrint( "nThreads=[%d]\n", m_nThreads );

#ifdef WIN32
	// ɶ˿
	m_hIOCP = CreateIoCompletionPort( INVALID_HANDLE_VALUE, NULL, 0, 0 );
	if ( NULL == m_hIOCP )
	{
		ErrCode dwErr = GetLastError();
		assert(0);
		GDPrint( "CreateIoCompletionPort1 Failed: [%d]\n", GetLastError() );
		return trace.SetRetResult( false );
	}
#else
	// epoll
	m_nEPFD = epoll_create(256);
	if ( m_nEPFD < 0 )
	{
		ErrCode dwErr = GetLastError();
		assert(0);
		GDPrint( "epoll_create failed: [%d]\n", GetLastError() );
		return trace.SetRetResult( false );
	}
#endif

#ifdef WIN32	
	// ʼٽ
	InitializeCriticalSection( &m_cs );
#else
	pthread_mutex_init( &m_cs, NULL );
#endif

#ifndef NDEBUG // ʼԶ̹()
	if (!pl.GetStringValue( "viewerip" ).empty())
	{
		if ( m_queue.init( pl.GetStringValue( "viewerip" ).c_str(), pl.GetNumValue( "viewerport" ), 0 ) )
		{
			GDPrint( "ʼɹ\n" );
		}
	}		
#endif

#ifdef WIN32
	// ¼
	m_hEvent = CreateEvent( NULL, false, false, NULL ); // ǰfalse:Զλfalse:ʼź
	if ( NULL == m_hEvent )
	{
		ErrCode dwErr = GetLastError();
		assert(0);
		GDPrint( "CreateEvent Failed: [%d]\n", dwErr );
		CloseHandle( m_hIOCP );
		return trace.SetRetResult( false );
	}
#else
	pthread_mutex_init( &m_mutex, NULL );
	int nRet = pthread_cond_init( &m_cond, NULL );
	if ( nRet < 0 )
	{
		ErrCode dwErr = GetLastError();
		assert(0);
		GDPrint( "CreateEvent Failed: [%d]\n", dwErr );
		close( m_nEPFD );
		return trace.SetRetResult( false );
	}
#endif

	// ߳(ֻһ߳)
#ifdef WIN32
	UINT dwThreadId = 0;
	HANDLE hThread = (HANDLE)_beginthreadex( NULL, 0, CNofier::FilterRoutine, this, 0, &dwThreadId );
	if ( NULL == hThread )
	{
		ErrCode dwErr = GetLastError();
		assert(0);
		GDPrint( "CreateThread Failed: [%d]\n", dwErr );
		CloseHandle( m_hIOCP );
		return trace.SetRetResult( false );
	}

	WaitForSingleObject( m_hEvent, INFINITE );
	CloseHandle( hThread );
#else
	pthread_t hThread;
	int nThreadId = pthread_create( &hThread, NULL, CNofier::FilterRoutine, (void*)this );
	if ( nThreadId )
	{
		ErrCode dwErr = GetLastError();
		assert(0);
		GDPrint( "pthread_create failed: [%d]\n", dwErr );
		close( m_nEPFD );
		return trace.SetRetValue( 0 );
	}

	pthread_cond_wait( &m_cond, &m_mutex );
#endif

	// ߳(߳)
	for ( int ii=0; ii<m_nThreads; ii++ )
	{
#ifdef WIN32
		hThread = (HANDLE)_beginthreadex( NULL, 0, CNofier::CompletionRoutine, this, 0, &dwThreadId );
		if ( NULL == hThread )
		{
			ErrCode dwErr = GetLastError();
			assert(0);
			GDPrint( "CreateThread Failed: [%d]\n", GetLastError() );
			CloseHandle( m_hIOCP );
			return trace.SetRetResult( false );
		}
#else
		nThreadId = pthread_create( &hThread, NULL, CNofier::CompletionRoutine, (void*)this );
		if ( nThreadId )
		{
			ErrCode dwErr = GetLastError();
			assert(0);

			GDPrint( "pthread_create failed: [%d]\n", GetLastError() );
			close( m_nEPFD );
			return trace.SetRetResult( false );
		}
#endif

#ifdef WIN32
		// ߳ȼΪ
		SetThreadPriority( hThread, 
			THREAD_PRIORITY_TIME_CRITICAL // Time-criticalʵʱ
			//THREAD_PRIORITY_HIGHEST		// Highestߣ
			//THREAD_PRIORITY_ABOVE_NORMAL  // Above normalWindows 98֧֣
			//THREAD_PRIORITY_NORMAL // Normal
			//THREAD_PRIORITY_BELOW_NORMAL  // Below normalWindows 98֧֣
			//THREAD_PRIORITY_LOWEST // Lowestͣ
			//THREAD_PRIORITY_IDLE // IdleУ
			);

		CloseHandle( hThread );
#endif
	}


	return trace.SetRetResult( true );
}

bool CNofier::WatchDirectory( CMonitorInformation& moniInfo )
{
	CGDTrace trace( LOG_L6, "CNofier:WatchDirectory(%s)", moniInfo.m_monitorPath.c_str() );
	
	// жǷ
	char szKey[525] = {0};
	sprintf( szKey, "%s", moniInfo.m_monitorPath.c_str() );
	map< string, CDirInfo* >::iterator it = m_dirmap.find( szKey );
	if ( it == m_dirmap.end() )
	{
		CDirInfo* pDirInfo = new CDirInfo( moniInfo, m_dwDefaultFlags );        
		if ( pDirInfo )
		{
#ifdef WIN32
			if ( pDirInfo->StartMonitor( m_hIOCP ) )
#else
			if ( pDirInfo->StartMonitor( m_nEPFD ) )
#endif
			{
				m_dirmap.insert( make_pair( string(szKey), pDirInfo ) );
				return trace.SetRetResult( true );
			}

			delete pDirInfo;
			pDirInfo = NULL;
		}
	}
	else
	{
		// Ŀ¼
		CDirInfo* pDirInfo = it->second;
		pDirInfo->SetPolicy( moniInfo );
	}

	return trace.SetRetResult( false );
}

bool CNofier::UnwatchDirectory( const char * sDirId )
{
	CGDTrace trace( LOG_L6, "CNofier:UnwatchDirectory(%s)", sDirId );
	
	CDirInfo* pDirInfo = NULL;
	map < string, CDirInfo* >::iterator iter = m_dirmap.find( string(sDirId) );
	
	if ( iter != m_dirmap.end() )
	{
		pDirInfo = (CDirInfo*) iter->second;
#ifdef WIN32
		if ( pDirInfo->StopMonitor( m_hIOCP ) )
#else
		if ( pDirInfo->StopMonitor( m_nEPFD ) )
#endif
		{
			m_dirmap.erase( iter );
			delete pDirInfo;
			pDirInfo = NULL;
			return trace.SetRetResult( true );
		}
	}

	return trace.SetRetResult( false );
}

bool CNofier::Uninitialize()
{
	// ɺtrace
	CGDTrace trace( LOG_L6, "CNofier:Uninitialize()" );
	
	// ֪ͨ¼ȡ߳˳
	for ( int i=0; i<m_nThreads; i++ )
	{
#ifdef WIN32
		PostQueuedCompletionStatus( m_hIOCP, 0, 0, NULL );
#else
		//
#endif
	}

	// ֪ͨ¼߳˳
	m_bWorking = false;
#ifdef WIN32
	::WaitForSingleObject( m_hEvent, INFINITE );
	::CloseHandle( m_hEvent );

	CloseHandle( m_hIOCP );
	m_hIOCP = NULL;
#else
	pthread_cond_wait( &m_cond, &m_mutex );
	pthread_cond_destroy( &m_cond );
	pthread_mutex_destroy( &m_mutex );

	close( m_nEPFD );
	m_nEPFD = 0;
#endif

	CDirInfo* pDirInfo = NULL;
	map < string, CDirInfo* >::iterator iter;
	for ( iter = m_dirmap.begin(); iter != m_dirmap.end(); ++iter )  
	{
		pDirInfo = (CDirInfo*) ( (*iter).second );
		if ( pDirInfo )
		{
			delete pDirInfo;
			pDirInfo = NULL;
		}
	}
	m_dirmap.clear();

	return trace.SetRetResult( true );
}

bool CNofier::IsWatched( const char* sDirId )
{
    CGDTrace trace( LOG_L6, "CWatcher:IsWatchingDirectory(%s)", sDirId );
	
    // жǷѾ
    if ( sDirId[0] == '/' || sDirId[1] == ':' )
    {
        return IsWatchingDirectory( sDirId ); // ͨĿ¼жǷ(MA˲֧ظ)
    }
    std::map<std::string, CDirInfo*>::iterator iter = m_dirmap.find( sDirId );
    if ( iter == m_dirmap.end() ) // ûнĿ¼ض
    {
        return trace.SetRetResult( false );
    }
    else
    {
        return trace.SetRetResult( true );
    }
}

bool CNofier::IsWatchingDirectory( const char * szDir )
{
	CGDTrace trace( LOG_L6, "CNofier:IsWatchingDirectory(%s)", szDir );
	
	bool bRet = false;

	CDirInfo* pDirInfo = NULL;
	map < string, CDirInfo* >::iterator iter = m_dirmap.begin();
	while( iter != m_dirmap.end() )
	{
#ifdef WIN32 // (¼CNofierWindowsƽ̨)
		if( ::_strnicmp(szDir, iter->first.c_str(), iter->first.length()) == 0 )
#else
		if( strncasecmp(szDir, iter->first.c_str(), iter->first.length()) == 0 )
#endif
		{
			bRet = true;
			break;
		}
		iter++;
	}

	return trace.SetRetResult(bRet);
}

bool CNofier::IsWatchedByWebsiteId( const char * sWebsiteId )
{
	CGDTrace trace( LOG_L6, "CNofier:IsWatchedByWebsiteId(%s)", sWebsiteId );
	
	// жǷѾ
	bool bRet = false;
	
	CDirInfo * pDirInfo = NULL;
	map < string, CDirInfo* >::iterator iter = m_dirmap.begin();
	while( iter != m_dirmap.end() )
	{
		pDirInfo = (CDirInfo*)iter->second;
		//		if( strcmp( sWebsiteId, pDirInfo->m_szDirId.c_str() ) == 0 )
		if( strcmp( sWebsiteId, pDirInfo->m_szDirId ) == 0 )
		{
			bRet = true;
			break;
		}
		iter++;
	}
	if ( !bRet ) // ûнĿ¼ض
	{
		return trace.SetRetResult( false );
	}
	else
	{
		// ״̬
		//		InforGuard::GDThreadPool::enumThreadStatus status = iter->second->GetStatus();
		//		bool bRet = ( InforGuard::GDThreadPool::TS_STARTING == status || InforGuard::GDThreadPool::TS_STARTED == status );
		return trace.SetRetResult( bRet );
	}
}

bool CNofier::GetMonitorEvent( EVENT_S& evt )
{
	// ȡ¼
	CEventInfo ei;
	if ( ! m_queue.pop_front( ei ) )
	{
		return false;
	}
	// 
	evt.event_type = ei.m_nAction;
	evt.attr_flag = ei.m_nAttribute;
#ifdef WIN32
	evt.evt_time = ei.m_time;
#else
#endif
	strcpy( evt.o_filepath, ei.m_szFileName );
	strcpy( evt.n_filepath, ei.m_newFileName );
	//Ŀ¼ID by wu_bin
	strcpy( evt.sDirId, ei.m_szDirId );
	
	//ɺtrace
	string sMoniInfo = evt.toString();
	
	CGDTrace trace( LOG_L6, "CNofier:GetMonitorEvent(%s)", sMoniInfo.c_str() );
#ifdef WIN32
#ifndef NDEBUG
	if ( evt.event_type == MONITOR_EVENT_WATCHER_CREATE )
	{
		::InterlockedIncrement( &GnuAddCount[5] );
	}
	else if ( evt.event_type == MONITOR_EVENT_WATCHER_DELETE )
	{
		::InterlockedIncrement( &GnuRemoveCount[5] );
	}
	else if ( evt.event_type == MONITOR_EVENT_WATCHER_CHANGE )
	{
		::InterlockedIncrement( &GnuChangeCount[5] );
	}
	else if ( evt.event_type == MONITOR_EVENT_WATCHER_RENAME )
	{
		::InterlockedIncrement( &GnuRenameCount[5] );
	}
#endif
#endif
	return trace.SetRetResult( true );
}

void GetAnotherKey( string& sKey, const char * sOldFileName, const char * sNewFileName )
{
	int nFirstPos = sKey.find_last_of("\\");
	int nNameLen = sKey.length() - nFirstPos;
	int nNamePos = strlen(sOldFileName) - nNameLen;
	sKey.replace( nFirstPos, sKey.length() - 1, sNewFileName + nNamePos );
}

// Ψһ߳ʵ¼ϲעⱣmaplistԪһԣ
// ¼˵
// ! ʾϵͳϱ¼
// $ ʾϱ¼
// + ʾϲ¼
// - ʾҺԵ¼
// . ʾֱӺԵ¼(Ŀ¼޸)
THREADPROC CNofier::FilterRoutine( void* lpParam )
{
	// watch־
	CThreadData::SetCurrentThreadLogFile( "watch", true ); // ߳()־

	// trace־
	CGDTrace trace( LOG_L6, "CNofier:FilterRoutine()" );
	
	CNofier* pNofier = (CNofier*)lpParam;
	string sNewKey;
	std::pair< std::map< string, CEventInfo* >::iterator, bool > ret;

#ifdef WIN32
	if ( ! SetEvent( pNofier->m_hEvent ) ) // set thread start event :ʱm_hEventΪ0
	{
		ErrCode dwErr = GetLastError();
		assert( 0 ); // ͨ
		return -1; // 
	}
#else
	pthread_cond_signal( &pNofier->m_cond );
#endif

	// ˣǴӶж¼ɢбȴ¼ϲ˵ͬʱҪҪȴϲ¼¼

	map < string, CEventInfo* >::iterator mapiter;
	pNofier->m_bWorking = true;
	while ( pNofier->m_bWorking )
	{
		DWORD dwNow = GetTickCount(); // ȡʱ䣬ںϲ¼ĩβ¼
		if ( ! pNofier->m_deque.empty() )
		{
			// 
#ifdef WIN32
			EnterCriticalSection( &pNofier->m_cs );
#else
			pthread_mutex_lock( &pNofier->m_cs );
#endif
			// ȡ
			CEventInfo* pei = (CEventInfo*)pNofier->m_deque.front();
			pNofier->m_deque.pop_front();
			// 
#ifdef WIN32
			LeaveCriticalSection( &pNofier->m_cs );
#else
			pthread_mutex_unlock( &pNofier->m_cs );
#endif
			assert( pei->m_pFileExclude );
			dwNow = pei->m_nTickCount; // ȡʱ䣬ںϲ¼ʱԱ⴦ӳٲƫ
			
			// ԭʼ¼
			CGDLog::WriteNofierEvent( "![%x][%s][%s]", pei->m_nAction, pei->m_szFileName, pei->m_newFileName );

			// ж¼ͣԴ
			do
			{
				CEventInfo* prv = NULL;
				CEventInfo* prv2 = NULL;
				// ǰ¼
				mapiter = pNofier->m_evtmap.find( pei->m_szKey );
				if ( pNofier->m_evtmap.end() == mapiter )
				{
					if ( FILE_ACTION_RENAMED_OLD_NAME == pei->m_nAction )
					{
						sNewKey = pei->m_szKey;
						GetAnotherKey( sNewKey, pei->m_szFileName, pei->m_newFileName );
						mapiter = pNofier->m_evtmap.find( sNewKey );
						if ( pNofier->m_evtmap.end() != mapiter )
						{
							// ǰ¼
							prv2 = (CEventInfo*)(mapiter->second);
							if ( !prv2 )
							{
								CGDLog::WriteNofierEvent("ERROR: NULL Pointer!!!");
								exit(-1);
							}
						}
						else
						{
							CEventInfo * nevt = new CEventInfo;
							nevt->m_nAction = FILE_ACTION_RENAMED_NEW_NAME;
							nevt->m_nAttribute = pei->m_nAttribute;
							strcpy( nevt->m_szDirId, pei->m_szDirId );
							nevt->m_nTickCount = pei->m_nTickCount;
							nevt->m_pFileExclude = pei->m_pFileExclude;
							strcpy( nevt->m_szFileName, pei->m_szFileName );
							strcpy( nevt->m_newFileName, pei->m_newFileName );					
							strcpy( nevt->m_szKey, sNewKey.c_str() );
							// ˶Уȴܵĺϲ
							ret = pNofier->m_evtmap.insert( make_pair( string(nevt->m_szKey), nevt ) );
							if( ret.second )
							{
								pNofier->m_filter.push_back( nevt ); // ļ
							}
							else
							{
								CGDLog::WriteErrLog( "2102:ϲ¼дʧ!" );
								assert(0);
							}
						}
					}
				}
				else
				{
					// ǰ¼
					prv = (CEventInfo*)(mapiter->second);
					if ( !prv )
					{
						CGDLog::WriteNofierEvent("ERROR: NULL Pointer!!!");
						exit(-1);
					}
				}
			
				// жļĿ¼
				try
				{
					if ( FILE_ACTION_RENAMED_OLD_NAME == pei->m_nAction )
					{
						if ( CFileOpt::IsDirectory( pei->m_newFileName ) )
						{
							pei->m_nAttribute = FILE_ATTRIBUTE_DIRECTORY;
						}
						else
						{
							pei->m_nAttribute = FILE_ATTRIBUTE_ARCHIVE;
						}
					}
					else if ( FILE_ACTION_REMOVED != pei->m_nAction )
					{
						if ( CFileOpt::IsDirectory( pei->m_szFileName ) )
						{
							pei->m_nAttribute = FILE_ATTRIBUTE_DIRECTORY;
						}
						else
						{
							pei->m_nAttribute = FILE_ATTRIBUTE_ARCHIVE;
						}
					}

					if ( FILE_ATTRIBUTE_DIRECTORY == pei->m_nAttribute )
					{					
						if ( FILE_ACTION_ADDED == pei->m_nAction )
						{
							// Ŀ¼½¼ϲֱϱ½Ŀ¼
							pei->m_nAction = MONITOR_EVENT_WATCHER_CREATE;
							// Ӧò
							try
							{
								if ( ! pei->m_pFileExclude->IsExcludeEvent( (eventType)pei->m_nAction, pei->m_szFileName ) )
								{
									pNofier->m_queue.push_back( *pei );
								}
							}
							catch ( CExceptionError )
							{
								//
							}
							catch ( ... )
							{
								//
							}
							
							delete pei; // ɾԭʼ
							pei = NULL;

							// ǰ¼ɾ֮
							if ( prv )
							{
								CGDLog::WriteNofierEvent( "-[%x][%s][%s]", prv->m_nAction, prv->m_szFileName, prv->m_newFileName );
								pNofier->m_evtmap.erase( prv->m_szKey );
								pNofier->m_filter.remove( prv );
								delete prv;
								prv = NULL;
							}
							else if ( prv2 )
							{
								CGDLog::WriteNofierEvent( "-[%x][%s][%s]", prv2->m_nAction, prv2->m_szFileName, prv2->m_newFileName );
								pNofier->m_evtmap.erase( prv2->m_szKey );
								pNofier->m_filter.remove( prv2 );
								delete prv2;
								prv = NULL;
							}
							break;
						}
						else if( FILE_ACTION_MODIFIED == pei->m_nAction )
						{
							CGDLog::WriteNofierEvent( ".[%x][%s][%s]", pei->m_nAction, pei->m_szFileName, pei->m_newFileName );
							delete pei;
							pei = NULL;
							break;
						}
					}
				}
				catch ( CExceptionError & ex )
				{
					ex.AddErrText("CNofier:FilterRoutine()");
					CGDLog::WriteErrLog( "¼жʧ\n%s", ex.GetErrText().c_str() );				
				}
				catch ( ... )
				{
					CGDLog::WriteErrLog( "¼жʧ(...)" );
				}
				
				if ( NULL == prv && NULL == prv2 )
				{
					// ¼-˶Уȴܵĺϲ
					ret = pNofier->m_evtmap.insert( make_pair( string( pei->m_szKey ), pei ) );
					if( ret.second )
					{
						pNofier->m_filter.push_back( pei ); // ¼
					}
					else
					{
						CGDLog::WriteErrLog( "2202:ϲ¼дʧ!" );
						mapiter = pNofier->m_evtmap.find( pei->m_szKey );
						if ( pNofier->m_evtmap.end() != mapiter )
						{
							assert(0);
						}
					}
					break;
				}				
				switch ( pei->m_nAction )
				{
				case FILE_ACTION_ADDED:
					switch ( prv->m_nAction )
					{
					case FILE_ACTION_ADDED:
						// ½+½=߼ңǰ¼ǰ¼ͷŵ¼
						CGDLog::WriteNofierEvent( "-[%x][%s][%s]", prv->m_nAction, prv->m_szFileName, prv->m_newFileName );
						prv->m_nTickCount = pei->m_nTickCount;
						delete pei;
						pei = NULL;
						break;
					case FILE_ACTION_REMOVED:
						// ɾ+½=ϲΪ޸ģǰ¼ͷŵǰ¼
						CGDLog::WriteNofierEvent( "+[%x][%s][%s]+[%x][%s][%s]=[%x][%s]", prv->m_nAction, prv->m_szFileName, prv->m_newFileName, pei->m_nAction, pei->m_szFileName, pei->m_newFileName, FILE_ACTION_MODIFIED, pei->m_szFileName );
						prv->m_nAction = FILE_ACTION_MODIFIED;
						prv->m_nTickCount = pei->m_nTickCount;
						delete pei;
						pei = NULL;
						break;
					case FILE_ACTION_MODIFIED:
						// ޸+½=߼ңǰ¼ǰ¼ͷŵ¼
						CGDLog::WriteNofierEvent( "-[%x][%s][%s]", prv->m_nAction, prv->m_szFileName, prv->m_newFileName );
						prv->m_nAction = pei->m_nAction;
						prv->m_nTickCount = pei->m_nTickCount;
						delete pei;
						pei = NULL;
						break;
					case FILE_ACTION_RENAMED_OLD_NAME:	//test1			
						// old->new+½old=Ϊ½new+޸oldǰ¼õǰ¼ѭ
						CGDLog::WriteNofierEvent( "+[%x][%s][%s]+[%x][%s][%s]=[%x][%s]+[%x][%s]", prv->m_nAction, prv->m_szFileName, prv->m_newFileName, pei->m_nAction, pei->m_szFileName, pei->m_newFileName, FILE_ACTION_MODIFIED, pei->m_szFileName, FILE_ACTION_ADDED, prv->m_newFileName );						
						prv->m_nAction = FILE_ACTION_MODIFIED;
						{
							sNewKey = pei->m_szKey;
							GetAnotherKey( sNewKey, pei->m_szFileName, prv->m_newFileName );
							strcpy( pei->m_szKey, sNewKey.c_str() );
							strcpy( pei->m_szFileName, prv->m_newFileName );
						}
						prv->m_newFileName[0] = '\0';
						pei->m_nAction = FILE_ACTION_ADDED;
						continue;
					case FILE_ACTION_RENAMED_NEW_NAME:
						// old->new+½new=߼ңǰ¼ͷǰ¼ѭ
						CGDLog::WriteNofierEvent( "-[%x][%s][%s]", prv->m_nAction, prv->m_szFileName, prv->m_newFileName );
						pNofier->m_evtmap.erase( prv->m_szKey );
						pNofier->m_filter.remove( prv );
						delete prv;
						prv = NULL;
						continue;
					default: // ǰ¼ʲôǣͷǰ¼ѭ
						assert(0);
						pNofier->m_evtmap.erase( prv->m_szKey );
						pNofier->m_filter.remove( prv );
						delete prv;
						prv = NULL;								
						continue;
					}				
					break;
				case FILE_ACTION_REMOVED:
					switch ( prv->m_nAction )
					{
					case FILE_ACTION_ADDED:
						// ½+ɾ=ͷǰ¼ͷŵǰ¼
						CGDLog::WriteNofierEvent( "+[%x][%s][%s]+[%x][%s][%s]=0", prv->m_nAction, prv->m_szFileName, prv->m_newFileName, pei->m_nAction, pei->m_szFileName, pei->m_newFileName  );
						pNofier->m_evtmap.erase( prv->m_szKey );
						pNofier->m_filter.remove( prv );
						delete prv;
						prv = NULL;
						delete pei ;
						pei = NULL ;
						break;
					case FILE_ACTION_REMOVED:
						// ɾ+ɾ=߼ңǰ¼ǰ¼ͷŵǰ¼
						CGDLog::WriteNofierEvent( "-[%x][%s][%s]", prv->m_nAction, prv->m_szFileName, prv->m_newFileName );
						prv->m_nTickCount = pei->m_nTickCount;
						delete pei;
						pei = NULL ;
						break ;
					case FILE_ACTION_MODIFIED:
						// ޸+ɾ=ģǰ¼ǰ¼ͷŵǰ¼
						CGDLog::WriteNofierEvent( "-[%x][%s][%s]", prv->m_nAction, prv->m_szFileName, prv->m_newFileName );
						prv->m_nTickCount = pei->m_nTickCount;
						delete pei;
						pei = NULL ;
						break ;
					case FILE_ACTION_RENAMED_OLD_NAME://test2
						// old->new+ɾold(ϲ)=߼ңǰ¼ͷǰ¼
						CGDLog::WriteNofierEvent( "[%x][%s][%s]+[%x][%s][%s]=[%x][%s]", prv->m_nAction, prv->m_szFileName, prv->m_newFileName, pei->m_nAction, pei->m_szFileName, pei->m_newFileName, FILE_ACTION_REMOVED, pei->m_szFileName );
						pNofier->m_evtmap.erase( prv->m_szKey );
						pNofier->m_filter.remove( prv );
						delete prv;
						prv = NULL;
						break;
					case FILE_ACTION_RENAMED_NEW_NAME://test3
						// old->new+ɾnew=ϲΪɾoldǰ¼ͷŵǰ¼
						CGDLog::WriteNofierEvent( "+[%x][%s][%s]+[%x][%s][%s]=[%x][%s]", prv->m_nAction, prv->m_szFileName, prv->m_newFileName, pei->m_nAction, pei->m_szFileName, pei->m_newFileName, FILE_ACTION_REMOVED, prv->m_szFileName );
						pNofier->m_evtmap.erase( prv->m_szKey );
						pNofier->m_filter.remove( prv );
						sNewKey = pei->m_szKey;
						GetAnotherKey( sNewKey,pei->m_szFileName, prv->m_szFileName );
						strcpy( pei->m_szKey, sNewKey.c_str() );
						strcpy( pei->m_szFileName, prv->m_szFileName );
						delete prv;
						prv = NULL;
						continue;
					default: // ǰ¼ʲôǣͷǰ¼ѭ
						assert(0);
						pNofier->m_evtmap.erase( prv->m_szKey );
						pNofier->m_filter.remove( prv );
						delete prv;
						prv = NULL;								
						continue;
					}
					break;
				case FILE_ACTION_MODIFIED:
					switch ( prv->m_nAction )
					{
					case FILE_ACTION_ADDED:
						// ½+޸=½ǰ¼ͷŵǰ¼
						CGDLog::WriteNofierEvent( "+[%x][%s][%s]+[%x][%s][%s]=1", prv->m_nAction, prv->m_szFileName, prv->m_newFileName, pei->m_nAction, pei->m_szFileName, pei->m_newFileName  );
						prv->m_nTickCount = pei->m_nTickCount;
						delete pei;
						pei = NULL;
						break;
					case FILE_ACTION_REMOVED:
						// ɾ+޸=߼ңǰ¼ǰ¼ͷŵǰ¼
						CGDLog::WriteNofierEvent( "-[%x][%s][%s]", prv->m_nAction, prv->m_szFileName, prv->m_newFileName );
						prv->m_nTickCount = pei->m_nTickCount;
						delete pei;
						pei = NULL;
						break;
					case FILE_ACTION_MODIFIED:
						// ޸+޸=޸ģԵǰ¼ǰ¼ͷŵǰ¼
						CGDLog::WriteNofierEvent( "+[%x][%s][%s]+[%x][%s][%s]=3", prv->m_nAction, prv->m_szFileName, prv->m_newFileName, pei->m_nAction, pei->m_szFileName, pei->m_newFileName  );
						prv->m_nTickCount = pei->m_nTickCount;
						delete pei;
						pei = NULL;
						break;
					case FILE_ACTION_RENAMED_OLD_NAME:
						// old->new+޸old=߼ңǰ¼ǰ¼ͷŵǰ¼
						CGDLog::WriteNofierEvent( "-[%x][%s][%s]", prv->m_nAction, prv->m_szFileName, prv->m_newFileName );
						prv->m_nTickCount = pei->m_nTickCount;
						delete pei;
						pei = NULL;
						break;
					case FILE_ACTION_RENAMED_NEW_NAME://test4
						// old->new+޸new=Ϊɾold+½newǰ¼õǰ¼
						CGDLog::WriteNofierEvent( "+[%x][%s][%s]+[%x][%s][%s]=[%x][%s]+[%x][%s]", prv->m_nAction, prv->m_szFileName, prv->m_newFileName, pei->m_nAction, pei->m_szFileName, pei->m_newFileName, FILE_ACTION_REMOVED, prv->m_szFileName, FILE_ACTION_ADDED, pei->m_szFileName );
						//add by dai_hyan
						prv->m_nAction = FILE_ACTION_ADDED;
						{
							sNewKey = pei->m_szKey;
							GetAnotherKey( sNewKey, pei->m_szFileName, prv->m_szFileName );
							strcpy( pei->m_szKey, sNewKey.c_str() );
						}
						pei->m_nAction = FILE_ACTION_REMOVED;
						strcpy(pei->m_szFileName, prv->m_szFileName);
						strcpy(prv->m_szFileName, prv->m_newFileName);
						prv->m_newFileName[0] = '\0';
						continue;
					default: // ǰ¼ʲôǣͷǰ¼ѭ
						assert(0);
						pNofier->m_evtmap.erase( prv->m_szKey );
						pNofier->m_filter.remove( prv );
						delete prv;
						prv = NULL;								
						continue;
					}
					break;	
				case FILE_ACTION_RENAMED_OLD_NAME:
					if( prv )//ֻоļǰ¼
					{
						switch( prv->m_nAction )
						{
						case FILE_ACTION_ADDED://ǰ½old test5
							// ½old+old->new=ϲΪ½newͷǰ¼õǰ¼ѭɾnewϲ
							CGDLog::WriteNofierEvent( "+[%x][%s][%s]+[%x][%s][%s]=[%x][%s]", prv->m_nAction, prv->m_szFileName, prv->m_newFileName, pei->m_nAction, pei->m_szFileName, pei->m_newFileName, FILE_ACTION_ADDED, pei->m_newFileName );
							pNofier->m_evtmap.erase( prv->m_szKey );
							pNofier->m_filter.remove( prv );
							delete prv;
							prv = NULL;
							pei->m_nAction = FILE_ACTION_ADDED;
							sNewKey = pei->m_szKey;
							GetAnotherKey( sNewKey, pei->m_szFileName, pei->m_newFileName );
							strcpy( pei->m_szKey, sNewKey.c_str() );
							strcpy( pei->m_szFileName, pei->m_newFileName );
							pei->m_newFileName[0] = '\0';							
							continue;
						case FILE_ACTION_REMOVED://ǰɾold
							// ɾold+old->new=߼ңǰ¼ǰ¼ͷŵǰ¼
							CGDLog::WriteNofierEvent( "-[%x][%s][%s]", prv->m_nAction, prv->m_szFileName, prv->m_newFileName );
							prv->m_nTickCount = pei->m_nTickCount;
							delete pei;
							pei = NULL;
							break;
						case FILE_ACTION_MODIFIED://ǰ޸old test6
							// ޸old+old->new=ɾold+½newǰ¼õǰ¼ѭɾnewϲ
							CGDLog::WriteNofierEvent( "+[%x][%s][%s]+[%x][%s][%s]=[%x][%s]+[%x][%s]", prv->m_nAction, prv->m_szFileName, prv->m_newFileName, pei->m_nAction, pei->m_szFileName, pei->m_newFileName, FILE_ACTION_REMOVED, pei->m_szFileName, FILE_ACTION_ADDED, pei->m_newFileName );
							prv->m_nAction = FILE_ACTION_REMOVED;
							prv->m_nTickCount = pei->m_nTickCount;
							pei->m_nAction = FILE_ACTION_ADDED;
							//add by dai_hyan 2009-8-24
							sNewKey = pei->m_szKey;
							GetAnotherKey( sNewKey, pei->m_szFileName, pei->m_newFileName );
							strcpy( pei->m_szKey, sNewKey.c_str() );
							strcpy( pei->m_szFileName, pei->m_newFileName );
							pei->m_newFileName[0] = '\0';				
							continue;
						case FILE_ACTION_RENAMED_OLD_NAME://ǰļ
							// old->new1+old->new2=߼ңǰ¼ͷǰ¼ѭ
							CGDLog::WriteNofierEvent( "-[%x][%s][%s]", prv->m_nAction, prv->m_szFileName, prv->m_newFileName );
							pNofier->m_evtmap.erase( prv->m_szKey );
							pNofier->m_filter.remove( prv );
							delete prv;
							prv = NULL;
							continue;
						case FILE_ACTION_RENAMED_NEW_NAME://ǰļ
							// old->new1+new1->new=ϲΪold->newͷǰ¼õǰ¼ѭ
							CGDLog::WriteNofierEvent( "+[%x][%s][%s]+[%x][%s][%s]=[%x][%s][%s]", prv->m_nAction, prv->m_szFileName, prv->m_newFileName, pei->m_nAction, pei->m_szFileName, pei->m_newFileName, FILE_ACTION_RENAMED_OLD_NAME, prv->m_szFileName, pei->m_newFileName );
							strcpy(pei->m_szFileName, prv->m_szFileName);
							pNofier->m_evtmap.erase( prv->m_szKey );
							pNofier->m_filter.remove( prv );
							delete prv;
							prv = NULL;								
							continue;
						default: // ǰ¼ʲôǣͷǰ¼ѭ
							assert(0);
							pNofier->m_evtmap.erase( prv->m_szKey );
							pNofier->m_filter.remove( prv );
							delete prv;
							prv = NULL;								
							continue;
						}							
					}
					else if( prv2 )//ֻļǰ¼                                                                                          
					{
						switch( prv2->m_nAction )
						{
						case FILE_ACTION_ADDED://ǰ½
							// ½new+old->new=߼ңǰ¼ͷǰ¼ѭ
							CGDLog::WriteNofierEvent( "-[%x][%s][%s]", prv2->m_nAction, prv2->m_szFileName, prv2->m_newFileName );
							pNofier->m_evtmap.erase( prv2->m_szKey );
							pNofier->m_filter.remove( prv2 );
							delete prv2;
							prv2 = NULL;
							continue;
						case FILE_ACTION_REMOVED://ǰɾ test7
							// ɾnew+old->new=Ϊɾold+޸newǰ¼ɾoldõǰ¼޸newѭ
							CGDLog::WriteNofierEvent( "+[%x][%s][%s]+[%x][%s][%s]=[%x][%s][%x][%s]", prv2->m_nAction, prv2->m_szFileName, prv2->m_newFileName, pei->m_nAction, pei->m_szFileName, pei->m_newFileName, FILE_ACTION_REMOVED, pei->m_szFileName, FILE_ACTION_MODIFIED, pei->m_newFileName );
							prv2->m_nAction = FILE_ACTION_MODIFIED;
							prv2->m_nTickCount = pei->m_nTickCount;
							pei->m_nAction = FILE_ACTION_REMOVED;
							pei->m_newFileName[0] = '\0';
							continue;
						case FILE_ACTION_MODIFIED://ǰ޸
							// ޸new+old->new=߼ңǰ¼ͷǰ¼ѭ
							CGDLog::WriteNofierEvent( "-[%x][%s][%s]", prv2->m_nAction, prv2->m_szFileName, prv2->m_newFileName );
							pNofier->m_evtmap.erase( prv2->m_szKey );
							pNofier->m_filter.remove( prv2 );
							delete prv2;
							prv2 = NULL;
							continue;
						case FILE_ACTION_RENAMED_OLD_NAME://ǰļ test8
							// old->new1+old2->old=Ϊ½new1+޸oldǰ¼õǰ¼ѭ
							CGDLog::WriteNofierEvent( "+[%x][%s][%s]+[%x][%s][%s]=[%x][%s]+[%x][%s]", prv2->m_nAction, prv2->m_szFileName, prv2->m_newFileName, pei->m_nAction, pei->m_szFileName, pei->m_newFileName, FILE_ACTION_ADDED, prv2->m_newFileName, FILE_ACTION_MODIFIED, pei->m_newFileName );
							prv2->m_nAction = FILE_ACTION_MODIFIED;
							prv2->m_nTickCount = pei->m_nTickCount;
							pei->m_nAction = FILE_ACTION_ADDED;
							//add by dai_hyan 2009-8-24
							sNewKey = prv2->m_szKey;
							GetAnotherKey( sNewKey, prv2->m_szFileName, prv2->m_newFileName );
							strcpy( pei->m_szKey, sNewKey.c_str() );
							strcpy(pei->m_szFileName, prv2->m_newFileName);
							prv2->m_newFileName[0] = '\0';
							pei->m_newFileName[0] = '\0';
							continue;
						case FILE_ACTION_RENAMED_NEW_NAME://ǰļ
							// old1->new+old2->new=߼ңǰ¼ͷǰ¼ѭ
							CGDLog::WriteNofierEvent( "-[%x][%s][%s]", prv2->m_nAction, prv2->m_szFileName, prv2->m_newFileName );
							pNofier->m_evtmap.erase( prv2->m_szKey );
							pNofier->m_filter.remove( prv2 );
							delete prv2;
							prv2 = NULL;
							continue;
						default: // ǰ¼ʲôǣͷǰ¼ѭ
							assert(0);
							pNofier->m_evtmap.erase( prv2->m_szKey );
							pNofier->m_filter.remove( prv2 );
							delete prv2;
							prv2 = NULL;								
							continue;
						}							
					}
					else // ¾ļ¼
					{
						// ¼ӵǰ¼ǰͳһʵ̲ᵽ
					}
					break;
				default:
					// δ֪¼
					break;
				}//switch
			} while(pei);
		}//if
		else
		{
			// Ϣ
			GDMINSLEEP();
		}
		
		static DWORD dwLastTime = GetTickCount();
		DWORD dwCurrTime = GetTickCount();
		//GDPrint( "GetTickCount=[%10d]\n", dwCurrTime );
		if ( dwLastTime + 300 <= dwCurrTime ) // 300һ¼
		{
			dwLastTime = dwCurrTime;
			
			// ˶
			CEventInfo* pei = NULL;
			while ( ! pNofier->m_filter.empty() )
			{
				pei = pNofier->m_filter.front();
				// жʱ
				if ( pei->m_nTickCount + pNofier->m_nMilliSeconds > dwNow )
				{
					// ʱ䲻
					break;
				}
				// ж½дǷ
#ifdef WIN32
				if ( FILE_ACTION_ADDED == pei->m_nAction || FILE_ACTION_MODIFIED == pei->m_nAction )
				{
					// ռļ
					//HANDLE hHandle = CreateFile( pei->m_szFileName,GENERIC_WRITE,0,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
					HANDLE hHandle = CreateFile( pei->m_szFileName, GENERIC_WRITE, FILE_SHARE_READ, 0, OPEN_EXISTING, 0, 0 );
					ErrCode dwErr = GetLastError();
					if ( INVALID_HANDLE_VALUE != hHandle )
					{
						CloseHandle( hHandle );
					}
					if ( 32 == dwErr ) // "ʹøļ޷"
					{
						// ½ļռáд
						pNofier->m_filter.pop_front();
						pei->m_nTickCount = GetTickCount();
						pNofier->m_filter.push_back( pei ); // ¼ĩβ
						continue;
					}
				}
#else
#endif
				// ʱˣ¼
				pNofier->m_filter.pop_front();
				pNofier->m_evtmap.erase( pei->m_szKey ); // ˴쳣by wang_jin at 2009-05-23
				// ж¼½Ŀ¼½ļɾĿ¼/ļ޸Ŀ¼޸ļĿ¼/ļ
				switch ( pei->m_nAction )
				{
				case FILE_ACTION_ADDED:// 0x00000001
					pei->m_nAction = MONITOR_EVENT_WATCHER_CREATE;
					break;
				case FILE_ACTION_REMOVED://  0x00000002
					pei->m_nAction = MONITOR_EVENT_WATCHER_DELETE;
					break;
				case FILE_ACTION_MODIFIED:// 0x00000003
					pei->m_nAction = MONITOR_EVENT_WATCHER_CHANGE;
					break;
				case FILE_ACTION_RENAMED_OLD_NAME://  0x00000004
					pei->m_nAction = MONITOR_EVENT_WATCHER_RENAME;
					break;
				case FILE_ACTION_RENAMED_NEW_NAME://  0x00000005
					delete pei;
					pei = NULL;
					break;
				default:
					CGDTrace::GDTrace( LOG_INF, "δ֪¼[%d]", pei->m_nAction );
					assert(0);
					break;
				}
				if ( pei )
				{
					// ־				
					CGDLog::WriteNofierEvent( "$[%x][%s][%s]", pei->m_nAction, pei->m_szFileName, pei->m_newFileName );
					// Ӧò
					try
					{
						if ( ! pei->m_pFileExclude->IsExcludeEvent( (eventType)pei->m_nAction, pei->m_szFileName ) )
						{
							pNofier->m_queue.push_back( *pei );
#ifdef WIN32
#ifndef NDEBUG
							if ( pei->m_nAction == MONITOR_EVENT_WATCHER_CREATE )
							{
								::InterlockedIncrement( &GnuAddCount[4] );
							}
							else if ( pei->m_nAction == MONITOR_EVENT_WATCHER_DELETE )
							{
								::InterlockedIncrement( &GnuRemoveCount[4] );
							}
							else if ( pei->m_nAction == MONITOR_EVENT_WATCHER_CHANGE )
							{
								::InterlockedIncrement( &GnuChangeCount[4] );
							}
							else if ( pei->m_nAction == MONITOR_EVENT_WATCHER_RENAME )
							{
								::InterlockedIncrement( &GnuRenameCount[4] );
							}
#endif
#endif
						}
					}
					catch ( CExceptionError )
					{
						//
					}
					catch ( ... )
					{
						//
					}
					// ɾ¼
					delete pei;
					pei = NULL;
				}
			}//for
		}//if
	}//while

#ifdef WIN32
	SetEvent( pNofier->m_hEvent );
#else
	pthread_cond_signal( &pNofier->m_cond );
#endif

	return 0;
}


// ߳()˳(мͣزӰ)
THREADPROC CNofier::CompletionRoutine( void* lpParam )
{
	// watch־
	CThreadData::SetCurrentThreadLogFile( "watch", true ); // ߳()־
	
	CGDTrace trace( LOG_L6, "CNofier:CompletionRoutine()" );

	CNofier* pNofier = (CNofier*)lpParam;

#ifdef WIN32
	DWORD dwBytes = 0;
	LPOVERLAPPED pOL = NULL;
#else
	int dwBytes = 0;
	int maxevents = 256;
	int evtlen = maxevents * sizeof(struct epoll_event);
	epoll_event * events = (epoll_event*)new char[evtlen];
	int buflen = 1024*1024;
	char * buffer = new char[buflen];
	int offset = 0;
	int EVENT_SIZE = sizeof(struct inotify_event);
#endif
	CDirInfo* pDirInfo = NULL;

	CGDTimer watchTimer(30);  //üʱ
	CGDLog::WriteThreadState("watch started"); //wu_binд߳״̬
	do
	{
		if (watchTimer.IsTimeout())
		{
			CGDLog::WriteThreadState("watch loop running(%d)...", watchTimer.GetMaxRunCount()); //wu_binд߳״̬
		}
#ifdef NDEBUG
#	define USE_TRY
#endif

#ifdef USE_TRY
		try
#endif
		{
#ifdef WIN32
			BOOL bRet = GetQueuedCompletionStatus( pNofier->m_hIOCP, 
				&dwBytes, (unsigned long*)&pDirInfo, &pOL, INFINITE );
			if ( !bRet )
			{
				// The io completion request failed...
				// probably because the handle to the directory that
				// was used in a call to ReadDirectoryChangesW has been closed.
				//
				// Calling pDirInfo->CloseDirectoryHandle() will cause GetQueuedCompletionStatus() to return false.
				ErrCode dwErr = GetLastError();
				//assert( 0 ); // ˳ʱᵽ֪Ϊʲô?
				continue;
			}
			if ( pDirInfo ) // ?? This is NULL if "PostQueuedCompletionStatus(m_hIOCP,0,0,NULL);" be called.
			{
				switch ( pDirInfo->m_status )
				{
				case CNofier::CDirInfo::STATUS_STARTING:
					pDirInfo->m_status = CNofier::CDirInfo::STATUS_STARTED;
					break;
				case CNofier::CDirInfo::STATUS_STOPPING:
					pDirInfo->m_status = CNofier::CDirInfo::STATUS_STOPED;
					break;
				case CNofier::CDirInfo::STATUS_CLOSING:
					pDirInfo->m_status = CNofier::CDirInfo::STATUS_CLOSED;
					break;
				case CNofier::CDirInfo::STATUS_NORMAL:
					// ¼
					if ( pDirInfo->m_evtBuf.offset )
					{
						dwBytes += pDirInfo->m_evtBuf.offset;
						pDirInfo->m_evtBuf.offset = 0;
					}
					ParseEvent( pNofier, pDirInfo, dwBytes );
					pDirInfo->ReadDirectoryChanges(); // 첽
					break;
				default:
					cout << "δ֪״̬" << pDirInfo->m_status << endl;
					break;
				}//switch(status)
			}//if(pDirInfo)
#else
			int nfds = epoll_wait( pNofier->m_nEPFD, events, buflen, -1 );
			if ( nfds <= 0 )
			{
				GDPrint( "epoll_wait() ret=[%d]\n", nfds );
				CGDLog::WriteErrLog( "epoll_wait() ret=[%d]\n", nfds );
			}
			for ( int i=0; i<nfds; i++ )
			{
				CDirInfo * pDirInfo = (CDirInfo*)events[i].data.ptr;
				if ( ! pDirInfo )
				{
					assert(0);
					continue;
				}
				dwBytes = pDirInfo->ReadDirectoryChanges(); // ͬǰ
				if ( dwBytes > 0 )
				{
					if ( pDirInfo->m_evtBuf.offset )
					{
						dwBytes += pDirInfo->m_evtBuf.offset;
						pDirInfo->m_evtBuf.offset = 0;
					}
					ParseEvent( pNofier, pDirInfo, dwBytes );
				}
			}
#endif
		}
#ifdef USE_TRY
		catch ( ... )
		{
			CGDLog::WriteErrLog( "1082:CNofier:CompletionRoutine() catch ..." );
		}
#endif
	//} while ( pDirInfo  );
	} while ( true  );

	CGDLog::WriteThreadState("watch exited");//wu_binд߳״̬
#ifdef WIN32
	return trace.SetRetValue( 0 );
#endif
}
/*		
#ifndef NDEBUG
{
	static FILE * fp1 = fopen("c:\\tmp\\evt_1.log","a");
	static int nCount1 = 0;
	if ( fp1 )
	{
		fprintf(fp1, "%d: [%d] Buffer Len:[%d]\n", ++nCount1, GetTickCount(), dwBytes );
		fflush(fp1);
	}
}
#endif
*/
//	// 
//	if ( InterlockedCompareExchange( &pDirInfo->m_nParsing, 1, 0 ) == 0 )
//	{
		// 
//		InterlockedExchange( &pDirInfo->m_nParsing, 0 );
//	}
