//
// ThreadPool.cpp
//

#pragma warning( disable: 4786 )

#include "GDThreadPool/ThreadPool.h"
#include "Common/DateTime.h"


namespace InforGuard   {
namespace GDThreadPool {

long CThreadPool::CMaintain::Run( void* param )
{
	CThreadPool* pThreadPool = (CThreadPool*)param;
	if ( pThreadPool == NULL )
	{
		return 0;
	}

	pThreadPool->Maintain();

	return 0;
}


CThreadPool::CThreadPool( CThread* pThread, const char* sName )
{
	m_pThread = pThread;
	m_poolName = sName;
	m_status = TP_NULL;
	m_threadTotalCount = 0;
}

CThreadPool::CThreadPool( CThread* pThread )
{
	m_pThread = pThread;
	m_poolName = "NoName";
	m_status = TP_NULL;
	m_threadTotalCount = 0;
}

CThreadPool::~CThreadPool()
{
	Stop();
	Uninit();
}


long CThreadPool::Init( void* param, const ThreadPoolInfo* info )
{
	m_param = param;

	if ( NULL != info )
	{
		m_threadPoolInfo = *info;
	}

	m_pMaintain = new CMaintain( "Maintain" );
	return 0;
}

long CThreadPool::Uninit()
{
	m_param = NULL;

	if ( NULL != m_pMaintain )
	{
		delete m_pMaintain;
		m_pMaintain = NULL;
	}
	return 0;
}


long CThreadPool::Start()
{
	if ( TP_NULL != m_status && TP_STOPED != m_status )
	{
		return 0;
	}
	m_status = TP_STARTING;
	m_pMaintain->Init( this );
	return m_pMaintain->Start();
}

bool CThreadPool::Stop()
{
	if ( TP_NULL == m_status || TP_STOPED == m_status )
	{
		return true;
	}
	m_status = TP_STOPING;
	return m_pMaintain->Stop();
}


enumThreadPoolStatus CThreadPool::GetStatus()
{
	return m_status;
}


long CThreadPool::Maintain()
{
	GDPrint( "ThreadPool-%s Maintain started ...\n", m_poolName.c_str() );

	m_status = TP_STARTED;

	while ( TP_STOPING != m_status )
	{
		unsigned int threadCount = m_threadList.size();		// ̸߳
		unsigned int threadFree = 0;						// ߳̿

		// ̵߳״̬
		list<CThread*>::iterator itThread;
		for( itThread = m_threadList.begin(); itThread != m_threadList.end(); ++itThread )
		{
			if ( TS_NULL == (*itThread)->GetStatus() )
			{
				itThread = m_threadList.erase( itThread );
				m_threadTotalCount --;
			}
			else if ( TS_FREE == (*itThread)->GetStatus() )	// ձ
			{
				threadFree++;
			}
		}

		if ( ( threadFree < m_threadPoolInfo.nThreadMaxFreeNum ) && ( threadCount < m_threadPoolInfo.nThreadMaxNum ) )
		{
			// ߳Ŀ㣬ǰ߳Ŀ߳
			m_threadTotalCount++;
			char threadName[GD_MAX_PATH] = {0};
			sprintf( threadName, "%s-%d", m_poolName.c_str(), m_threadTotalCount );
			CThread* thread = m_pThread->NewThread( threadName );
			thread->m_nThreadUseTimes = m_threadPoolInfo.nThreadUseTimes;
			thread->Init( m_param );
			long nRet = thread->Start();

			if ( 0 == nRet )
			{
				m_threadList.push_back( thread );
			}
			else
			{
				GDETrace( "CThread:Start() ret=[%d]", nRet );
			}
		}
		else if ( threadFree > m_threadPoolInfo.nThreadMaxFreeNum )
		{
			// ߳Ŀ϶࣬ǰ߳Ŀ϶࣬ͷ߳
			list<CThread*>::iterator itFree;
			for( itFree = m_threadList.begin(); itFree != m_threadList.end(); ++itFree )
			{
				if ( TS_FREE == (*itFree)->GetStatus() )	// ͷ
				{
					(*itFree)->Stop();
					(*itFree)->Uninit();
					delete (*itFree);
					itFree = m_threadList.erase( itFree );
					break; // һͷһ߳
				}
			}
		}
		else
		{
			CDateTime::Sleep( 100 ); // ̣߳Ϣһ
		}

		if ( ( threadFree + 1 >= m_threadPoolInfo.nThreadMaxFreeNum ) || ( threadCount + 1 >= m_threadPoolInfo.nThreadMaxNum ) )
		{
			CDateTime::Sleep( 100 ); // ֻ޷߳ʱſϢ
		}
	}

	// ȴ߳̽
	list<CThread*>::iterator iter;
	for( iter = m_threadList.begin(); iter != m_threadList.end(); )
	{
		(*iter)->Stop();
		(*iter)->Uninit();
		delete (*iter);
		iter = m_threadList.erase( iter );
	}
	
	m_status = TP_STOPED;

	return 0;
}


}
}


