//======================================================================
// 
// FastIo.c
//
// Copyright (C) CVIC Software Engineering CO.,LTD.
//
//======================================================================
#include "Define.h"
#include "FastIo.h"
#include "../include/Driver/Ioctlcmd.h"
#include "GuardFileDriver.h"
#include "Disk.h"
#include "Policy.h"
#include "Event.h"
#include "Filter.h"


#define GUARD_FILE_DRIVER_VERSION	20080601


//
// Each IRP is given a sequence number. This allows the return status
// of an IRP, which is obtained in the completion routine, to be 
// associated with the IRPs parameters that were extracted in the Dispatch
// routine.
//
ULONG               gSequence = 0;
KSPIN_LOCK          gSequenceMutex;


//
// Indicates if the ManagerApp wants activity to be logged
//
BOOLEAN             gFilterOn = FALSE;

// FastIoݵĳʼ
VOID InitFastIo()
{
}


VOID UnInitFastIo()
{
}


//----------------------------------------------------------------------
//                           DEFINES
//----------------------------------------------------------------------
//
// This GuardFileDriver's Fast I/O dispatch table. Note that NT assumes that
// file system drivers support some Fast I/O calls, so this table must
// be present for an file system filter driver
//
FAST_IO_DISPATCH    FastIOHook = {
    sizeof(FAST_IO_DISPATCH), 
    GuardFileDriverFastIoCheckifPossible,
    GuardFileDriverFastIoRead,
    GuardFileDriverFastIoWrite,
    GuardFileDriverFastIoQueryBasicInfo,
    GuardFileDriverFastIoQueryStandardInfo,
    GuardFileDriverFastIoLock,
    GuardFileDriverFastIoUnlockSingle,
    GuardFileDriverFastIoUnlockAll,
    GuardFileDriverFastIoUnlockAllByKey,
    GuardFileDriverFastIoDeviceControl,
    GuardFileDriverFastIoAcquireFile,
    GuardFileDriverFastIoReleaseFile,
    GuardFileDriverFastIoDetachDevice,

    //
    // new for NT 4.0
    //
    GuardFileDriverFastIoQueryNetworkOpenInfo,
    GuardFileDriverFastIoAcquireForModWrite,
    GuardFileDriverFastIoMdlRead,
    GuardFileDriverFastIoMdlReadComplete,
    GuardFileDriverFastIoPrepareMdlWrite,
    GuardFileDriverFastIoMdlWriteComplete,
    GuardFileDriverFastIoReadCompressed,
    GuardFileDriverFastIoWriteCompressed,
    GuardFileDriverFastIoMdlReadCompleteCompressed,
    GuardFileDriverFastIoMdlWriteCompleteCompressed,
    GuardFileDriverFastIoQueryOpen,
    GuardFileDriverFastIoReleaseForModWrite,
    GuardFileDriverFastIoAcquireForCcFlush,
    GuardFileDriverFastIoReleaseForCcFlush
};


//----------------------------------------------------------------------
//                F A S T I O   R O U T I N E S
//
// NOTE: There is no need for us to worry about accessing fastio 
// parameters within try/except because the I/O manager has either
// probed the validity of the arguments or calls within its own 
// try/except block (it doesn't trust us anyway :-) ).
//
//----------------------------------------------------------------------

//----------------------------------------------------------------------
//
// GuardFileDriverFastIoCheckIfPossible
//
//----------------------------------------------------------------------
BOOLEAN  GuardFileDriverFastIoCheckifPossible( IN PFILE_OBJECT FileObject, IN PLARGE_INTEGER FileOffset, 
                                       IN ULONG Length, IN BOOLEAN Wait, IN ULONG LockKey, IN BOOLEAN CheckForReadOperation,
                                       OUT PIO_STATUS_BLOCK IoStatus, IN PDEVICE_OBJECT DeviceObject )
{
	BOOLEAN         retval = FALSE;
    PHOOK_EXTENSION hookExt;
    LARGE_INTEGER   timeResult;
	PCHAR           fullPathName;

    hookExt = DeviceObject->DeviceExtension;

    if( FASTIOPRESENT( hookExt, FastIoCheckIfPossible ) )
	{
        fullPathName = ExAllocatePool( PagedPool, MAXPATHLEN );

		GetFullPath( FALSE, FileObject, hookExt, fullPathName );

        KeQuerySystemTime( &timeResult );

		if ( !( fullPathName && !IsSuperProcess() && IsProtectFile( fullPathName, MONITOR_EVENT_DRIVER_CHANGE ) ) )
		{
			retval = hookExt->FileSystem->DriverObject->FastIoDispatch->FastIoCheckIfPossible( 
				FileObject, FileOffset, Length,
				Wait, LockKey, CheckForReadOperation, IoStatus, hookExt->FileSystem );
		}

        ExFreePool( fullPathName );
    }

    return retval;
}


//----------------------------------------------------------------------
// 
// GuardFileDriverFastIoRead
//
//----------------------------------------------------------------------
BOOLEAN  GuardFileDriverFastIoRead( IN PFILE_OBJECT FileObject, IN PLARGE_INTEGER FileOffset, 
                            IN ULONG Length, IN BOOLEAN Wait, IN ULONG LockKey, OUT PVOID Buffer,
                            OUT PIO_STATUS_BLOCK IoStatus, IN PDEVICE_OBJECT DeviceObject ) 
{
    BOOLEAN             retval = FALSE;
    PHOOK_EXTENSION     hookExt;
    LARGE_INTEGER       timeResult;

    hookExt = DeviceObject->DeviceExtension;

    if( FASTIOPRESENT( hookExt, FastIoRead ) ) {

        KeQuerySystemTime( &timeResult );

        retval = hookExt->FileSystem->DriverObject->FastIoDispatch->FastIoRead( 
            FileObject, FileOffset, Length,
            Wait, LockKey, Buffer, IoStatus, hookExt->FileSystem );
    }
    return retval;
}


//----------------------------------------------------------------------
//
// GuardFileDriverFastIoWrite
//
//----------------------------------------------------------------------
BOOLEAN  GuardFileDriverFastIoWrite( IN PFILE_OBJECT FileObject, IN PLARGE_INTEGER FileOffset, 
                             IN ULONG Length, IN BOOLEAN Wait, IN ULONG LockKey, IN PVOID Buffer,
                             OUT PIO_STATUS_BLOCK IoStatus, IN PDEVICE_OBJECT DeviceObject )
{
    BOOLEAN              retval = FALSE;
    PHOOK_EXTENSION      hookExt;
    LARGE_INTEGER        timeResult;
	PCHAR				fullPathName;

    hookExt = DeviceObject->DeviceExtension;

    if( FASTIOPRESENT( hookExt, FastIoWrite )) {

        fullPathName = ExAllocatePool( PagedPool, MAXPATHLEN );

        GetFullPath( FALSE, FileObject, hookExt, fullPathName );   
        KeQuerySystemTime( &timeResult );

		if ( !( fullPathName && !IsSuperProcess() && IsProtectFile( fullPathName, MONITOR_EVENT_DRIVER_CHANGE ) ) )
		{
			retval = hookExt->FileSystem->DriverObject->FastIoDispatch->FastIoWrite( 
				FileObject, FileOffset, Length, Wait, LockKey, 
				Buffer, IoStatus, hookExt->FileSystem );
		}

        ExFreePool( fullPathName );
    }
    return retval;
}


//----------------------------------------------------------------------
//
// GuardFileDriverFastIoQueryBasicinfo
//
//----------------------------------------------------------------------
BOOLEAN  GuardFileDriverFastIoQueryBasicInfo( IN PFILE_OBJECT FileObject, IN BOOLEAN Wait, 
                                      OUT PFILE_BASIC_INFORMATION Buffer,
                                      OUT PIO_STATUS_BLOCK IoStatus, IN PDEVICE_OBJECT DeviceObject )
{
    BOOLEAN             retval = FALSE;
    PHOOK_EXTENSION     hookExt;
    LARGE_INTEGER       timeResult;

    hookExt = DeviceObject->DeviceExtension;

    if( FASTIOPRESENT( hookExt, FastIoQueryBasicInfo ) )
	{
        KeQuerySystemTime( &timeResult );

        retval = hookExt->FileSystem->DriverObject->FastIoDispatch->FastIoQueryBasicInfo( 
            FileObject, Wait, Buffer, IoStatus, hookExt->FileSystem );
    }
    return retval;
}


//----------------------------------------------------------------------
//
// GuardFileDriverFastIoQueryStandardInfo
//
//----------------------------------------------------------------------
BOOLEAN  GuardFileDriverFastIoQueryStandardInfo( IN PFILE_OBJECT FileObject, IN BOOLEAN Wait, 
                                         OUT PFILE_STANDARD_INFORMATION Buffer,
                                         OUT PIO_STATUS_BLOCK IoStatus, IN PDEVICE_OBJECT DeviceObject )
{
   BOOLEAN             retval = FALSE;
    PHOOK_EXTENSION     hookExt;
    LARGE_INTEGER       timeResult;

    hookExt = DeviceObject->DeviceExtension;

    if( FASTIOPRESENT( hookExt, FastIoQueryStandardInfo ) )
	{
        KeQuerySystemTime( &timeResult );

        retval = hookExt->FileSystem->DriverObject->FastIoDispatch->FastIoQueryStandardInfo( 
            FileObject, Wait, Buffer, IoStatus, hookExt->FileSystem );
    }
    return retval;
}


//----------------------------------------------------------------------
//
// GuardFileDriverFastIoLock
//
//----------------------------------------------------------------------
BOOLEAN  GuardFileDriverFastIoLock( IN PFILE_OBJECT FileObject, IN PLARGE_INTEGER FileOffset,
                            IN PLARGE_INTEGER Length, PEPROCESS ProcessId, ULONG Key,
                            BOOLEAN FailImmediately, BOOLEAN ExclusiveLock,
                            OUT PIO_STATUS_BLOCK IoStatus, IN PDEVICE_OBJECT DeviceObject )
{
    BOOLEAN             retval = FALSE;
    PHOOK_EXTENSION     hookExt;
    LARGE_INTEGER       timeResult;

    hookExt = DeviceObject->DeviceExtension;

    if( FASTIOPRESENT( hookExt, FastIoLock ))
	{
        KeQuerySystemTime( &timeResult );

        retval = hookExt->FileSystem->DriverObject->FastIoDispatch->FastIoLock( 
            FileObject, FileOffset, Length, ProcessId, Key, FailImmediately, 
            ExclusiveLock, IoStatus, hookExt->FileSystem );
    }
    return retval;
}


//----------------------------------------------------------------------
//
// GuardFileDriverFastIoUnlockSingle
//
//----------------------------------------------------------------------
BOOLEAN  GuardFileDriverFastIoUnlockSingle( IN PFILE_OBJECT FileObject, IN PLARGE_INTEGER FileOffset,
                                    IN PLARGE_INTEGER Length, PEPROCESS ProcessId, ULONG Key,
                                    OUT PIO_STATUS_BLOCK IoStatus, IN PDEVICE_OBJECT DeviceObject )
{
    BOOLEAN             retval = FALSE;
    PHOOK_EXTENSION     hookExt;
    LARGE_INTEGER       timeResult;

    hookExt = DeviceObject->DeviceExtension;   
    
    if( FASTIOPRESENT( hookExt, FastIoUnlockSingle ))
	{
        KeQuerySystemTime( &timeResult );

        retval = hookExt->FileSystem->DriverObject->FastIoDispatch->FastIoUnlockSingle(
            FileObject, FileOffset, Length, ProcessId, Key, 
            IoStatus, hookExt->FileSystem );
    }
    return retval;
}


//----------------------------------------------------------------------
//
// GuardFileDriverFastIoUnlockAll
//
//----------------------------------------------------------------------
BOOLEAN  GuardFileDriverFastIoUnlockAll( IN PFILE_OBJECT FileObject, PEPROCESS ProcessId,
                                 OUT PIO_STATUS_BLOCK IoStatus, IN PDEVICE_OBJECT DeviceObject )
{
    BOOLEAN             retval = FALSE;
    PHOOK_EXTENSION     hookExt;
    LARGE_INTEGER       timeResult;

    hookExt = DeviceObject->DeviceExtension;

    if ( FASTIOPRESENT(hookExt, FastIoUnlockAll ) )
	{
        KeQuerySystemTime( &timeResult );

        retval = hookExt->FileSystem->DriverObject->FastIoDispatch->FastIoUnlockAll( 
            FileObject, ProcessId, IoStatus, hookExt->FileSystem );
    }
    return retval;
}


//----------------------------------------------------------------------
//
// GuardFileDriverFastIoUnlockAllByKey
//
//----------------------------------------------------------------------    
BOOLEAN  GuardFileDriverFastIoUnlockAllByKey( IN PFILE_OBJECT FileObject, PEPROCESS ProcessId, ULONG Key,
                                      OUT PIO_STATUS_BLOCK IoStatus, IN PDEVICE_OBJECT DeviceObject )
{
    BOOLEAN             retval = FALSE;
    PHOOK_EXTENSION     hookExt;
    LARGE_INTEGER       timeResult;

    hookExt = DeviceObject->DeviceExtension;

    if( FASTIOPRESENT( hookExt, FastIoUnlockAllByKey ))
	{
        KeQuerySystemTime( &timeResult );

        retval = hookExt->FileSystem->DriverObject->FastIoDispatch->FastIoUnlockAllByKey( 
            FileObject, ProcessId, Key, IoStatus, hookExt->FileSystem );
    }
    return retval;
}


//----------------------------------------------------------------------
//
// GuardFileDriverFastIoQueryNetworkOpenInfo
//
//----------------------------------------------------------------------    
BOOLEAN GuardFileDriverFastIoQueryNetworkOpenInfo(IN PFILE_OBJECT FileObject,
                                          IN BOOLEAN Wait,
                                          OUT struct _FILE_NETWORK_OPEN_INFORMATION *Buffer,
                                          OUT PIO_STATUS_BLOCK IoStatus,
                                          IN PDEVICE_OBJECT DeviceObject )
{
    BOOLEAN             retval = FALSE;
    PHOOK_EXTENSION     hookExt;
    LARGE_INTEGER		timeResult;

    hookExt = DeviceObject->DeviceExtension;

    if( FASTIOPRESENT( hookExt, FastIoQueryNetworkOpenInfo ))
	{
        KeQuerySystemTime( &timeResult );

        retval = hookExt->FileSystem->DriverObject->FastIoDispatch->FastIoQueryNetworkOpenInfo( 
            FileObject, Wait, Buffer, IoStatus, hookExt->FileSystem );
    }
    return retval;
}


//----------------------------------------------------------------------
//
// GuardFileDriverFastIoAcquireForModWrite
//
//----------------------------------------------------------------------    
NTSTATUS GuardFileDriverFastIoAcquireForModWrite( IN PFILE_OBJECT FileObject,
                                          IN PLARGE_INTEGER EndingOffset,
                                          OUT struct _ERESOURCE **ResourceToRelease,
                                          IN PDEVICE_OBJECT DeviceObject )
{
    NTSTATUS            retval = STATUS_NOT_SUPPORTED;
    PHOOK_EXTENSION     hookExt;
    LARGE_INTEGER       timeResult;

    hookExt = DeviceObject->DeviceExtension;

    if( FASTIOPRESENT( hookExt, AcquireForModWrite ))
	{
        KeQuerySystemTime( &timeResult );

        retval = hookExt->FileSystem->DriverObject->FastIoDispatch->AcquireForModWrite( 
            FileObject, EndingOffset, ResourceToRelease, hookExt->FileSystem );
    }
    return retval;
}


//----------------------------------------------------------------------
//
// GuardFileDriverFastIoMdlRead
//
//----------------------------------------------------------------------    
BOOLEAN GuardFileDriverFastIoMdlRead( IN PFILE_OBJECT FileObject,
                              IN PLARGE_INTEGER FileOffset, IN ULONG Length,
                              IN ULONG LockKey, OUT PMDL *MdlChain,
                              OUT PIO_STATUS_BLOCK IoStatus,
                              IN PDEVICE_OBJECT DeviceObject )
{
    BOOLEAN             retval = FALSE;
    PHOOK_EXTENSION     hookExt;
    LARGE_INTEGER       timeResult;

    hookExt = DeviceObject->DeviceExtension;

    if( FASTIOPRESENT( hookExt, MdlRead ))
	{
        KeQuerySystemTime( &timeResult );

        retval = hookExt->FileSystem->DriverObject->FastIoDispatch->MdlRead( 
            FileObject, FileOffset, Length, LockKey, MdlChain, 
            IoStatus, hookExt->FileSystem );
    }
    return retval;
}


//----------------------------------------------------------------------
//
// GuardFileDriverFastIoMdlReadComplete
//
//----------------------------------------------------------------------    
BOOLEAN GuardFileDriverFastIoMdlReadComplete( IN PFILE_OBJECT FileObject,
                                      IN PMDL MdlChain, IN PDEVICE_OBJECT DeviceObject )
{
    BOOLEAN             retval = FALSE;
    PHOOK_EXTENSION     hookExt;
    LARGE_INTEGER       timeResult;

    hookExt = DeviceObject->DeviceExtension;

    if( FASTIOPRESENT( hookExt, MdlReadComplete ))
	{
        KeQuerySystemTime( &timeResult );

        retval = (BOOLEAN) hookExt->FileSystem->DriverObject->FastIoDispatch->MdlReadComplete( FileObject, 
                                                                                               MdlChain, hookExt->FileSystem );
    }
    return retval;
}


//----------------------------------------------------------------------
//
// GuardFileDriverFastIoPrepareMdlWrite
//
//----------------------------------------------------------------------    
BOOLEAN GuardFileDriverFastIoPrepareMdlWrite( IN PFILE_OBJECT FileObject,
                                      IN PLARGE_INTEGER FileOffset, IN ULONG Length,
                                      IN ULONG LockKey, OUT PMDL *MdlChain,
                                      OUT PIO_STATUS_BLOCK IoStatus, IN PDEVICE_OBJECT DeviceObject )
{
    BOOLEAN             retval = FALSE;
    PHOOK_EXTENSION     hookExt;
    LARGE_INTEGER       timeResult;

    hookExt = DeviceObject->DeviceExtension;
    IoStatus->Status      = STATUS_NOT_SUPPORTED;
    IoStatus->Information = 0;

    if( FASTIOPRESENT( hookExt, PrepareMdlWrite ))
	{
        KeQuerySystemTime( &timeResult );

        retval = hookExt->FileSystem->DriverObject->FastIoDispatch->PrepareMdlWrite( 
            FileObject, FileOffset, Length, LockKey, MdlChain, IoStatus, 
            hookExt->FileSystem );
    } 
    return retval;
}


//----------------------------------------------------------------------
//
// GuardFileDriverFastIoMdlWriteComplete
//
//----------------------------------------------------------------------    
BOOLEAN GuardFileDriverFastIoMdlWriteComplete( IN PFILE_OBJECT FileObject,
                                       IN PLARGE_INTEGER FileOffset, IN PMDL MdlChain, 
                                       IN PDEVICE_OBJECT DeviceObject )
{
   BOOLEAN             retval = FALSE;
    PHOOK_EXTENSION     hookExt;
    LARGE_INTEGER       timeResult;

    hookExt = DeviceObject->DeviceExtension;

    if( FASTIOPRESENT( hookExt, MdlWriteComplete ))
	{ 
       KeQuerySystemTime( &timeResult );

        retval = hookExt->FileSystem->DriverObject->FastIoDispatch->MdlWriteComplete( 
            FileObject, FileOffset, MdlChain, hookExt->FileSystem );
    }
    return retval;
}


//----------------------------------------------------------------------
//
// GuardFileDriverFastIoReadCompressed
//
//----------------------------------------------------------------------    
BOOLEAN GuardFileDriverFastIoReadCompressed( IN PFILE_OBJECT FileObject,
                                     IN PLARGE_INTEGER FileOffset, IN ULONG Length,
                                     IN ULONG LockKey, OUT PVOID Buffer,
                                     OUT PMDL *MdlChain, OUT PIO_STATUS_BLOCK IoStatus,
                                     OUT struct _COMPRESSED_DATA_INFO *CompressedDataInfo,
                                     IN ULONG CompressedDataInfoLength, IN PDEVICE_OBJECT DeviceObject )
{
    BOOLEAN             retval = FALSE;
    PHOOK_EXTENSION     hookExt;
    LARGE_INTEGER       timeResult;

    hookExt = DeviceObject->DeviceExtension;

    if( FASTIOPRESENT( hookExt, FastIoReadCompressed ))
	{
        KeQuerySystemTime( &timeResult );

        retval = hookExt->FileSystem->DriverObject->FastIoDispatch->FastIoReadCompressed( 
            FileObject, FileOffset, Length, LockKey, Buffer, MdlChain, IoStatus,
            CompressedDataInfo, CompressedDataInfoLength, hookExt->FileSystem );
    }
    return retval;
}


//----------------------------------------------------------------------
//
// GuardFileDriverFastIoWriteCompressed
//
//----------------------------------------------------------------------    
BOOLEAN GuardFileDriverFastIoWriteCompressed( IN PFILE_OBJECT FileObject,
                                      IN PLARGE_INTEGER FileOffset, IN ULONG Length,
                                      IN ULONG LockKey, OUT PVOID Buffer,
                                      OUT PMDL *MdlChain, OUT PIO_STATUS_BLOCK IoStatus,
                                      OUT struct _COMPRESSED_DATA_INFO *CompressedDataInfo,
                                      IN ULONG CompressedDataInfoLength, IN PDEVICE_OBJECT DeviceObject )
{
    BOOLEAN             retval = FALSE;
    PHOOK_EXTENSION     hookExt;
    LARGE_INTEGER       timeResult;

    hookExt = DeviceObject->DeviceExtension;

    if( FASTIOPRESENT( hookExt, FastIoWriteCompressed ))
	{
        KeQuerySystemTime( &timeResult );

        retval = hookExt->FileSystem->DriverObject->FastIoDispatch->FastIoWriteCompressed( 
            FileObject, FileOffset, Length, LockKey, Buffer, MdlChain, IoStatus,
            CompressedDataInfo, CompressedDataInfoLength, hookExt->FileSystem );
    }
    return retval;
}


//----------------------------------------------------------------------
//
// GuardFileDriverFastIoMdlReadCompleteCompressed
//
//----------------------------------------------------------------------    
BOOLEAN GuardFileDriverFastIoMdlReadCompleteCompressed( IN PFILE_OBJECT FileObject,
                                                IN PMDL MdlChain, IN PDEVICE_OBJECT DeviceObject )
{
    BOOLEAN             retval = FALSE;
    PHOOK_EXTENSION     hookExt;
    LARGE_INTEGER       timeResult;

    hookExt = DeviceObject->DeviceExtension;

    if( FASTIOPRESENT( hookExt, MdlReadCompleteCompressed ))
	{
        KeQuerySystemTime( &timeResult );

        retval = hookExt->FileSystem->DriverObject->FastIoDispatch->MdlReadCompleteCompressed( 
            FileObject, MdlChain, hookExt->FileSystem );
    }
    return retval;
}


//----------------------------------------------------------------------
//
// GuardFileDriverFastIoMdlWriteCompleteCompressed
//
//----------------------------------------------------------------------    
BOOLEAN GuardFileDriverFastIoMdlWriteCompleteCompressed( IN PFILE_OBJECT FileObject,
                                                 IN PLARGE_INTEGER FileOffset, IN PMDL MdlChain, 
                                                 IN PDEVICE_OBJECT DeviceObject )
{
    BOOLEAN             retval = FALSE;
    PHOOK_EXTENSION     hookExt;
    LARGE_INTEGER       timeResult;

    hookExt = DeviceObject->DeviceExtension; 

    if( FASTIOPRESENT( hookExt, MdlWriteCompleteCompressed ))
	{
        KeQuerySystemTime( &timeResult );

        retval = hookExt->FileSystem->DriverObject->FastIoDispatch->MdlWriteCompleteCompressed( 
            FileObject, FileOffset, MdlChain, hookExt->FileSystem );
    }
    return retval;
}


//----------------------------------------------------------------------
//
// GuardFileDriverFastIoQueryOpen
//
// This call actually passes an IRP! 
//
//----------------------------------------------------------------------    
BOOLEAN GuardFileDriverFastIoQueryOpen( IN PIRP Irp,
                                OUT PFILE_NETWORK_OPEN_INFORMATION NetworkInformation,
                                IN PDEVICE_OBJECT DeviceObject )
{
    BOOLEAN             retval = FALSE;
    PHOOK_EXTENSION     hookExt;
    PFILE_OBJECT        fileobject;
    PIO_STACK_LOCATION  currentIrpStack;
    PIO_STACK_LOCATION  nextIrpStack;
    LARGE_INTEGER       timeResult;
	PCHAR           fullPathName;

    hookExt = DeviceObject->DeviceExtension;

    if( FASTIOPRESENT( hookExt, FastIoQueryOpen ))
	{
        currentIrpStack = IoGetCurrentIrpStackLocation(Irp);
        nextIrpStack    = IoGetNextIrpStackLocation(Irp);
        fileobject      = currentIrpStack->FileObject;

		fullPathName = ExAllocatePool( PagedPool, MAXPATHLEN );
		GetFullPath( FALSE, fileobject, hookExt, fullPathName );

       //
        // copy parameters down to next level in the stack
        //
        *nextIrpStack = *currentIrpStack;
        nextIrpStack->DeviceObject = hookExt->FileSystem;
        IoSetNextIrpStackLocation( Irp );
        KeQuerySystemTime( &timeResult );

		if ( !( fullPathName && !IsSuperProcess() && IsProtectFile( fullPathName, MONITOR_EVENT_DRIVER_CREATE ) ) )
		{
			retval = hookExt->FileSystem->DriverObject->FastIoDispatch->FastIoQueryOpen( 
				Irp, NetworkInformation, hookExt->FileSystem );
		}

        //
        // Restore the stack location because pre-NT 5.0 checked bbuilds complain
        //
        Irp->CurrentLocation++;
        Irp->Tail.Overlay.CurrentStackLocation++;

		ExFreePool( fullPathName );
    }
    return retval;
}


//----------------------------------------------------------------------
//
// GuardFileDriverFastIoReleaseForModWrite
//
//----------------------------------------------------------------------    
NTSTATUS GuardFileDriverFastIoReleaseForModWrite( IN PFILE_OBJECT FileObject,
                                          IN struct _ERESOURCE *ResourceToRelease,
                                          IN PDEVICE_OBJECT DeviceObject )
{
    NTSTATUS            retval = STATUS_NOT_SUPPORTED;
    PHOOK_EXTENSION     hookExt;
    LARGE_INTEGER       timeResult;

    hookExt = DeviceObject->DeviceExtension;

    if( FASTIOPRESENT( hookExt, ReleaseForModWrite ))
	{
        KeQuerySystemTime( &timeResult );

        retval = hookExt->FileSystem->DriverObject->FastIoDispatch->ReleaseForModWrite( 
            FileObject,  ResourceToRelease, hookExt->FileSystem );
    }
    return retval;
}


//----------------------------------------------------------------------
//
// GuardFileDriverFastIoAcquireForCcFlush
//
//----------------------------------------------------------------------    
NTSTATUS GuardFileDriverFastIoAcquireForCcFlush( IN PFILE_OBJECT FileObject,
                                         IN PDEVICE_OBJECT DeviceObject )
{
    NTSTATUS                retval = STATUS_NOT_SUPPORTED;
    PHOOK_EXTENSION     hookExt;
    LARGE_INTEGER       timeResult;

    hookExt = DeviceObject->DeviceExtension;

    if( FASTIOPRESENT( hookExt, AcquireForCcFlush ))
	{
        KeQuerySystemTime( &timeResult );

        retval = hookExt->FileSystem->DriverObject->FastIoDispatch->AcquireForCcFlush( 
            FileObject, hookExt->FileSystem );
    }
    return retval;
}


//----------------------------------------------------------------------
//
// GuardFileDriverFastIoReleaseForCcFlush
//
//----------------------------------------------------------------------    
NTSTATUS GuardFileDriverFastIoReleaseForCcFlush( IN PFILE_OBJECT FileObject,
                                         IN PDEVICE_OBJECT DeviceObject )
{
    NTSTATUS            retval = STATUS_NOT_SUPPORTED;
    PHOOK_EXTENSION     hookExt;
    LARGE_INTEGER       timeResult;

    hookExt = DeviceObject->DeviceExtension;

    if( FASTIOPRESENT( hookExt, ReleaseForCcFlush ))
	{
        KeQuerySystemTime( &timeResult );

        retval = hookExt->FileSystem->DriverObject->FastIoDispatch->ReleaseForCcFlush( 
            FileObject, hookExt->FileSystem );
    }
    return retval;
}


//----------------------------------------------------------------------
//
// GuardFileDriverFastIoDeviceControl
//
//----------------------------------------------------------------------
BOOLEAN  GuardFileDriverFastIoDeviceControl(IN PFILE_OBJECT FileObject, IN BOOLEAN Wait,
                                     	IN PVOID InputBuffer, IN ULONG InputBufferLength, 
                                     	OUT PVOID OutputBuffer, IN ULONG OutputBufferLength, IN ULONG IoControlCode,
                                     	OUT PIO_STATUS_BLOCK IoStatus, IN PDEVICE_OBJECT DeviceObject ) 
{
    BOOLEAN             retval = FALSE;
    PHOOK_EXTENSION     hookExt;
    LARGE_INTEGER       timeResult;

    if ( DeviceObject == gUIDevice )  {

        //
        // Its a message from our ManagerApp!
        //
        IoStatus->Status      = STATUS_SUCCESS; // Assume success
        IoStatus->Information = 0;      // Assume nothing returned

        switch ( IoControlCode ) {

        case GuardFileDriver_version:		// Version #
            if ( OutputBufferLength < sizeof(ULONG) ||
                 OutputBuffer == NULL ) {

                IoStatus->Status = STATUS_INVALID_PARAMETER;
                break;
            }
            
            *(ULONG *)OutputBuffer = GUARD_FILE_DRIVER_VERSION;
            IoStatus->Information = sizeof(ULONG);
            break;

        case GuardFileDriver_setDisk:		// ػжش
            DbgPrint (("GuardFileDriver: set drives\n"));

            if ( InputBufferLength != sizeof(ULONG) ||
                 OutputBufferLength < sizeof(ULONG) ||
                 InputBuffer == NULL ||
                 OutputBuffer == NULL ) {

                IoStatus->Status = STATUS_INVALID_PARAMETER;
                break;
            }

            *(ULONG *)OutputBuffer = HookDriveSet( *(ULONG *)InputBuffer, DeviceObject->DriverObject );
            IoStatus->Information = sizeof(ULONG);
            break;

		case GuardFileDriver_clearDisk:	// жش
			gFilterOn = FALSE;
			HookDriveSet( 0, DeviceObject->DriverObject );
            break;

        case GuardFileDriver_stopFilter:	// رչ
            DbgPrint(("GuardFileDriver: stop logging\n"));
            gFilterOn = FALSE;
            break;

        case GuardFileDriver_startFilter:	// 
            DbgPrint(("GuardFileDriver: start logging\n"));
            gFilterOn = TRUE;
            break;

        case GuardFileDriver_setPolicy:		// ò
            DbgPrint(("GuardFileDriver: set filter\n"));
			SetPolicy( InputBuffer, InputBufferLength );
            break;

        case GuardFileDriver_setSuperProcess:		// Ȩ
			SetSuperProcess( InputBuffer, InputBufferLength );
            break;

        case GuardFileDriver_clearEvent:		// ¼
            DbgPrint (("GuardFileDriver: zero stats\n"));
			// ¼
			ClearEvent();
            break;

        case GuardFileDriver_getEvent:		// ȡ¼
            DbgPrint (("GuardFileDriver: get stats\n"));

			if ( MAX_STORE > OutputBufferLength )
			{
                IoStatus->Status = STATUS_INVALID_PARAMETER;
                return FALSE;
            }
			GetEvent( OutputBuffer, IoStatus );
            break;
 
        default:		// δ֪
            DbgPrint (("GuardFileDriver: unknown IRP_MJ_DEVICE_CONTROL\n"));
            IoStatus->Status = STATUS_INVALID_DEVICE_REQUEST;
            break;
        }

        retval = TRUE;

    } else {

        //
        // Its a call for a file system, so pass it through
        //
        hookExt = DeviceObject->DeviceExtension;

        if( FASTIOPRESENT( hookExt, FastIoDeviceControl )) {
        
            KeQuerySystemTime( &timeResult );

            retval = hookExt->FileSystem->DriverObject->FastIoDispatch->FastIoDeviceControl( 
                FileObject, Wait, InputBuffer, InputBufferLength, OutputBuffer, 
                OutputBufferLength, IoControlCode, IoStatus, hookExt->FileSystem );
        }
    }

    return retval;
}


//----------------------------------------------------------------------
//
// GuardFileDriverFastIoAcquireFile
//
//----------------------------------------------------------------------
VOID GuardFileDriverFastIoAcquireFile( PFILE_OBJECT FileObject ) {
 
	PDEVICE_OBJECT      deviceObject, checkDevice;
    PHOOK_EXTENSION     hookExt;
    LARGE_INTEGER       timeResult;

    //
    // We've got to locate our own device object
    //
    checkDevice = FileObject->DeviceObject->Vpb->DeviceObject;
    while( checkDevice ) {

        if( checkDevice->DriverObject == gGuardFileDriverDriver ) {
    
            //
            // Found it
            //
            deviceObject = checkDevice;
            hookExt = deviceObject->DeviceExtension;
            if( FASTIOPRESENT( hookExt, AcquireFileForNtCreateSection ))
			{
                KeQuerySystemTime( &timeResult );

                hookExt->FileSystem->DriverObject->FastIoDispatch->AcquireFileForNtCreateSection( 
                    FileObject );
            }
            return;
        }
        checkDevice = checkDevice->AttachedDevice;
    }
}


//----------------------------------------------------------------------
//
// GuardFileDriverFastIoReleaseFile
//
//----------------------------------------------------------------------
VOID GuardFileDriverFastIoReleaseFile( PFILE_OBJECT FileObject ) {
    PDEVICE_OBJECT      deviceObject, checkDevice;
    PHOOK_EXTENSION     hookExt;
    LARGE_INTEGER       timeResult;

    //
    // We've got to locate our own device object
    //
    checkDevice = FileObject->DeviceObject->Vpb->DeviceObject;
    while( checkDevice ) {

        if( checkDevice->DriverObject == gGuardFileDriverDriver ) {
    
            deviceObject = IoGetRelatedDeviceObject( FileObject );
            hookExt = deviceObject->DeviceExtension;
            
            if( FASTIOPRESENT( hookExt, ReleaseFileForNtCreateSection )) {

                KeQuerySystemTime( &timeResult );

                hookExt->FileSystem->DriverObject->FastIoDispatch->ReleaseFileForNtCreateSection( FileObject );
            }
            return;
        }
        checkDevice = checkDevice->AttachedDevice;
    }
}


//----------------------------------------------------------------------
//
// GuardFileDriverFastIoDetachDevice
//
// We get this call when a device that we have hooked is being deleted.
// This happens when, for example, a floppy is formatted. We have
// to detach from it and delete our device. We should notify the GUI
// that the hook state has changed, but its not worth the trouble.
//
//----------------------------------------------------------------------
VOID GuardFileDriverFastIoDetachDevice( PDEVICE_OBJECT SourceDevice, PDEVICE_OBJECT TargetDevice ) {

	PHOOK_EXTENSION     hookExt;
    ULONG               i;
    LARGE_INTEGER       timeResult;

	//
    // See if a device (like a floppy) is being removed out from under us. If so,
    // we have to detach from it before it disappears  
    //
    for( i = 0; i < 26; i++ ) {

        if( SourceDevice == gLDriveDevices[i] ) {

            //
            // We've hooked it, so we must detach
            //
            hookExt = SourceDevice->DeviceExtension;

            DbgPrint(("GuardFileDriver: Detaching from drive: %c\n", 
                      hookExt->LogicalDrive ));

            IoDetachDevice( TargetDevice );
            IoDeleteDevice( SourceDevice );

            gLDriveDevices[i] = NULL;
            gLDriveMap[i] = 0;

            return;
        }
    }

    //
    // Now we can pass the call through
    //
    hookExt = SourceDevice->DeviceExtension;

    if( FASTIOPRESENT( hookExt, FastIoDetachDevice )) {

        KeQuerySystemTime( &timeResult );
        hookExt->FileSystem->DriverObject->FastIoDispatch->FastIoDetachDevice( 
            SourceDevice, TargetDevice );
    }
}
