//
// NumberParser.cpp
//


#include "Groundwork/NumberParser.h"
#include "Groundwork/Exception.h"
#include <cstdio>
#include <cctype>


#if defined(_MSC_VER)
	#define I64_FMT "I64"
#elif defined(__APPLE__) 
	#define I64_FMT "q"
#else
	#define I64_FMT "ll"
#endif


namespace InforGuard {
namespace Groundwork {


int NumberParser::parse(const string& s)
{
	int result;
	if (tryParse(s, result))
		return result;
	else
		throw SyntaxException("Not a valid integer", s);
}


bool NumberParser::tryParse(const string& s, int& value)
{
	char temp;
	return sscanf(s.c_str(), "%d%c", &value, &temp) == 1;
}


unsigned NumberParser::parseUnsigned(const string& s)
{
	unsigned result;
	if (tryParseUnsigned(s, result))
		return result;
	else
		throw SyntaxException("Not a valid unsigned integer", s);
}


bool NumberParser::tryParseUnsigned(const string& s, unsigned& value)
{
	char temp;
	return sscanf(s.c_str(), "%u%c", &value, &temp) == 1;
}


unsigned NumberParser::parseHex(const string& s)
{
	unsigned result;
	if (tryParseHex(s, result))
		return result;
	else
		throw SyntaxException("Not a valid hexadecimal integer", s);
}


bool NumberParser::tryParseHex(const string& s, unsigned& value)
{
	char temp;
	return sscanf(s.c_str(), "%x%c", &value, &temp) == 1;
}


#if defined(GROUNDWORK_HAVE_INT64)


Int64 NumberParser::parse64(const string& s)
{
	Int64 result;
	if (tryParse64(s, result))
		return result;
	else
		throw SyntaxException("Not a valid integer", s);
}


bool NumberParser::tryParse64(const string& s, Int64& value)
{
	char temp;
	return sscanf(s.c_str(), "%"I64_FMT"d%c", &value, &temp) == 1;
}


UInt64 NumberParser::parseUnsigned64(const string& s)
{
	UInt64 result;
	if (tryParseUnsigned64(s, result))
		return result;
	else
		throw SyntaxException("Not a valid unsigned integer", s);
}


bool NumberParser::tryParseUnsigned64(const string& s, UInt64& value)
{
	char temp;
	return sscanf(s.c_str(), "%"I64_FMT"u%c", &value, &temp) == 1;
}


UInt64 NumberParser::parseHex64(const string& s)
{
	UInt64 result;
	if (tryParseHex64(s, result))
		return result;
	else
		throw SyntaxException("Not a valid hexadecimal integer", s);
}


bool NumberParser::tryParseHex64(const string& s, UInt64& value)
{
	char temp;
	return sscanf(s.c_str(), "%"I64_FMT"x%c", &value, &temp) == 1;
}


#endif // defined(GROUNDWORK_HAVE_INT64)


double NumberParser::parseFloat(const string& s)
{
	double result;
	if (tryParseFloat(s, result))
		return result;
	else
		throw SyntaxException("Not a valid floating-point number", s);
}

	
bool NumberParser::tryParseFloat(const string& s, double& value)
{
	char temp;
	return sscanf(s.c_str(), "%lf%c", &value, &temp) == 1;
}


} // namespace Groundwork
}	// namespace InforGuard
