/***********************************************************************
*
* ļ              WaterMarkDB.cpp
* ģĿģ		࣬ʵˮӡݿ
* ߼ڣ        zhang_dpeng 2007-09-01
* 汾                1.0
* ȨϢ            CVICSE Middleware
* ļ¼            
*
***********************************************************************/

#include "Common/DateTime.h"
#include "Common/GDFile.h"
#include "DBTree/DB.h"
#include "FileOpt/FileOptDef.h"
#include "GDMD5/GDMD5.h"
#include "GDLog/GDTrace.h"
#include "WaterMark/WaterMarkDB.h"
#include "FileOpt/FileOpt.h"
#include "FileOpt/FileList.h"

#ifdef WIN32
#include <windows.h>
#else
#include <sys/stat.h>
#endif 

#ifndef NOT_USE_STD_LIB
#	include <fstream>
#else
#	include <fstream.h>
#endif

#define FILEREADBUFLEN (1024 * 10)
#define UPDATEMD5RESTSIGN 10	// md5ɶٴκϢ


struct RECORD_S 
{
	char sWm[WMARK_LEN];
};

struct KEY_RECORD
{
	char sFilePath[256];
	RECORD_S record;
};

// by liu_yfeng 2009-03-11 Ƿˮӡı־
bool CWaterMarkDB::m_bUseDb = true;

CWaterMarkDB::CWaterMarkDB()
: m_bIsOpen(false)
{
	m_pDB = CDB::Instance();
}

CWaterMarkDB::~CWaterMarkDB()
{
	if ( m_bIsOpen )
	{
		if ( m_pDB )
		{
			m_pDB->Uninitialize();
			delete m_pDB;
			m_pDB = NULL;
		}
	}
	//Close();
}

int CWaterMarkDB::EncodeToUTF8(const wchar_t* szSrc, char* szDst)
{
#ifdef WIN32
	WORD ch;
	
	BYTE bt1, bt2, bt3, bt4, bt5, bt6;
	
	int n, nMax = wcslen(szSrc);
	
	char sTemp[GD_MAX_PATH] = {0};
	int nPos = 0;
	
	for (n = 0; n < nMax; ++n)
	{
		ch = (WORD)szSrc[n];
		
		if (ch == '=')
		{
			szDst[nPos++] = ch;
		}
		else if (ch < 128)
		{
			szDst[nPos++] = szSrc[n];
		}
		else if (ch <= 2047)
		{
			bt1 = (BYTE)(192 + (ch / 64));
			bt2 = (BYTE)(128 + (ch % 64));
			
			szDst[nPos++] = bt1;
			szDst[nPos++] = bt2;
		}
		else if (ch <= 65535)
		{
			bt1 = (BYTE)(224 + (ch / 4096));
			bt2 = (BYTE)(128 + ((ch / 64) % 64));
			bt3 = (BYTE)(128 + (ch % 64));
			
			szDst[nPos++] = bt1;
			szDst[nPos++] = bt2;
			szDst[nPos++] = bt3;
		}
		else if (ch <= 2097151)
		{
			bt1 = (BYTE)(240 + (ch / 262144));
			bt2 = (BYTE)(128 + ((ch / 4096) % 64));
			bt3 = (BYTE)(128 + ((ch / 64) % 64));
			bt4 = (BYTE)(128 + (ch % 64));
			
			szDst[nPos++] = bt1;
			szDst[nPos++] = bt2;
			szDst[nPos++] = bt3;
			szDst[nPos++] = bt4;
		}
		else if (ch <=67108863)
		{
			bt1 = (BYTE)(248 + (ch / 16777216));
			bt2 = (BYTE)(128 + ((ch / 262144) % 64));
			bt3 = (BYTE)(128 + ((ch / 4096) % 64));
			bt4 = (BYTE)(128 + ((ch / 64) % 64));
			bt5 = (BYTE)(128 + (ch % 64));
			
			szDst[nPos++] = bt1;
			szDst[nPos++] = bt2;
			szDst[nPos++] = bt3;
			szDst[nPos++] = bt4;
			szDst[nPos++] = bt5;
		}
		else if (ch <=2147483647)
		{
			bt1 = (BYTE)(252 + (ch / 1073741824));
			bt2 = (BYTE)(128 + ((ch / 16777216) % 64));
			bt3 = (BYTE)(128 + ((ch / 262144) % 64));
			bt4 = (BYTE)(128 + ((ch / 4096) % 64));
			bt5 = (BYTE)(128 + ((ch / 64) % 64));
			bt6 = (BYTE)(128 + (ch % 64));
			
			szDst[nPos++] = bt1;
			szDst[nPos++] = bt2;
			szDst[nPos++] = bt3;
			szDst[nPos++] = bt4;
			szDst[nPos++] = bt5;
			szDst[nPos++] = bt6;
		}
		
	}
#endif
	return 0;
}

void CWaterMarkDB::ConvertPath(char* path)
{
	int len = 0;

	len = strlen(path);

	for (int i = 0; i < len; i++)
        path[i] = tolower(path[i]);
}


long CWaterMarkDB::Init(const char* sDbPath, const char* sDbName, long nKey, bool bRunSave, long nSize, long nSaveCount, long nSaveSeconds, bool bUseDb ) throw( CExceptionError )
{
	CGDTrace trace( LOG_L6, "CWaterMarkDB:Init(%s,%s,%x,%s,%d,%d,%d)", 
		sDbPath==NULL?"(null)":sDbPath, 
		sDbName==NULL?"(null)":sDbName, nKey, bRunSave?"true":"false", nSize, nSaveCount, nSaveSeconds );
	
	// by liu_yfeng 2009-03-11 Ƿˮӡı־
	m_bUseDb = bUseDb;

	long nRet = 0;
	// by liu_yfeng 2009-03-11 Ƿˮӡж
	if ( !m_bUseDb )
	{
		return 0;
	}

	if ( NULL == sDbPath || NULL == sDbName || '\0' == *sDbPath || '\0' == *sDbName )
	{
		nRet = -1;
	}
	else
	{
        // ȷDBPath
        if( !CFileOpt::IsDirectory(sDbPath) )
        {
            try
            {
                CFileOpt::MakeDir(sDbPath);
            }
            catch ( CExceptionError & ex )
            {
                return (long)ex.GetErrCode();
            }
            catch(...)
            {
                return -1;
            }
        }

		nRet = m_pDB->Initialize( sDbPath, sDbName, nKey, bRunSave, nSize, nSaveCount, nSaveSeconds );
	}

	trace.SetRetValue( nRet );
	return nRet;
}


bool CWaterMarkDB::Uninit()
{
    if ( !m_bUseDb )
    {
        return true;
	}

    if( !m_pDB )
    {
        return true;
    }

	return m_pDB->Uninitialize();
}


long CWaterMarkDB::Set(const char* sFilePath, const char* sWm) throw( CExceptionError )
{
	string sParam;
	sParam = sParam
		+ "sFilePath[" + ( sFilePath != NULL ? sFilePath : "NULL" ) + "]"
		+ "sWm[" + ( sWm != NULL ? sWm : "NULL" ) + "]";
	
	CGDTrace trace( LOG_L6, "CWaterMarkDB:Set(%s)", sParam.c_str() );

	// by liu_yfeng 2009-03-11 Ƿˮӡж
	if ( !m_bUseDb )
	{
		return 0;
	}
	if ( NULL == sFilePath || NULL == sWm || '\0' == *sFilePath )
	{
		return -1;
	}

	return m_pDB->SetData( sFilePath, sWm );
}

long CWaterMarkDB::Get(const char* sFilePath, char * sWm) throw( CExceptionError )
{
	CGDTrace trace( LOG_L6, "CWaterMarkDB:Get(%s)", sFilePath==NULL?"(null)":sFilePath );

	long nRet = 0;
	// by liu_yfeng 2009-03-11 Ƿˮӡж
	if ( !m_bUseDb )
	{
		return 0;
	}

	if ( NULL == sFilePath || NULL == sWm || '\0' == *sFilePath )
	{
		nRet = 40404; // 40404 ʾûˮӡ
	}
	else
	{
		nRet = m_pDB->GetData( sFilePath, sWm );
	}

	return trace.SetRetValue( nRet );
}


bool CWaterMarkDB::Find(const char* sFilePath)
{
	CGDTrace trace( LOG_L6, "CWaterMarkDB:Find(%s)", sFilePath==NULL?"(null)":sFilePath );

	long nRet = 0;
	// by liu_yfeng 2009-03-11 Ƿˮӡж
	if ( !m_bUseDb )
	{
		return false;
	}

	if ( NULL == sFilePath || '\0' == *sFilePath )
	{
		nRet = -1;
	}
	else
	{
		char sWM[ WMARK_LEN ] = {0};
		nRet = m_pDB->GetData( sFilePath, sWM );
	}

	return trace.SetRetResult( (nRet == 0) );
}

long CWaterMarkDB::Add( const char* sFilePath ) throw( CExceptionError )
{
	CGDTrace trace( LOG_L6, "CWaterMarkDB:Add(%s)", sFilePath==NULL?"(null)":sFilePath );
//	printf("func add sFilePath is %s\n",sFilePath);
	long nRet = 0;
	// by liu_yfeng 2009-03-11 Ƿˮӡж
	if ( !m_bUseDb )
	{
		return 0;
	}

	if ( NULL == sFilePath || '\0' == *sFilePath )
	{
		nRet = -1;
	}
	else
	{
		try
		{
			bool bFolder = true;
			if ( CFileOpt::IsExist ( sFilePath, &bFolder ) ) // жǷ
			{
				if ( bFolder == false ) // ļ
				{
					char sWM[ WMARK_LEN ] = {0};
					nRet = CWaterMarkDB::BuildWaterMark( sFilePath, sWM ); //ˮӡ
					if ( nRet == 0 ) //ˮӡɹ
					{
						printf("file OK!\n");
						nRet = m_pDB->SetData( sFilePath, sWM );
					}
					else
					{
						printf("file fail!\n");
						nRet = -2; // ˮӡɹ
					}
				}
				else // Ŀ¼
				{
					CFileList objFileList;
					FileList fileList;
					nRet = objFileList.CreateList( sFilePath, fileList ); //ȡĿ¼µļб
					
					for ( FileList::iterator iter = fileList.begin(); iter != fileList.end(); ++iter )  
					{					
						if ( !(*iter).isfolder ) // Ϊļ
						{
							char sWM[ WMARK_LEN ] = {0};
							nRet = CWaterMarkDB::BuildWaterMark( (*iter).name.c_str(), sWM ); //ˮӡ
							if ( nRet == 0 ) //ˮӡɹ
							{
								nRet = m_pDB->SetData( (*iter).name.c_str(), sWM );
							}
							else
							{
								nRet = -2; // ˮӡɹ
							}
						}
						else
						{
							nRet = m_pDB->SetData( (*iter).name.c_str(), NULL ); // ΪĿ¼ˮӡΪָ
						}
					}// for
				} // if
			}
			else //
			{
				nRet = -3; // ļ
			}
		}
		catch ( CExceptionError& )
		{
			nRet = -4; // 쳣
		}
	}

	return trace.SetRetValue( nRet );
}



long CWaterMarkDB::Rename(const char* sOldFilePath, const char* sNewFilePath) throw( CExceptionError )
{
	CGDTrace trace( LOG_L6, "CWaterMarkDB:Rename(%s,%s)", 
		sOldFilePath==NULL?"(null)":sOldFilePath, 
		sNewFilePath==NULL?"(null)":sNewFilePath );

	long nRet = 0;
	// by liu_yfeng 2009-03-11 Ƿˮӡж
	if ( !m_bUseDb )
	{
		return 0;
	}

	if ( NULL == sOldFilePath || NULL == sNewFilePath || '\0' == *sOldFilePath || '\0' == *sNewFilePath )
	{
		nRet = -1;
	}
	else
	{
		nRet = m_pDB->RenameNode( sOldFilePath, GetNameFromPath( sNewFilePath ).c_str() ) ? 0 : 1;
	}

	return trace.SetRetValue( nRet );
}

long CWaterMarkDB::Remove(const char* sFilePath) throw( CExceptionError )
{
	CGDTrace trace( LOG_L6, "CWaterMarkDB:Remove(%s)", sFilePath==NULL?"(null)":sFilePath );

	long nRet = 0;
	// by liu_yfeng 2009-03-11 Ƿˮӡж
	if ( !m_bUseDb )
	{
		return 0;
	}

	if ( NULL == sFilePath || '\0' == *sFilePath )
	{
		nRet = -1;
	}
	else
	{
		nRet = m_pDB->RemoveData( sFilePath ) ? 0 : 1;
	}

	return trace.SetRetValue( nRet );
}

long CWaterMarkDB::BuildWaterMark(const char* sFileName, char* sWm) throw( CExceptionError )
{
	CGDTrace trace( LOG_L6, "CWaterMarkOpt:BuildWaterMark(%s,)", sFileName==NULL?"(null)":sFileName );

	if ( NULL == sFileName || '\0' == *sFileName )
	{
		return trace.SetRetValue( -1 );
	}

	unsigned char buf[FILEREADBUFLEN] = {0};
	int nLen = 0;
	long nRet = 0;
	long nRestSign = 1;

	FILE* fp = CGDFile::open(sFileName, "rb");
	if (fp == NULL)
	{
		ErrCode errCode = GetLastError();
		CExceptionError ex;
		ex.AddErrText("%s",GetErrText(errCode));              //wu_binǰ̨Ϣ

		ex.AddErrText( "CWaterMarkOpt:BuildWaterMark(%s) - CGDFile:open() [%d] [%s]", sFileName, errCode, GetErrText( errCode ) ); 
		throw ex;
	}

	GDMD5 md5;

	int nCount = ( (1024 * 1024 * 2 ) / FILEREADBUFLEN );	// ļ2MBֻ2MBˮӡ

	do
	{
		::memset(&buf, 0, FILEREADBUFLEN);
		nLen = ::fread(buf, 1, FILEREADBUFLEN, fp);

		if (nLen >= 0)
		{
			md5.update(buf, nLen);

			if (nRestSign++ == UPDATEMD5RESTSIGN)
			{
				GDMINWAIT(); // cpuռ
				nRestSign = 1;
			}
			nCount--;
			if ( nCount <= 0 )	// ļ
			{
				// ȡļС
				char fileSizeTemp[32] = {0};
#ifdef WIN32
				WIN32_FILE_ATTRIBUTE_DATA fad;
				if (GetFileAttributesEx( sFileName, GetFileExInfoStandard, &fad ) != 0)
				{
					LARGE_INTEGER li;
					li.LowPart  = fad.nFileSizeLow;
					li.HighPart = fad.nFileSizeHigh;
					sprintf( fileSizeTemp, "%I64d", li.QuadPart );
				}
#else
				struct stat st;
				if (stat(sFileName, &st) == 0)
				{
					sprintf( fileSizeTemp, "%lld", st.st_size );
				}
#endif
				md5.update( (unsigned char*) fileSizeTemp, strlen( fileSizeTemp ) );
				break;
			}
		}
	} while ( nLen > 0 );

	fclose(fp);

	string strTemp = DigestEngine::digestToHex(md5.digest());
	strncpy( sWm, strTemp.c_str(), strTemp.length() );

	trace.SetOutParam( ",%s", sWm );
	return trace.SetRetValue( nRet );
}


void CWaterMarkDB::PrintAllRecord()
{
}


// 
string CWaterMarkDB::GetNameFromPath( const char* pathName )
{
	CGDTrace trace( LOG_L6, "CWaterMarkOpt:GetNameFromPath(%s,)", pathName );
	// by liu_yfeng 2009-03-11 Ƿˮӡж
	if ( !m_bUseDb )
	{
		return "";
	}

	string name = pathName;
	string::size_type nCur = name.find_last_of( S_PATHLASH );
	
	if ( nCur != string::npos )
	{
		string str = name.substr( nCur + 1 );
		trace.SetRetText( str.c_str() );
		return str;
	}
	trace.SetRetText( name.c_str() );
	return name;
}

