#include "articlelistpage.h"
#include "imagebutton.h"
#include <QLabel>
#include <QPalette>
#include <QFont>
#include <QPushButton>
#include <QMap>
#include <QtCore/qmath.h>
#include <QDebug>
#include <QScrollBar>

ArticleListPage* ArticleListPage::articleListInstance = NULL;

ArticleListPage::ArticleListPage(QObject *parent) : QObject(parent)
{
    middle = Middle::getMiddleInstance();
    articlePage = ArticlePage::getArticleInstance();
    szzb = SzzbPage::getSzzbInstance();
    numsOfPage = 10;
}

ArticleListPage* ArticleListPage::getArticleListInstance()
{
    if (articleListInstance == NULL) {
        articleListInstance = new ArticleListPage();
    }

    return articleListInstance;
}

void ArticleListPage::setWidget(QWidget *articleList, QStackedWidget *stack, QWidget *next, QWidget *szb)
{
    articleListPage = articleList;
    stackWidget = stack;
    nextPage = next;
    szbPage = szb;

    topImage = new QLabel(articleListPage);
    topImage->hide();
    leftView = new QListView(articleListPage);
    leftView->hide();
    rightView = new QListView(articleListPage);
    rightView->hide();
    btnBack = new ImageButton(articleListPage);
    btnBack->hide();
    btnSzb = new ImageButton(articleListPage);
    btnSzb->hide();
    btnPreP = new ImageButton(articleListPage);
    btnPreP->hide();
    btnNextP = new ImageButton(articleListPage);
    btnNextP->hide();

    connect(leftView,SIGNAL(clicked(QModelIndex)),this,SLOT(leftItemClicked(QModelIndex)));
    connect(rightView,SIGNAL(clicked(QModelIndex)),this,SLOT(rightItemClicked(QModelIndex)));

    connect(btnSzb, SIGNAL(clicked()), this, SLOT(onSzbClick()));
    connect(btnBack, SIGNAL(clicked()), this, SLOT(onBackClick()));
    connect(btnPreP, SIGNAL(clicked()), this, SLOT(onPrePageClicked()));
    connect(btnNextP, SIGNAL(clicked()), this, SLOT(onNextPageClicked()));
}

void ArticleListPage::setArticleListPage(QWidget *parent, QString grade, QString level, ImageButton *btnP, QString type)
{
    parentPage = parent;
    btnParent = btnP;
    grade_ = grade;
    level_ = level;
    stop = false;
    curentPage = 1;
    type_ = type;
    // 设置背景
    setBackGround(type);
    befSetArticleList(type);
    setArticleList(1);
    setButtons();
}

void ArticleListPage::setBackGround(QString type)
{
    QString bgImageFile, topImageFile;

    bgImageFile = "QWidget#articleListPage{image:url(:/nianji/shouye/bj_01.jpg)}";
    articleListPage->setStyleSheet(bgImageFile);

    topImage->setGeometry(387,34,253,57);
    if (type == "dbxz")
        topImageFile = "image:url(:/nianji/shouye/dbxz_top.png)";
    if (type == "xbxz")
        topImageFile = "image:url(:/nianji/shouye/xbxz_top.png)";
    if (type == "mbkt")
        topImageFile = "image:url(:/mbkt/mbkt_top.png)";

    topImage->setStyleSheet(topImageFile);
    topImage->show();
}

void ArticleListPage::befSetArticleList(QString type)
{
    int count;
    int p1, p2;

    articleList = middle->getCourseList(type);
    count = articleList.count();
    if (count < 1) {
        qDebug() << "Error article list count:" << count;
        return;
    }

    // 每页最多显示28篇课文
    // 计算需要多少页 page
    p1 = count / numsOfPage;
    p2 = count % numsOfPage;
    if (p2 > 0)
        pages = p1 + 1;
    else
        pages = p1;
}

// 设置文章列表
void ArticleListPage::setArticleList(int pageNum)
{
    QStringList leftList, rightList;
    int count;
    int start , end;

    count = articleList.count();
    if (count < 1) {
        qDebug() << "Error article list count:" << count;
        return;
    }

    if (pageNum < 0 || pageNum > pages) {
        qDebug() << "pageNum 无效 :" << pageNum;
        return;
    }

    start = (pageNum-1)*numsOfPage;
    end = qMin(pageNum*numsOfPage, count);

    half = qCeil(double(end-start)/2);
    qDebug() << "start" << start << "end" << end;
    qDebug() << "article list half:" << half;

    articleInfo.clear();
    for (int i=start; i<start+half; i++) {
        if (stop)
            return;
        QVariant tmp = articleList[i];
        leftList.insert(i, tmp.toMap()["name"].toString());

        QList<QString> info;
        info.append(tmp.toMap()["catalog_code"].toString());
        articleInfo.append(info);
    }

    for (int j=start+half; j<end; j++) {
        if (stop)
            return;
        QVariant tmp = articleList[j];
        rightList.insert(j, tmp.toMap()["name"].toString());

        QList<QString> info;
        info.append(tmp.toMap()["catalog_code"].toString());
        articleInfo.append(info);
    }

    setListView(leftList, rightList);
}

void ArticleListPage::setListView(QStringList leftList, QStringList rightList)
{
    QStringListModel *leftListModel = new QStringListModel(leftList);
    QStringListModel *righeListModel = new QStringListModel(rightList);

    leftView->setModel(leftListModel);
    leftView->setGeometry(270,120,250,500);
    leftView->setSpacing(5);
    leftView->setCursor(QCursor(Qt::PointingHandCursor));
    leftView->setEditTriggers(QAbstractItemView::NoEditTriggers);
    leftView->verticalScrollBar()->setStyleSheet("QScrollBar:vertical{width:6px;background:transparent;background-color:rgb(248, 248, 248);margin:0px,0px,0px,0px;padding-top:10px;padding-bottom:10px;}"
                       "QScrollBar::handle:vertical{width:6px;background:lightgray ;border-radius:5px;min-height:20px;}"
                       "QScrollBar::handle:vertical:hover{width:6px;background:gray;border-radius:5px;min-height:20px;}"
                       "QScrollBar::add-line:vertical{height:10px;width:10px;border-image:url(:/button/images/button/down.png);subcontrol-position:bottom;}"
                       "QScrollBar::sub-line:vertical{height:10px;width:10px;border-image:url(:/button/images/button/up.png);subcontrol-position:top;}"
                       "QScrollBar::add-line:vertical:hover{height:10px;width:10px;border-image:url(:/button/images/button/down_mouseDown.png);subcontrol-position:bottom;}"
                       "QScrollBar::sub-line:vertical:hover{height:10px;width:10px;border-image:url(:/button/images/button/up_mouseDown.png);subcontrol-position:top;}"
                       "QScrollBar::add-page:vertical,QScrollBar::sub-page:vertical{background:transparent;border-radius:5px;}");
    leftView->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    // 去边框
    leftView->setFrameStyle(0);
    // 设置样式
    leftView->setStyleSheet("QListView{font-size: 20px; font-family:宋体}"
                            "QListView::item:selected:!active{background-color: transparent}"
                            "QListView::item:hover:active{background-color: rgb(251,151,5)}"
                            "QListView::item:hover{background-color: rgb(251,151,5)}"
                            "QListView{background-color:transparent}");
    leftView->show();

    rightView->setModel(righeListModel);
    rightView->setGeometry(520,120,250,500);
    rightView->setSpacing(5);
    rightView->setCursor(QCursor(Qt::PointingHandCursor));
    rightView->setEditTriggers(QAbstractItemView::NoEditTriggers);
    // 去掉滚动条
    rightView->verticalScrollBar()->setStyleSheet("QScrollBar:vertical{width:6px;background:transparent;background-color:rgb(248, 248, 248);margin:0px,0px,0px,0px;padding-top:10px;padding-bottom:10px;}"
                       "QScrollBar::handle:vertical{width:6px;background:lightgray ;border-radius:5px;min-height:20px;}"
                       "QScrollBar::handle:vertical:hover{width:6px;background:gray;border-radius:5px;min-height:20px;}"
                       "QScrollBar::add-line:vertical{height:10px;width:10px;border-image:url(:/button/images/button/down.png);subcontrol-position:bottom;}"
                       "QScrollBar::sub-line:vertical{height:10px;width:10px;border-image:url(:/button/images/button/up.png);subcontrol-position:top;}"
                       "QScrollBar::add-line:vertical:hover{height:10px;width:10px;border-image:url(:/button/images/button/down_mouseDown.png);subcontrol-position:bottom;}"
                       "QScrollBar::sub-line:vertical:hover{height:10px;width:10px;border-image:url(:/button/images/button/up_mouseDown.png);subcontrol-position:top;}"
                       "QScrollBar::add-page:vertical,QScrollBar::sub-page:vertical{background:transparent;border-radius:5px;}");
//    rightView->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    rightView->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    // 去边框
    rightView->setFrameStyle(0);
    // 设置样式
    rightView->setStyleSheet("QListView{font-size: 20px; font-family:宋体}"
                             "QListView::item:selected:!active{background-color: transparent}"
                             "QListView::item:hover:active{background-color: rgb(251,151,5)}"
                             "QListView::item:hover{background-color: rgb(251,151,5)}"
                             "QListView{background-color:transparent}");
    rightView->show();
}

void ArticleListPage::setButtons()
{
    // 生字表
    btnSzb->setGeometry(755,168,142,89);
//    btnSzb->show();
    QPixmap szbP1(":/nianji/shouye/nj_szb01.png");
    QPixmap szbP2(":/nianji/shouye/nj_szb02.png");
    btnSzb->setNormalImage(szbP1);
    btnSzb->setPressImage(szbP2);
    btnSzb->setCursor(QCursor(Qt::PointingHandCursor));

    // 返回
    btnBack->setGeometry(769,528,140,108);
    btnBack->show();
    QPixmap backP1(":/nianji/shouye/fh_01.png");
    QPixmap backP2(":/nianji/shouye/fh_02.png");
    btnBack->setNormalImage(backP1);
    btnBack->setPressImage(backP2);
    btnBack->setCursor(QCursor(Qt::PointingHandCursor));

    // 上一页
    btnPreP->setGeometry(848,290,57,57);
    QPixmap preP1(":/nianji/shouye/jtl_01.png");
    QPixmap preP2(":/nianji/shouye/jtl_02.png");
    btnPreP->setNormalImage(preP1);
    btnPreP->setPressImage(preP2);
    btnPreP->setCursor(QCursor(Qt::PointingHandCursor));

    // 下一页
    btnNextP->setGeometry(905,290,57,57);
    QPixmap nextP1(":/nianji/shouye/jtr_01.png");
    QPixmap nextP2(":/nianji/shouye/jtr_02.png");
    btnNextP->setNormalImage(nextP1);
    btnNextP->setPressImage(nextP2);
    btnNextP->setCursor(QCursor(Qt::PointingHandCursor));
    if (pages > 1)
        btnNextP->show();
}

void ArticleListPage::onSzbClick()
{
    btnSzb->blockSignals(true);
    stackWidget->setCurrentWidget(szbPage);
    szzb->setSzzbPage(articleListPage, btnSzb, grade_, level_);
}

void ArticleListPage::onBackClick()
{
    stop = true;
    btnParent->blockSignals(false);
    stackWidget->setCurrentWidget(parentPage);
    QStringListModel *listModel = new QStringListModel(QStringList{});
    leftView->setModel(listModel);
    rightView->setModel(listModel);
    btnPreP->hide();
    btnNextP->hide();
}

void ArticleListPage::leftItemClicked(QModelIndex index)
{
    leftView->blockSignals(true);
    int i = index.row();
    QString title = index.data().toString();

    // 跳转下一页
    stackWidget->setCurrentWidget(nextPage);
    articlePage->setArticlePage(articleListPage, articleInfo[i], title, leftView, type_);
}

void ArticleListPage::rightItemClicked(QModelIndex index)
{
    rightView->blockSignals(true);
    int i = index.row() + half;
    QString title = index.data().toString();

    // 跳转下一页
    stackWidget->setCurrentWidget(nextPage);
    articlePage->setArticlePage(articleListPage, articleInfo[i], title, rightView, type_);
}

void ArticleListPage::onPrePageClicked()
{
    curentPage--;
    btnNextP->show();

    if (curentPage == 1)
        btnPreP->hide();

    setArticleList(curentPage);
}

void ArticleListPage::onNextPageClicked()
{
    curentPage++;
    btnPreP->show();

    if (curentPage == pages)
        btnNextP->hide();

    setArticleList(curentPage);
}
