#include "articlesubpage.h"
#include <QDebug>

ArticleSubPage* ArticleSubPage::articleSubInstance = NULL;

ArticleSubPage::ArticleSubPage(QObject *parent) : QObject(parent)
{
    middle = Middle::getMiddleInstance();
    wordPage = WordPage::getWordPageInstance();
}

ArticleSubPage* ArticleSubPage::getArticleSubPageInstance()
{
    if (articleSubInstance == NULL) {
        articleSubInstance = new ArticleSubPage();
    }

    return articleSubInstance;
}

void ArticleSubPage::setWidget(QWidget *articleSub, QStackedWidget *stack, QWidget *next)
{
    articleSubPage = articleSub;
    stackWidget = stack;
    nextPage = next;

    titleText = new QLabel(articleSubPage);
    titleText->hide();
    btnBack = new ImageButton(articleSubPage);
    btnBack->hide();
    topImage = new QLabel(articleSubPage);
    topImage->hide();
    imageList = new QListWidget(articleSubPage);
    imageList->hide();

    connect(btnBack, SIGNAL(clicked()), this, SLOT(onBackClick()));
    connect(imageList,SIGNAL(clicked(QModelIndex)),this,SLOT(itemClicked(QModelIndex)));
}

void ArticleSubPage::setArticleSubPage(QWidget *parent, QList<QString> info, QString title_, QListView *listViewP, QString type)
{
    parentPage = parent;
    listViewParent = listViewP;
    QString flag, video, code;
    int videoSize;

    if (info.count() == 4) {
        flag = info[0];
        video = info[1];
        videoSize = info[2].toInt();
        code = info[3];
        qDebug() << "flag:" << flag << "video:" << video << "videoSize" << videoSize << "code:" << code;
    }

    setBackGround(title_, type);
    setButtons();

    if (flag == "2") {
        setWordList(code);
    }
}

void ArticleSubPage::setBackGround(QString title, QString type)
{
    QFont font;
    QPalette pa;
    QString topImageFile;

    pa.setColor(QPalette::WindowText,QColor(251,151,5));
    font.setFamily("宋体");
    font.setPixelSize(35);
    font.setLetterSpacing(QFont::AbsoluteSpacing,1);

    articleSubPage->setStyleSheet("QWidget#articleSubPage{image:url(:/nianji/shouye/bj_01.jpg)}");

    titleText->setText(title);
    titleText->setAlignment(Qt::AlignCenter);
    titleText->setFont(font);
    titleText->setGeometry(0,115,1000,50);
    titleText->show();

    topImage->setGeometry(387,34,253,57);
    if (type == "dbxz")
        topImageFile = "image:url(:/nianji/shouye/dbxz_top.png)";
    if (type == "xbxz")
        topImageFile = "image:url(:/nianji/shouye/xbxz_top.png)";
    if (type == "mbkt")
        topImageFile = "image:url(:/mbkt/mbkt_top.png)";

    topImage->setStyleSheet(topImageFile);
    topImage->show();
}

void ArticleSubPage::setWordList(QString code)
{
    QList<QVariant> wordList;
    QFont font;

    font.setFamily("华文楷体");
    font.setPixelSize(70);
    font.setLetterSpacing(QFont::AbsoluteSpacing,1);

    imageList->clear();
    imageList->show();
    imageList->setFont(font);
    imageList->setSelectionMode(QAbstractItemView::MultiSelection);
    imageList->setGeometry(315,220,440,400);
    //设置QListWidget中单元项的间距
    imageList->setSpacing(15);
    //设置QListWidget的显示模式
    imageList->setViewMode(QListView::IconMode);
    //设置自动适应布局调整（Adjust适应，Fixed不适应），默认不适应
    imageList->setResizeMode(QListWidget::Adjust);
    //设置不能移动
    imageList->setMovement(QListWidget::Static);
    //去掉边框
    imageList->setFrameStyle(0);
    // 去掉滚动条
    imageList->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    imageList->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    imageList->setCursor(QCursor(Qt::PointingHandCursor));

    imageList->setStyleSheet("QListWidget::item:hover{background-color: rgb(251,151,5);border-color:rgb(251,151,5)}"
                               "QListWidget::item:selected{background:lightgray;}"
                               "QListWidget::item{border-width: 2px; border-style: solid; border-color: rgb(243,112,94);}"
                               "QListWidget{background-color:transparent}");

    wordList = middle->getLectureWordList(code);
    foreach (QVariant word, wordList) {
        QListWidgetItem *item = new QListWidgetItem;
        item->setData(Qt::WhatsThisRole, word.toMap()["zi"].toString());
        item->setText(word.toMap()["zi"].toString());
        item->setSizeHint(QSize(120,120));
        imageList->addItem(item);
    }
}

void ArticleSubPage::setButtons()
{
    btnBack->setGeometry(855,520,109,101);
    btnBack->show();
    QPixmap backP1(":/nianji/yiji/fh_03.png");
    QPixmap backP2(":/nianji/yiji/fh_04.png");
    btnBack->setNormalImage(backP1);
    btnBack->setPressImage(backP2);
    btnBack->setCursor(QCursor(Qt::PointingHandCursor));
}

void ArticleSubPage::onBackClick()
{
    listViewParent->blockSignals(false);
    listViewParent->clearSelection();
    stackWidget->setCurrentWidget(parentPage);
    titleText->clear();
    imageList->clear();
    imageList->hide();
}

void ArticleSubPage::itemClicked(QModelIndex index)
{
    imageList->blockSignals(true);
    QString word = index.data(Qt::WhatsThisRole).toString();

    wordPage->setWordPage(articleSubPage, word, imageList, "", "");
    stackWidget->setCurrentWidget(nextPage);
}
