#include "jbbhsubpage.h"
#include <QFont>
#include <QDebug>
#include <QScrollBar>
#include <windows.h>

JbbhSubPage* JbbhSubPage::jbbhSubInstance = NULL;

JbbhSubPage::JbbhSubPage(QObject *parent) : QObject(parent)
{
    middle = Middle::getMiddleInstance();
    jbbhVideo = JbbhVideoPage::getJbbhVideoPageInstance();
}

JbbhSubPage* JbbhSubPage::getJbbhSubInstance()
{
    if (jbbhSubInstance == NULL)
        jbbhSubInstance = new JbbhSubPage();

    return jbbhSubInstance;
}

void JbbhSubPage::setWidget(QWidget *jbbhSub, QStackedWidget *stack, QWidget *next)
{
    jbbhSubPage = jbbhSub;
    nextPage = next;
    stackWidget = stack;

    topImage = new QLabel(jbbhSubPage);
    topImage->hide();
    bihuaText = new QLabel(jbbhSubPage);
    bihuaText->hide();
    btnBack = new ImageButton(jbbhSubPage);
    btnBack->hide();
    imageList = new QListWidget(jbbhSubPage);
    imageList->hide();

    connect(btnBack, SIGNAL(clicked()), this, SLOT(onBackClicked()));
    connect(imageList,SIGNAL(clicked(QModelIndex)),this,SLOT(itemClicked(QModelIndex)));
}

void JbbhSubPage::setJbbhSubPage(QWidget *parent, QString group_code, QString bihua, ImageButton *btnP)
{
    parentPage = parent;
    btnParent = btnP;
    QList<QVariant> bihuaList;
    stop = false;

    bihuaList = middle->getStroke(group_code);

    setBackGround(bihua);
    setButtons();
    setListView(bihuaList);
}

void JbbhSubPage::setBackGround(QString bihua)
{
    QFont font;
    QPalette pa;

    pa.setColor(QPalette::WindowText,QColor(255,255,255));
    font.setFamily("宋体");
    font.setPixelSize(60);
    font.setLetterSpacing(QFont::AbsoluteSpacing,1);
    font.setBold(true);

    jbbhSubPage->setStyleSheet("QWidget#jbbhSubPage{image:url(:/jbbh/bj_07.png)}");

    topImage->setGeometry(352,47,290,87);
    topImage->show();
    topImage->setStyleSheet("image:url(:/jbbh/jbbh_bt.png)");

    bihuaText->setGeometry(777,45,100,100);
    bihuaText->show();
    bihuaText->setText(bihua);
    bihuaText->setFont(font);
    bihuaText->setPalette(pa);
}

void JbbhSubPage::setButtons()
{
    // 返回
    btnBack->setGeometry(686,440,248,208);
    btnBack->show();
    QPixmap backP1(":/jbbh/fh_05.png");
    QPixmap backP2(":/jbbh/fh_06.png");
    btnBack->setNormalImage(backP1);
    btnBack->setPressImage(backP2);
    btnBack->setCursor(QCursor(Qt::PointingHandCursor));
}

void JbbhSubPage::setListView(QList<QVariant> bihuaList)
{
    QFont font;

    font.setFamily("楷体");
    font.setPixelSize(12);
    font.setLetterSpacing(QFont::AbsoluteSpacing,1);
//    font.setBold(true);

    imageList->clear();
    imageList->setGeometry(205,180,600,260);
    imageList->show();
    //设置QListWidget的显示模式
    imageList->setViewMode(QListView::IconMode);
    //设置QListWidget中单元项的图片大小
    imageList->setIconSize(QSize(80,110));
    //设置QListWidget中单元项的间距
    imageList->setSpacing(12);
    //设置自动适应布局调整（Adjust适应，Fixed不适应），默认不适应
    imageList->setResizeMode(QListWidget::Adjust);
    //设置不能移动
    imageList->setMovement(QListWidget::Static);
    //去掉边框
    imageList->setFrameStyle(0);
    //设置鼠标样式
    imageList->setCursor(QCursor(Qt::PointingHandCursor));
    imageList->setFocusPolicy(Qt::NoFocus);
//    imageList->setEditTriggers(QAbstractItemView::NoEditTriggers);
    // 去掉滚动条
    imageList->verticalScrollBar()->setStyleSheet("QScrollBar:vertical{width:6px;background:transparent;background-color:rgb(248, 248, 248);margin:0px,0px,0px,0px;padding-top:10px;padding-bottom:10px;}"
                       "QScrollBar::handle:vertical{width:6px;background:lightgray ;border-radius:5px;min-height:20px;}"
                       "QScrollBar::handle:vertical:hover{width:6px;background:gray;border-radius:5px;min-height:20px;}"
                       "QScrollBar::add-line:vertical{height:10px;width:10px;border-image:url(:/button/images/button/down.png);subcontrol-position:bottom;}"
                       "QScrollBar::sub-line:vertical{height:10px;width:10px;border-image:url(:/button/images/button/up.png);subcontrol-position:top;}"
                       "QScrollBar::add-line:vertical:hover{height:10px;width:10px;border-image:url(:/button/images/button/down_mouseDown.png);subcontrol-position:bottom;}"
                       "QScrollBar::sub-line:vertical:hover{height:10px;width:10px;border-image:url(:/button/images/button/up_mouseDown.png);subcontrol-position:top;}"
                       "QScrollBar::add-page:vertical,QScrollBar::sub-page:vertical{background:transparent;border-radius:5px;}");
//    imageList->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    imageList->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    //去掉选中的背景框
    imageList->setSelectionMode(QAbstractItemView::NoSelection);
    imageList->setFocusPolicy(Qt::NoFocus);

    imageList->blockSignals(true);
    bihuaInfo.clear();
    foreach (QVariant bihua, bihuaList) {
        if (stop)
            return;
        QList<QString> info;
        QString fbVideo, qbVideo, mbVideo;
        QString fbVideoSize, qbVideoSize, mbVideoSize;

        fbVideo = bihua.toMap()["fb_video_url"].toString();
        qbVideo = bihua.toMap()["gb_video_url"].toString();
        mbVideo = bihua.toMap()["mb_video_url"].toString();
        fbVideoSize = bihua.toMap()["fb_video_size"].toString();
        qbVideoSize = bihua.toMap()["gb_video_size"].toString();
        mbVideoSize = bihua.toMap()["mb_video_size"].toString();

        info.append(bihua.toMap()["content_url"].toString());
        info.append(bihua.toMap()["note"].toString());
        info.append(bihua.toMap()["feature"].toString());
        info.append(fbVideo);
        info.append(qbVideo);
        info.append(mbVideo);
        info.append(fbVideoSize);
        info.append(qbVideoSize);
        info.append(mbVideoSize);

        bihuaInfo.append(info);

        //定义QListWidgetItem对象
        QListWidgetItem *imageItem = new QListWidgetItem;
        //为单元项设置属性
        QPixmap image;
        if (middle->getClientMode()) {
            image.load(middle->getImageFile2(bihua.toMap()["img_url"].toString(), "/strokes/"));
        } else {
            bool ret = image.loadFromData(middle->getImageData(bihua.toMap()["img_url"].toString()));
            if (!ret) {
                Sleep(1000);
                image.loadFromData(middle->getImageData(bihua.toMap()["img_url"].toString()));
            }
        }
//        image = image.scaled(QSize(80,80),Qt::KeepAspectRatio);
        imageItem->setIcon(QIcon(image));
        imageItem->setText(bihua.toMap()["title"].toString());
        imageItem->setFont(font);
        imageItem->setFlags(imageItem->flags() & ~ Qt::ItemIsSelectable);
        //设置文字位置
        imageItem->setTextAlignment(Qt::AlignTop | Qt::AlignHCenter);
//        imageItem->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
        //重新设置单元项图片的宽度和高度
        imageItem->setSizeHint(QSize(80,110));
        //将单元项添加到QListWidget中
        imageList->addItem(imageItem);
    }
    imageList->blockSignals(false);
}

void JbbhSubPage::onBackClicked()
{
    stop = true;
    imageList->clear();
    btnParent->blockSignals(false);
    stackWidget->setCurrentWidget(parentPage);
}

void JbbhSubPage::itemClicked(QModelIndex index)
{
    imageList->blockSignals(true);
    qDebug() << "jbbh item click" << index.row();

    jbbhVideo->setJbbhVideoPage(jbbhSubPage, bihuaInfo[index.row()], imageList);
    stackWidget->setCurrentWidget(nextPage);
}
