#include "jbbhvideopage.h"
#include <QPixmap>
#include <QStringList>
#include <QStringListModel>
#include <QListView>
#include <QDebug>
#include <QApplication>
#include <QDesktopWidget>
#include <QFont>

JbbhVideoPage* JbbhVideoPage::jbbhVideoInstance = NULL;

JbbhVideoPage::JbbhVideoPage(QWidget *parent) : QWidget(parent)
{
    middle = Middle::getMiddleInstance();
    isFullScreen = false;
    video = new Video();
//    connect(video, SIGNAL(endReached()), this, SLOT(onVideoEndReached()));
}

JbbhVideoPage* JbbhVideoPage::getJbbhVideoPageInstance()
{
    if (jbbhVideoInstance == NULL)
        jbbhVideoInstance = new JbbhVideoPage();

    return jbbhVideoInstance;
}

void JbbhVideoPage::setWidget(QWidget *jbbhVideo, QStackedWidget *stack, QWidget *root, MyLabel *video,
                              QSlider *slider, QLabel *timeLabel, QLabel *lenLabel, QSlider *_volSlider,
                              ImageButton *play, ImageButton *vol, ImageButton *full, QLabel *ctlRect)
{
    jbbhVideoPage = jbbhVideo;
    stackWidget = stack;
    videoLabel = video;
    rootWidget = root;

//    BgLabel = new QLabel(jbbhVideoPage);
    topImage = new QLabel(jbbhVideoPage);
    topImage->hide();
    bihuaImage = new QLabel(jbbhVideoPage);
    bihuaImage->hide();
    noteLabel = new QLabel(jbbhVideoPage);
    noteLabel->hide();
    featureView = new QListView(jbbhVideoPage);
    featureView->hide();
    btnFb = new ImageButton(jbbhVideoPage);
    btnFb->hide();
    btnQb = new ImageButton(jbbhVideoPage);
    btnQb->hide();
    btnMb = new ImageButton(jbbhVideoPage);
    btnMb->hide();
    btnBack = new ImageButton(jbbhVideoPage);
    btnBack->hide();

    posSlider = slider;
    timer = new QTimer();
    videoCtlTimer = new QTimer();
    curTimeLabel = timeLabel;
    lengthLabel = lenLabel;

    posSlider->installEventFilter(this);

    videoBgLabel = new QLabel(jbbhVideoPage);
    volSlider = _volSlider;
    btnPlay = play;
    btnVol = vol;
    btnFull = full;
    videoCtlRect = ctlRect;
    volSlider->installEventFilter(this);

    connect(btnFb, SIGNAL(clicked()), this, SLOT(onFbClick()));
    connect(btnQb, SIGNAL(clicked()), this, SLOT(onQbClick()));
    connect(btnMb, SIGNAL(clicked()), this, SLOT(onMbClick()));
    connect(btnBack, SIGNAL(clicked()), this, SLOT(onBackClick()));
    connect(videoLabel, SIGNAL(LabelDoubleClicked()), this, SLOT(onVideoDoubleClick()));
//    connect(videoLabel, SIGNAL(LabelClicked()), this, SLOT(onVideoClick()));
    connect(videoLabel, SIGNAL(LabelMouseMove()), this, SLOT(onMouseMove()));

    connect(posSlider, SIGNAL(sliderMoved(int)), this, SLOT(changePosition(int)));
//    connect(posSlider, SIGNAL(sliderPressed()), this, SLOT(onPressed()));
    connect(timer, SIGNAL(timeout()), this, SLOT(updateInfo()));
    connect(videoCtlTimer, SIGNAL(timeout()), this, SLOT(hideVideoCtl()));

    connect(btnPlay, SIGNAL(clicked()), this, SLOT(onVideoClick()));
    connect(btnFull, SIGNAL(clicked()), this, SLOT(onVideoDoubleClick()));
    connect(btnVol, SIGNAL(clicked()), this, SLOT(onVolClick()));
    connect(volSlider, SIGNAL(sliderMoved(int)), this, SLOT(onVolSliderMoved(int)));
}

void JbbhVideoPage::setJbbhVideoPage(QWidget *parent, QList<QString> bihuaInfo, QListWidget *listWidgetP)
{
    parentPage = parent;
    listWidgetParent = listWidgetP;

    //bihuaInfo <image, note, feature, fbVideo, qbVideo, mbVideo>
    if (bihuaInfo.count() != 9)
        return;

    fbVideo = bihuaInfo[3];
    qbVideo = bihuaInfo[4];
    mbVideo = bihuaInfo[5];
    fbVideoSize = bihuaInfo[6].toInt();
    qbVideoSize = bihuaInfo[7].toInt();
    mbVideoSize = bihuaInfo[8].toInt();

    setBackGround(bihuaInfo[0]);
    setText(bihuaInfo[1], bihuaInfo[2]);
    setButtons();
    setVideo();
}

void JbbhVideoPage::setBackGround(QString image)
{
    // 设置背景图片
//    BgLabel->setGeometry(0,0,1000,700);
//    BgLabel->setStyleSheet("image:url(:/jbbh/bj_18.jpg)");
    jbbhVideoPage->setStyleSheet("QWidget#jbbhVideoPage{image:url(:/jbbh/bj_18.jpg)}");

    topImage->setGeometry(695,42,284,85);
    topImage->show();
    topImage->setStyleSheet("image:url(:/jbbh/bhjj_bt.png)");

    bihuaImage->setGeometry(745,162,180,180);
    bihuaImage->show();
    QPixmap bihuaPix;
    if (middle->getClientMode()) {
        bihuaPix.load((middle->getImageFile2(image, "/strokes/")));
    } else {
        bihuaPix.loadFromData(middle->getImageData(image));
    }
    bihuaImage->setPixmap(bihuaPix);
}

void JbbhVideoPage::setText(QString note, QString feature)
{
    QFont font;
    QPalette pa;

    pa.setColor(QPalette::WindowText,QColor(255,255,255));
    font.setFamily("宋体");
    font.setPixelSize(25);
    font.setLetterSpacing(QFont::AbsoluteSpacing,1);
    font.setBold(true);

    noteLabel->setGeometry(740,357,200,40*2);
    noteLabel->show();
    noteLabel->setText(note);
//    noteLabel->setText("一二三->四五六七八九十十一");
    noteLabel->setWordWrap(true);
    noteLabel->setAlignment(Qt::AlignCenter);
    noteLabel->setFont(font);
    noteLabel->setPalette(pa);

    // 字形结构
    QStringList featrueList;
    featrueList = feature.split(",");
    QStringListModel *featureModel = new QStringListModel(featrueList);
    featureView->setModel(featureModel);
    featureView->setSpacing(3);
    featureView->setGeometry(784,432,120,80);
    featureView->show();
    featureView->setFocusPolicy(Qt::NoFocus);
    featureView->setEditTriggers(QAbstractItemView::NoEditTriggers);
    featureView->blockSignals(true);
    // 去边框
    featureView->setFrameStyle(0);
    // 设置样式
    featureView->setStyleSheet("QListView{font: bold; font-size: 25px; font-family: 宋体; color: white}"
                               "QListView::item:selected:!active{background-color: transparent; color:white}"
                               "QListView::item:selected:active{background-color: transparent; color:white}"
                               "QListView{background-color:transparent}");
}

void JbbhVideoPage::setButtons()
{
    // 粉笔视频
    btnFb->setGeometry(54,562,191,56);
//    btnFb->show();
    QPixmap fbP1(":/jbbh/bh_fbsf01.png");
    QPixmap fbP2(":/jbbh/bh_fbsf02.png");
    btnFb->setNormalImage(fbP1);
    btnFb->setPressImage(fbP2);
    btnFb->setCursor(QCursor(Qt::PointingHandCursor));

    // 硬笔视频
    btnQb->setGeometry(271,562,192,56);
//    btnQb->show();
    QPixmap qbP1(":/jbbh/bh_ybsf01.png");
    QPixmap qbP2(":/jbbh/bh_ybsf02.png");
    btnQb->setNormalImage(qbP1);
    btnQb->setPressImage(qbP2);
    btnQb->setCursor(QCursor(Qt::PointingHandCursor));

    // 毛笔视频
    btnMb->setGeometry(484,562,191,56);
//    btnMb->show();
    QPixmap mbP1(":/jbbh/bh_mbsf01.png");
    QPixmap mbP2(":/jbbh/bh_mbsf02.png");
    btnMb->setNormalImage(mbP1);
    btnMb->setPressImage(mbP2);
    btnMb->setCursor(QCursor(Qt::PointingHandCursor));

    // 返回
    btnBack->setGeometry(786,542,141,95);
    btnBack->show();
    QPixmap backP1(":/jbbh/fh_07.png");
    QPixmap backP2(":/jbbh/fh_08.png");
    btnBack->setNormalImage(backP1);
    btnBack->setPressImage(backP2);
    btnBack->setCursor(QCursor(Qt::PointingHandCursor));
}

void JbbhVideoPage::setVideo()
{
    QFont font;
    QPalette pa;
    QFont fontVideo;
    QPalette paVideo;

    font.setFamily("黑体");
    font.setPixelSize(15);
    pa.setColor(QPalette::WindowText,QColor(211,211,211));

    fontVideo.setFamily("楷体");
    fontVideo.setPixelSize(30);
    fontVideo.setLetterSpacing(QFont::AbsoluteSpacing,1);
    fontVideo.setBold(true);
    paVideo.setColor(QPalette::WindowText,QColor(2,173,237));

    initVideoCtl();
    showVideoCtl();
    videoStop = false;

    QString videoUrl = middle->getVideoPath("ggxz", fbVideo, fbVideoSize);
    videoLabel->show();
    videoLabel->setGeometry(stackWidget->x()+50,stackWidget->y()+86,640,435);
    videoLabel->setFont(fontVideo);
    videoLabel->setPalette(paVideo);
    videoLabel->setAlignment(Qt::AlignCenter);
//    video->loadVlc("c:\\work\\test2.wmv", (void*)videoLabel->winId());
    qDebug() << "fbVideo" << videoUrl;
//    videoUrl = "http://www.cguoguo.org.cn/jfx-web/attachment/吩.mp4";
    videoLabel->clear();
    if (fbVideo == "0") {
        videoLabel->setText("视频正在努力制作中，敬请期待！");
    }
    video->loadVlc(videoUrl, (void*)videoLabel->winId());
    video->setVolume(middle->getVolume());
    video->play();

    setVideoCtl();
    initVolCtl();
    middle->download(fbVideo, fbVideoSize);
}

void JbbhVideoPage::onFbClick()
{
    QString videoUrl = middle->getVideoPath("ggxz", fbVideo, fbVideoSize);

    videoLabel->show();
//    videoLabel->setGeometry(stackWidget->x()+50,stackWidget->y()+86,640,440);
    qDebug() << "fbVideo" << videoUrl;
    video->stop();
    videoLabel->clear();
    if (fbVideo == "0") {
        videoLabel->setText("视频正在努力制作中，敬请期待！");
    }
//    video->loadVlc("c:\\work\\test2.wmv", (void*)videoLabel->winId());
    video->loadVlc(videoUrl, (void*)videoLabel->winId());
    video->setVolume(middle->getVolume());
    initVolCtl();
    video->play();
    setPauseIcon();
    middle->download(fbVideo, fbVideoSize);
}

void JbbhVideoPage::onQbClick()
{
    QString videoUrl = middle->getVideoPath("ggxz", qbVideo, qbVideoSize);
    videoLabel->show();
//    videoLabel->setGeometry(stackWidget->x()+50,stackWidget->y()+86,640,440);
    qDebug() << "qbVideo" << videoUrl;
    video->stop();
    videoLabel->clear();
    if (qbVideo == "0") {
        videoLabel->setText("视频正在努力制作中，敬请期待！");
    }
//    video->loadVlc("c:\\work\\test2.wmv", (void*)videoLabel->winId());
    video->loadVlc(videoUrl, (void*)videoLabel->winId());
    video->setVolume(middle->getVolume());
    initVolCtl();
    video->play();
    setPauseIcon();
    middle->download(qbVideo, qbVideoSize);
}

void JbbhVideoPage::onMbClick()
{
    QString videoUrl = middle->getVideoPath("ggxz", mbVideo, mbVideoSize);
    videoLabel->show();
//    videoLabel->setGeometry(stackWidget->x()+50,stackWidget->y()+86,640,440);
    qDebug() << "mbVideo" << videoUrl;
    video->stop();
    videoLabel->clear();
    if (mbVideo == "0") {
        videoLabel->setText("视频正在努力制作中，敬请期待！");
    }
//    video->loadVlc("c:\\work\\test2.wmv", (void*)videoLabel->winId());
    video->loadVlc(videoUrl, (void*)videoLabel->winId());
    video->setVolume(middle->getVolume());
    initVolCtl();
    video->play();
    setPauseIcon();
    middle->download(mbVideo, mbVideoSize);
}

void JbbhVideoPage::onVideoDoubleClick()
{
    qDebug() << "video double click";
    if (isFullScreen) {
        isFullScreen = false;
        emit showNormal();

        videoLabel->setGeometry(stackWidget->x()+50,stackWidget->y()+86,640,435);
        videoLabel->show();

        showVideoCtl();
        posSlider->setGeometry(stackWidget->x()+55,stackWidget->y()+526,630,8);
        curTimeLabel->setGeometry(stackWidget->x()+97,stackWidget->y()+546,80,20);
        lengthLabel->setGeometry(stackWidget->x()+182,stackWidget->y()+546,100,20);
        btnPlay->setGeometry(stackWidget->x()+65,stackWidget->y()+546,18,20);
        btnVol->setGeometry(stackWidget->x()+597,stackWidget->y()+521,20,18);
        btnFull->setGeometry(stackWidget->x()+647,stackWidget->y()+546,22,16);
        volSlider->setGeometry(stackWidget->x()+622,stackWidget->y()+526,60,8);

        if (videoCtlTimer->isActive()) {
            videoCtlTimer->stop();
        }
    } else {
        emit fullScreen();
        isFullScreen = true;
        videoLabel->resize(rootWidget->width(),QApplication::desktop()->height());
        videoLabel->move(0,0);
        videoLabel->show();

        posSlider->setGeometry(10,QApplication::desktop()->height()-60,rootWidget->width()-20,8);
        curTimeLabel->setGeometry(60,QApplication::desktop()->height()-40,80,20);
        lengthLabel->setGeometry(145,QApplication::desktop()->height()-40,100,20);
        btnPlay->setGeometry(30,QApplication::desktop()->height()-40,18,20);
        btnVol->setGeometry(rootWidget->width()-120,QApplication::desktop()->height()-65,20,18);
        btnFull->setGeometry(rootWidget->width()-70,QApplication::desktop()->height()-40,22,16);
        volSlider->setGeometry(rootWidget->width()-90,QApplication::desktop()->height()-60,60,8);
        videoCtlRect->setGeometry(0,QApplication::desktop()->height()-65,rootWidget->width(),65);
        videoCtlRect->show();

        videoCtlRect->raise();
        posSlider->raise();
        curTimeLabel->raise();
        lengthLabel->raise();
        btnPlay->raise();
        btnVol->raise();
        btnFull->raise();
        volSlider->raise();

        if (videoCtlTimer->isActive()) {
            videoCtlTimer->stop();
        }
        videoCtlTimer->start(5000);
    }
}

void JbbhVideoPage::onVideoClick()
{
    if (videoStop) {
        video->stop();
        video->play();

        setPauseIcon();
        videoStop = false;
        return;
    }
    if (video->getVideoState()) {
        setPlayIcon();
        video->pause();
    } else {
        setPauseIcon();
        video->play();
    }
}

void JbbhVideoPage::onBackClick()
{
    timer->stop();
    listWidgetParent->blockSignals(false);
    listWidgetParent->clearSelection();
    stackWidget->setCurrentWidget(parentPage);
    video->stop();
    videoLabel->hide();
    hideVideoCtl();
    if (videoCtlTimer->isActive()) {
        videoCtlTimer->stop();
    }
    lengthLabel->clear();
    curTimeLabel->clear();
}

void JbbhVideoPage::changePosition(int pos)
{
    qDebug() << pos;
    float posf = (float)(pos)/100.0;
    video->setPosition(posf);
}

void JbbhVideoPage::onPressed()
{
    int pos = posSlider->value();
    qDebug() << pos;
    float posf = (float)(pos)/100.0;
    video->setPosition(posf);
}

void JbbhVideoPage::updateInfo()
{
    curTimeLabel->setText(video->getTime());
    lengthLabel->setText("/ " + video->getLength());

    if (video->getLengthInt() > 0 && (video->getLengthInt()-video->getTimeInt()) < 1100) {
        videoStop = true;
        video->pause();
        setPlayIcon();
    }

    if (video->getLengthInt() != 0) {
        int pos = video->getTimeInt()*100/video->getLengthInt();
        qDebug() << pos;
        posSlider->setValue(pos);
    }
}

bool JbbhVideoPage::eventFilter(QObject *obj, QEvent *event)
{
    if(obj==posSlider)
    {
        if (event->type()==QEvent::MouseButtonPress)           //判断类型
        {
            QMouseEvent *mouseEvent = static_cast<QMouseEvent *>(event);
            if (mouseEvent->button() == Qt::LeftButton)	//判断左键
            {
               int dur = posSlider->maximum() - posSlider->minimum();
               int pos = posSlider->minimum() + dur * ((double)mouseEvent->x() / posSlider->width());
               if(pos != posSlider->sliderPosition())
                {
                   changePosition(pos);
                }
            }

        }
    } else if (obj==volSlider) {
        if (event->type()==QEvent::MouseButtonPress)           //判断类型
        {
            QMouseEvent *mouseEvent = static_cast<QMouseEvent *>(event);
            if (mouseEvent->button() == Qt::LeftButton)	//判断左键
            {
               int dur = volSlider->maximum() - volSlider->minimum();
               int pos = volSlider->minimum() + dur * ((double)mouseEvent->x() / volSlider->width());
               if(pos != volSlider->sliderPosition())
                {
                  volSlider->setValue(pos);
                  qDebug() << "vol value:" << pos;
                  video->setVolume(pos);
                  middle->setVolume(pos);
                }
            }

        }
    }
    return QObject::eventFilter(obj,event);
}

void JbbhVideoPage::resizeVideo()
{
    if (!videoLabel->isHidden()) {
        videoLabel->setGeometry(stackWidget->x()+50,stackWidget->y()+86,640,435);
        posSlider->setGeometry(stackWidget->x()+55,stackWidget->y()+526,630,8);
        curTimeLabel->setGeometry(stackWidget->x()+97,stackWidget->y()+546,80,20);
        lengthLabel->setGeometry(stackWidget->x()+182,stackWidget->y()+546,100,20);
        btnPlay->setGeometry(stackWidget->x()+65,stackWidget->y()+546,18,20);
        btnVol->setGeometry(stackWidget->x()+597,stackWidget->y()+521,20,18);
        btnFull->setGeometry(stackWidget->x()+647,stackWidget->y()+546,22,16);
        volSlider->setGeometry(stackWidget->x()+622,stackWidget->y()+526,60,8);
    }
}

void JbbhVideoPage::hideVideoCtl()
{
    posSlider->hide();
    curTimeLabel->hide();
    lengthLabel->hide();
    volSlider->hide();
    btnPlay->hide();
    btnVol->hide();
    btnFull->hide();
    videoBgLabel->hide();
    videoCtlRect->hide();
}

void JbbhVideoPage::showVideoCtl()
{
    posSlider->show();
    curTimeLabel->show();
    lengthLabel->show();
//    volSlider->show();
    btnPlay->show();
//    btnVol->show();
    btnFull->show();
    videoBgLabel->show();
}

//void JbbhVideoPage::updateVideoCtl()
//{
//    posSlider->hide();
//    curTimeLabel->hide();
//    lengthLabel->hide();
//}

void JbbhVideoPage::onMouseMove()
{
    if (isFullScreen) {
        showVideoCtl();
        videoCtlRect->show();

        if (videoCtlTimer->isActive()) {
            videoCtlTimer->stop();
        }
        videoCtlTimer->start(5000);
    }
}

void JbbhVideoPage::onVideoEndReached()
{
    video->stop();
    setPlayIcon();
}

void JbbhVideoPage::initVideoCtl()
{
    QFont font;
    font.setFamily("宋体");
    font.setPixelSize(20);

    QPalette pa;
    pa.setColor(QPalette::WindowText,QColor(211,211,211));

    // 设置控制栏背景
    //videoLabel->setGeometry(stackWidget->x()+174,stackWidget->y()+116,640,500);
    // videoLabel->setGeometry(stackWidget->x()+50,stackWidget->y()+86,640,470);
    videoBgLabel->setGeometry(50,496,640,60);
    videoBgLabel->raise();
    videoBgLabel->hide();
    videoBgLabel->setStyleSheet("background-color: black");

    // 设置进度条
    posSlider->raise();
    posSlider->setGeometry(stackWidget->x()+55,stackWidget->y()+526,630,8);

    // 设置视频时间显示
    curTimeLabel->setGeometry(stackWidget->x()+97,stackWidget->y()+546,80,20);
    curTimeLabel->setFont(font);
    curTimeLabel->setPalette(pa);
    curTimeLabel->raise();
    curTimeLabel->setAlignment(Qt::AlignRight);
    lengthLabel->setGeometry(stackWidget->x()+182,stackWidget->y()+546,100,20);
    lengthLabel->setFont(font);
    lengthLabel->setPalette(pa);
    lengthLabel->raise();
    lengthLabel->setAlignment(Qt::AlignLeft);
    timer->start(1000);

    // 设置控制按钮图标
    QPixmap playP1(":/player/play01.png");
    QPixmap playP2(":/player/play02.png");
    btnPlay->setNormalImage(playP1);
    btnPlay->setPressImage(playP2);
    QPixmap volP1(":/player/vol01.png");
    QPixmap volP2(":/player/vol02.png");
    btnVol->setNormalImage(volP1);
    btnVol->setPressImage(volP2);
    QPixmap fullP1(":/player/full01.png");
    QPixmap fullP2(":/player/full02.png");
    btnFull->setNormalImage(fullP1);
    btnFull->setPressImage(fullP2);

    // 设置控制按钮坐标
    btnPlay->raise();
    btnPlay->setGeometry(stackWidget->x()+65,stackWidget->y()+546,18,20);
    btnVol->raise();
    btnVol->setGeometry(stackWidget->x()+597,stackWidget->y()+521,20,18);
    btnFull->raise();
    btnFull->setGeometry(stackWidget->x()+647,stackWidget->y()+546,22,16);

    // 设置声音控制
    volSlider->raise();
    volSlider->setGeometry(stackWidget->x()+622,stackWidget->y()+526,60,8);
}

void JbbhVideoPage::setVideoCtl()
{
    // 设置播放按钮
    if (video->getVideoState()) {
        QPixmap playP1(":/player/pause01.png");
        QPixmap playP2(":/player/pause02.png");
        btnPlay->setNormalImage(playP1);
        btnPlay->setPressImage(playP2);
        btnPlay->repaint();
    } else {
        QPixmap playP1(":/player/play01.png");
        QPixmap playP2(":/player/play02.png");
        btnPlay->setNormalImage(playP1);
        btnPlay->setPressImage(playP2);
        btnPlay->repaint();
    }

    // 设置声音
    int vol = video->getVolume();
    qDebug() << "get volume:" << vol;
    volSlider->setValue(vol);
}

void JbbhVideoPage::onVolClick()
{
    qDebug() << video->getMute();

    if (video->getMute()) { //当前状态为静音
        video->setMute(0);
        Common::muteStatus = false;

        QPixmap volP1(":/player/vol01.png");
        QPixmap volP2(":/player/vol02.png");
        btnVol->setNormalImage(volP1);
        btnVol->setPressImage(volP2);
        btnVol->repaint();
    } else {
        video->setMute(1);
        Common::muteStatus = true;

        QPixmap muteP1(":/player/mute01.png");
        QPixmap muteP2(":/player/mute02.png");
        btnVol->setNormalImage(muteP1);
        btnVol->setPressImage(muteP2);
        btnVol->repaint();
    }
}

void JbbhVideoPage::initVolCtl()
{
    qDebug() << Common::muteStatus;

    if (Common::muteStatus) { //当前状态为静音
        video->setMute(1);

        QPixmap muteP1(":/player/mute01.png");
        QPixmap muteP2(":/player/mute02.png");
        btnVol->setNormalImage(muteP1);
        btnVol->setPressImage(muteP2);
        btnVol->repaint();
    } else {
        video->setMute(0);

        QPixmap volP1(":/player/vol01.png");
        QPixmap volP2(":/player/vol02.png");
        btnVol->setNormalImage(volP1);
        btnVol->setPressImage(volP2);
        btnVol->repaint();
    }
}

void JbbhVideoPage::onVolSliderMoved(int pos)
{
    video->setVolume(pos);
    middle->setVolume(pos);
}

void JbbhVideoPage::setPlayIcon()
{
    QPixmap playP1(":/player/play01.png");
    QPixmap playP2(":/player/play02.png");
    btnPlay->setNormalImage(playP1);
    btnPlay->setPressImage(playP2);
    btnPlay->repaint();
}

void JbbhVideoPage::setPauseIcon()
{
    QPixmap playP1(":/player/pause01.png");
    QPixmap playP2(":/player/pause02.png");
    btnPlay->setNormalImage(playP1);
    btnPlay->setPressImage(playP2);
    btnPlay->repaint();
}
