#include "middle.h"
#include "tool.h"
#include <QDebug>
#include <QApplication>
#include <QMessageBox>
#include <QDate>
#include <QSettings>
#include <QFileInfo>
#include <QFile>
#include <QVariantMap>
#include <QMessageBox>
#include <QTextCodec>
#include "parser.h"
#include "sync.h"

Middle* Middle::middleInstance = NULL;
Middle::Middle(QObject *parent) : QObject(parent)
{
//    sync = new Sync();
//    sync->create();
    restAPI = RestAPI::getRestApiInstance();
    tool = Tool::getToolInstance();
//    QByteArray replay = restAPI->loginAPIServer("apiuser","123456");
//    if(replay.isNull()) {
//        offline = true;
//    }

//    initVersions();
//    initVersionMap();
//    initClientVersion();
//    initCache();
    offline = false;
    dbOk = false;
    initDatabase();

    manager = new QNetworkAccessManager(this);
    downloading = false;
}

Middle* Middle::getMiddleInstance()
{
    if (middleInstance == NULL) {
        middleInstance = new Middle();
    }

    return middleInstance;
}

void Middle::loginServer()
{
//    if (!offline)
//        restAPI->loginAPIServer("apiuser","123456");
    initVersions();
    initVersionMap();
    initClientVersion();
    baseUrl = getResUrl();
    trainUrl = getTrainResUrl();
}

QString Middle::setVersion(QString ver)
{
    version = this->versionMap(ver);
    qDebug() << "setVersion" << version;

    return version;
}


void Middle::initClientVersion()
{
    QByteArray ret;
    QList<QVariant> list;
    QVariant verInfo;

    if (offline) {
        list = dbGetList("select * from g_upgrade");
        if (list.count() == 1) {
            verInfo = list[0];
            newVersion = verInfo.toMap()["version"].toString();
            clientUrl = verInfo.toMap()["upgrade_package"].toString();
            enforce = verInfo.toMap()["enforce"].toString();
            qDebug() << "initClientVersion" << newVersion << clientUrl << enforce;
        } else {
            newVersion = "";
            clientUrl = "";
            enforce = "";
            newDBVersion = "";
            dbUrl = "";
        }
    } else {
        ret = restAPI->getLatestVersion();
        if (ret.length() == 0) {
            newVersion = "";
            clientUrl = "";
            enforce = "";
            newDBVersion = "";
            dbUrl = "";
            offline = true;
            return;
        } else {
            list = tool->jsonParser(ret, "upgrades");
            foreach (QVariant tmp, list) {
                if (tmp.toMap()["type"].toString() == "client") {
                    newVersion = tmp.toMap()["version"].toString();
                    clientUrl = tmp.toMap()["upgrade_package"].toString();
                    enforce = tmp.toMap()["enforce"].toString();
                } else if(tmp.toMap()["type"].toString() == "sqlitedb") {
                    newDBVersion = tmp.toMap()["version"].toString();
                    dbUrl = tmp.toMap()["upgrade_package"].toString();
                }
            }
        }
        qDebug() << newVersion << clientUrl << enforce << newDBVersion << dbUrl;
    }
}

QString Middle::getNewVersion()
{
    return newVersion;
}

QString Middle::getClientUrl()
{
    return trainUrl + clientUrl;
}

QString Middle::getEnforce()
{
    return enforce;
}

QString Middle::getNewDBVersion()
{
    return newDBVersion;
}

QString Middle::getdbUrl()
{
    return getBaseUrl() + dbUrl;
}

QString Middle::getCurrentVersion()
{
    QString currentVer;

    currentVer = tool->readXml("Version");

    qDebug() << "getCurrentVersion" << currentVer;
    return currentVer;
}

QString Middle::getCurrentDBVersion()
{
    QString currentVer;

    currentVer = tool->readIni("dbVersion");

    qDebug() << __FUNCTION__ << currentVer;
    return currentVer;
}

void Middle::setCurrentDBVersion(QString version) {
    tool->writeIni("dbVersion", version);
}

void Middle::initVersions()
{
    QByteArray query;
    QList<QVariant> list;

    versionList.clear();

    if (offline) {
        list = dbGetList("select * from g_version");
    } else {
        query = restAPI->getVersionList();
        if (query.length() == 0) {
            offline = true;
            return;
        }
        list = tool->jsonParser(query, "versions");
    }

    if (list.count() > 0) {
        foreach (QVariant tmp, list) {
            QList<QString> ver;
            ver.append(tmp.toMap()["name"].toString());
            ver.append(tmp.toMap()["version"].toString());

            versionList.append(ver);
        }
    }
}

QList< QList<QString> > Middle::getVersionList()
{
    return versionList;
}

QList<QString> Middle::getVersions()
{
    QList< QList<QString> > list;
    QList<QString> ret;

    list = this->getVersionList();
    for (int i=0; i< list.count(); i++) {
        ret.append(list[i][0]);
    }

    qDebug() << __FUNCTION__ << ret;
    return ret;
}

void Middle::setUserVersion(QString ver)
{
    tool->writeIni("Version", ver);
}

QString Middle::getUserVersion()
{
    QString ver = tool->readIni("Version");
    return ver;
}

void Middle::initVersionMap()
{
    QList< QList<QString> > list;

    map.insert("写字姿势", "xzzs");
    map.insert("基本笔画", "jbbh");
    map.insert("偏旁部首", "ppbs");
    map.insert("书法课堂", "sfkt");
    map.insert("书写章法", "sxzf");
    map.insert("缓存播放视频", "cache");

    list = this->getVersionList();
    for (int i=0; i<list.count(); i++) {
        map.insert(list[i][0], list[i][1]);
    }
    qDebug() << __FUNCTION__ << map;
}

QString Middle::versionMap(QString ver)
{
    return map[ver];
}

int Middle::getWeekInfo()
{
    QDate date = date.currentDate();
    return date.dayOfWeek();
}

int Middle::enrollSchool(QString school_number, QString serial, QString &msg)
{
    QString macList = tool->getMacAddressList();
    QString mac = tool->getMacAddress();
    QByteArray reply;
    int ret;
    QString clientVer;
    clientVer = getClientVersion();

    reply = restAPI->enrollSchool(school_number, serial, macList, clientVer);
    ret = tool->jsonParserString(reply, "errorCode").toInt();

    // 注册返回为空，或者解析错误码失败设置为离线模式
    qDebug() << "注册返回信息 reply:" << reply << "ret:" << ret ;
    if (reply.length() == 0 || ret == -1) {
        offline = true;
        //断网情况，保存登录信息
        tool->writeIni("SchoolNumber", school_number);
        tool->writeIni("Serial", serial);
        tool->writeIni("Volume", "50");
        return -99;
    }

    if (ret != 0) {
        ret = tool->jsonParserString(reply, "errorCode").toInt();
        msg = tool->jsonParserString(reply, "message");
        if (msg == "Parse error")
            msg = "服务异常,请联系管理员!";

    }
    qDebug() << "enrollSchool" << ret;

    if (ret == 0) {
        tool->writeIni("SchoolNumber", school_number);
        tool->writeIni("Serial", serial);
        tool->writeIni("Volume", "50");

        // 注册完成后登录
        reply = restAPI->login(school_number, serial, mac, clientVer);
        ret = tool->jsonParserString(reply, "errorCode").toInt();
        msg = tool->jsonParserString(reply, "message");
        qDebug() << "login" << ret;
    }

    return ret;
}

int Middle::login(QString &msg)
{
    // del
//    return 0;
    int ret = -1;
    QByteArray reply;
    QString school_number = tool->readIni("SchoolNumber");
    QString serial = tool->readIni("Serial");
    QString flag = tool->readIni("Flag");
    QString mac = tool->getMacAddress();
    QString clientVer;

    clientVer = getClientVersion();
    if (school_number.length()>0 && serial.length()>0 && mac.length()>0) {
        reply = restAPI->login(school_number, serial, mac, clientVer);
        if (reply.length() == 0 && flag != "0") {
            offline = true;
            return -1;
        }
        ret = tool->jsonParserString(reply, "errorCode").toInt();
        if (ret == -1) {
            offline = true;
            return -1;
        }
        msg = tool->jsonParserString(reply, "message");
        qDebug() << __FUNCTION__ << ret;
    }
    else {
        msg = "获取硬件信息失败!";
    }

    return ret;
}

QString Middle::getOption(QString key)
{
    QByteArray ret;
    QString value;

    if (offline) {
        value = dbGetString("g_option", "option_value", "option_key", key);
    } else {
        ret = restAPI->getOption(key);
        if (ret.length() > 0) {
            value = tool->jsonParserString(ret, "option_value");
        } else {
            offline = true;
            return "";
        }
    }

//    qDebug() << value;

    return value;
}

QList<QVariant> Middle::getCourseList(QString type)
{
    QList<QVariant> courseList;
    QByteArray ret;

    ret = restAPI->getCourseList(type);
    if (ret.length() > 0) {
        courseList = tool->jsonParser(ret, "catalog");
    }

    return courseList;
}

QList<QVariant> Middle::getLectureList(QString code)
{
    QList<QVariant> lectureList;
    QByteArray ret;

    ret = restAPI->getLectureList(code);
    if (ret.length() > 0) {
        lectureList = tool->jsonParser(ret, "lecture");
    }

    return lectureList;
}

QList<QVariant> Middle::getLectureWordList(QString code)
{
    QList<QVariant> lectureInfoList;
    QByteArray ret;

    ret = restAPI->getLectureWordList(code);
    if (ret.length() > 0) {
        lectureInfoList = tool->jsonParser(ret, "lectureWord");
    }

    return lectureInfoList;
}

QList<QVariant> Middle::getCourseWordList(QString course_code)
{
    QList<QVariant> wordList, tmpList;
    QByteArray ret;
    QString sql;

    if (offline) {
        sql = "select * from g_course_word where course_code='" + course_code +"'";
//        qDebug() << sql;
        tmpList = dbGetList(sql);

        foreach (QVariant tmp, tmpList) {
            QString zi = tmp.toMap()["zi"].toString();
            sql = "select * from g_word where zi='" + zi + "'";
            wordList.append(dbGetList(sql));
        }
    } else {
        ret = restAPI->getCourseWordList(course_code);
        if (ret.length() > 0) {
            wordList = tool->jsonParser(ret, "course_word");
        } else {
            offline = true;
            return wordList;
        }
    }

    return wordList;
}

QList<QVariant> Middle::getCourseGroupWordList(QString course_code)
{
    QList<QVariant> wordList;
    QByteArray ret;
    QString sql;

    if(offline) {
        sql = "select * from g_groupwords where code='" + course_code + "'";
        wordList = dbGetList(sql);
    } else {
        ret = restAPI->getCourseWordList(course_code);
        if (ret.length() > 0) {
            wordList = tool->jsonParser(ret, "groupwords");
        } else {
            offline = true;
            return wordList;
        }
    }

    return wordList;
}

QList<QVariant> Middle::getCourseChengYuList(QString course_code)
{
    QList<QVariant> chengYuList;
    QByteArray ret;
    QString sql;

    if(offline) {
        sql = "select * from g_course_idiom where code='" + course_code + "'";
        chengYuList = dbGetList(sql);
    } else {
        ret = restAPI->getCourseWordList(course_code);
        if (ret.length() > 0) {
            chengYuList = tool->jsonParser(ret, "course_idiom");
        } else {
            offline = true;
            return chengYuList;
        }
    }

    return chengYuList;
}

QList<QVariant> Middle::getCourseWhrList(QString course_code)
{
    QList<QVariant> whrList;
    QByteArray ret;
    QString sql;

    if(offline) {
        // TODO change table name
        sql = "select * from g_course_wohuiren where course_code='" + course_code + "'";
        whrList = dbGetList(sql);
    } else {
        ret = restAPI->getCourseWordList(course_code);
        if (ret.length() > 0) {
            whrList = tool->jsonParser(ret, "course_wohuiren");
        } else {
            offline = true;
            return whrList;
        }
    }

    return whrList;
}

QList<QVariant> Middle::getStroke(QString group_code)
{
    QList<QVariant> strokeList;
    QByteArray ret;
    QString sql;

    if (offline) {
        sql = "select * from g_strokes where group_code='" + group_code + "'";
        strokeList = dbGetList(sql);
    } else {
        ret = restAPI->getStrokesList(group_code);
        if (ret.length() > 0) {
            strokeList = tool->jsonParser(ret, "strokes");
        } else {
            offline = true;
            return strokeList;
        }
    }

    return strokeList;
}

QList<QVariant> Middle::getRadical(QString group_code)
{
    QList<QVariant> bushouList;
    QByteArray ret;
    QString sql;

    if (offline) {
        sql = "select * from g_bushou where group_code='" + group_code + "'";
        bushouList = dbGetList(sql);
    }
    ret = restAPI->getBuShouList(group_code);

    if (ret.length() > 0) {
        bushouList = tool->jsonParser(ret, "bushous");
    } else {
        offline = true;
        return bushouList;
    }

    return bushouList;
}

QList<QVariant> Middle::getClassicalList(QString grade)
{
    QList<QVariant> articleList;
    QByteArray ret;
    QString sql;

    if (offline) {
        sql = "select * from g_poem where grade='" + grade + "'";
        articleList = dbGetList(sql);
    } else {
        ret = restAPI->getPoemList(grade);
        if (ret.length() > 0) {
            articleList = tool->jsonParser(ret, "poems");
        } else {
            offline = true;
            return articleList;
        }
    }

    return articleList;
}

QList<QVariant> Middle::getClassicalItem(QString poem_id)
{
    QList<QVariant> poemList;
    QByteArray ret;
    QString sql;

    if (offline) {
        sql = "select * from g_poemitem where poem_id='" + poem_id + "'";
        poemList = dbGetList(sql);
    } else {
        ret = restAPI->getPoemItem(poem_id);
        if (ret.length() > 0) {
            poemList = tool->jsonParser(ret, "poemitems");
        } else {
            offline = true;
            return poemList;
        }
    }

    return poemList;
}

QList<QVariant> Middle::getWordInfo(QString word)
{
    QList<QVariant> wordList;
    QByteArray ret;
    QString sql;

    if (offline) {
        sql = "select * from g_word where zi='" + word + "'";
        wordList = dbGetList(sql);
    } else {
        ret = restAPI->getWordInfo(word.toUtf8());
        if (ret.length() > 0) {
            wordList = tool->jsonParser(ret, "word");
        } else {
            offline = true;
            return wordList;
        }
    }

    return wordList;
}

QList<QVariant> Middle::getCalligraphyWord(QString word)
{
    QList<QVariant> wordList;
    QByteArray ret;
    QString sql;

    if (offline) {
        sql = "select * from g_calligraphy_word where zi='" + word + "'";
        wordList = dbGetList(sql);
    } else {
        ret = restAPI->getCalligraphyWord(word.toUtf8());
        if (ret.length() > 0) {
            wordList = tool->jsonParser(ret, "CalligraphyWord");
        } else {
            offline = true;
            return wordList;
        }
    }

    return wordList;
}

QList<QVariant> Middle::getCalligraphyList(QString grade, QString level, QString type)
{
    QList<QVariant> calligraphyList;
    QByteArray ret;
    QString sql;

    if (offline) {
        sql = "select * from g_calligraphy where grade='" + grade + "' and level='" +
                level + "' and type='" + type + "'";
        calligraphyList = dbGetList(sql);
    } else {
        ret = restAPI->getCalligraphyList(grade, level, type);
        if (ret.length() > 0) {
            calligraphyList = tool->jsonParser(ret, "calligraphys");
        } else {
            offline = true;
            return calligraphyList;
        }
    }

    return calligraphyList;
}

QList<QVariant> Middle::getCalligraphyLessonList(QString calligraphy_id)
{
    QList<QVariant> calligraphyLessonList;
    QByteArray ret;
    QString sql;

    if (offline) {
        sql = "select * from g_calligraphylesson where calligraphy_id='" + calligraphy_id + "'";
        calligraphyLessonList = dbGetList(sql);
    } else {
        ret = restAPI->getCalligraphyLessonList(calligraphy_id);
        if (ret.length() > 0) {
            calligraphyLessonList = tool->jsonParser(ret, "calligraphylessons");
        } else {
            offline = true;
            return calligraphyLessonList;
        }
    }

    return calligraphyLessonList;
}

QList<QVariant> Middle::getCalligraphyLessonktlxList(QString calligraphy_id)
{
    QList<QVariant> calligraphyLessonList;
    QByteArray ret;

    // TODO offline
    ret = restAPI->getCalligraphyLessonktlxList(calligraphy_id);
    if (ret.length() > 0) {
        calligraphyLessonList = tool->jsonParser(ret, "calligraphylessonsktlx");
    } else {
        offline = true;
        return calligraphyLessonList;
    }

    return calligraphyLessonList;
}

QString Middle::getXieZiZiShiVideoUrl(QString code)
{
    QByteArray ret;
    QList<QVariant> ParserRet;
    QString url, size;
    QString sql;

    if (offline) {
        sql = "select * from g_xzzs where code='" + code + "'";
        ParserRet = dbGetList(sql);
    } else {
        ret = restAPI->getXieZiZiShiList(code);
        if (ret.length() == 0) {
            offline = true;
            return "";
        }

        ParserRet = tool->jsonParser(ret, "xzzss");
        if (ParserRet.count() == 0)
            return "";
    }

    url = ParserRet[0].toMap()["video_url"].toString();
    size = ParserRet[0].toMap()["video_size"].toString();
    url = url + "," + size;

//    qDebug() << url;

    return url;
}

//------------------------- 生字表 ------------------------------------------
QList<QVariant> Middle::getPinyinIndexList(QString grade, QString level)
{
    QByteArray ret;
    QList<QVariant> indexList;
    QString sql;

    if (offline) {
        if (grade.length()>0 && level.length()>0) {
            sql = "select w.index2 as 'index' from g_word w where w.zi in (select cw.zi from g_course_word cw where cw.course_code in "
                  "(select code from g_course c where c.version='" + version +"' and c.grade=" + grade +
                    " and c.level=" + level +")group by cw.zi) group by w.index2";
        } else {
            sql = "select w.index2 as 'index' from g_word w where w.zi in (select cw.zi from g_course_word cw where cw.course_code in "
                  "(select code from g_course c where c.version='" + version +"')group by cw.zi) group by w.index2";
        }
//        qDebug() << sql;
        indexList = dbGetList(sql);
    } else {
        ret = restAPI->getPinyinIndexList(version, grade, level);
        if (ret.length() > 0) {
            indexList = tool->jsonParser(ret, "PinyinIndexList");
        } else {
            offline = true;
            return indexList;
        }
    }
//        qDebug() << indexList;

    return indexList;
}

QList<QVariant> Middle::getBushouIndexList(QString grade, QString level)
{
    QByteArray ret;
    QList<QVariant> indexList;
    QString sql;

    if (offline) {
        if (grade.length()>0 && level.length()>0) {
            sql = "select w.bs_bihua from g_word w where w.zi in (select cw.zi from g_course_word cw where cw.course_code in "
                  "(select code from g_course c where c.version='" + version +"' and c.grade=" + grade +
                    " and c.level=" + level +")group by cw.zi) group by w.bs_bihua";
        } else {
            sql = "select w.bs_bihua from g_word w where w.zi in (select cw.zi from g_course_word cw where cw.course_code in "
                  "(select code from g_course c where c.version='" + version +"')group by cw.zi) group by w.bs_bihua";
        }
//        qDebug() << "sql" << sql;
        indexList = dbGetList(sql);
    } else {
        ret = restAPI->getBushouIndexList(version, grade, level);
        if (ret.length() > 0) {
            indexList = tool->jsonParser(ret, "BushouIndexList");
        } else {
            offline = true;
            return indexList;
        }
    }

    return indexList;
}

QList<QVariant> Middle::getPinyinList(QString index, QString grade, QString level)
{
    QByteArray ret;
    QList<QVariant> pinyinList;
    QString sql;

    if (offline) {
        if (grade.length()>0 && level.length()>0) {
            sql = "select w.wd_pinyin from g_word w where w.index2='" + index + "' and w.zi in (select cw.zi from g_course_word cw where cw.course_code in "
                  "(select code from g_course c where c.version='" + version +"' and c.grade=" + grade +
                    " and c.level=" + level +")group by cw.zi) group by w.wd_pinyin";
        } else {
            sql = "select w.wd_pinyin from g_word w where w.index2='" + index + "' and w.zi in (select cw.zi from g_course_word cw where cw.course_code in "
                  "(select code from g_course c where c.version='" + version +"')group by cw.zi) group by w.wd_pinyin";
        }
//        qDebug() << "sql" << sql;
        pinyinList = dbGetList(sql);
    } else {
        ret = restAPI->getPinyinList(version, grade, level, index);
        if (ret.length() > 0) {
            pinyinList = tool->jsonParser(ret, "PinyinList");
        } else {
            offline = true;
            return pinyinList;
        }
    }

    return pinyinList;
}

QList<QVariant> Middle::getBushouList(QString index, QString grade, QString level)
{
    QByteArray ret;
    QList<QVariant> bushouList;
    QString sql;
    QString index_ = index.trimmed();

    if (offline) {
        if (grade.length()>0 && level.length()>0) {
            sql = "select w.bushou from g_word w where w.bs_bihua='" + index_ + "' and w.zi in (select cw.zi from g_course_word cw where cw.course_code in "
                  "(select code from g_course c where c.version='" + version +"' and c.grade=" + grade +
                    " and c.level=" + level +")group by cw.zi) group by w.bushou";
        } else {
            sql = "select w.bushou from g_word w where w.bs_bihua='" + index_ + "' and w.zi in (select cw.zi from g_course_word cw where cw.course_code in "
                  "(select code from g_course c where c.version='" + version +"')group by cw.zi) group by w.bushou";
        }
        qDebug() << "getBushouList sql" << sql;
        bushouList = dbGetList(sql);
    } else {
        ret = restAPI->getBushouList(version, grade, level, index);
        if (ret.length() > 0) {
            bushouList = tool->jsonParser(ret, "Bushou");
        } else {
            offline = true;
            return bushouList;
        }
    }

    return bushouList;
}

QList<QVariant> Middle::getZiBihuaIndexList(QString pinyin, QString grade, QString level)
{
    QByteArray ret;
    QList<QVariant> indexList;
    QString sql;
    QString pinyin_ = pinyin.trimmed();

    if (offline) {
        if (grade.length()>0 && level.length()>0) {
            sql = "select w.zi_bihua from g_word w where w.wd_pinyin='" + pinyin_ + "' and w.zi in (select cw.zi from g_course_word cw where cw.course_code in "
                  "(select code from g_course c where c.version='" + version +"' and c.grade=" + grade +
                    " and c.level=" + level +")group by cw.zi) group by w.zi_bihua";
        } else {
            sql = "select w.zi_bihua from g_word w where w.wd_pinyin='" + pinyin_ + "' and w.zi in (select cw.zi from g_course_word cw where cw.course_code in "
                  "(select code from g_course c where c.version='" + version +"')group by cw.zi) group by w.zi_bihua";
        }
//        qDebug() << "sql" << sql;
        indexList = dbGetList(sql);
    } else {
        ret = restAPI->getZiBihuaIndexList(version, grade, level, pinyin);
        if (ret.length() > 0) {
            indexList = tool->jsonParser(ret, "ZiBihuaIndexList");
        } else {
            offline = true;
            return indexList;
        }
    }

    return indexList;
}

QList<QVariant> Middle::getWbBihuaIndexList(QString bushou, QString grade, QString level)
{
    QByteArray ret;
    QList<QVariant> indexList;
    QString sql;
    QString bushou_ = bushou.trimmed();

    if (offline) {
        if (grade.length()>0 && level.length()>0) {
            sql = "select w.bw_bihua from g_word w where w.bushou='" + bushou_ + "' and w.zi in (select cw.zi from g_course_word cw where cw.course_code in "
                  "(select code from g_course c where c.version='" + version +"' and c.grade=" + grade +
                    " and c.level=" + level +")group by cw.zi) group by w.bw_bihua";
        } else {
            sql = "select w.bw_bihua from g_word w where w.bushou='" + bushou_ + "' and w.zi in (select cw.zi from g_course_word cw where cw.course_code in "
                  "(select code from g_course c where c.version='" + version +"')group by cw.zi) group by w.bw_bihua";
        }
//        qDebug() << "sql" << sql;
        indexList = dbGetList(sql);
//        qDebug() << indexList;
    } else {
        ret = restAPI->getWbBihuaIndexList(version, grade, level, bushou);
        if (ret.length() > 0) {
            indexList = tool->jsonParser(ret, "WbBihuaIndexList");
        } else {
            offline = true;
            return indexList;
        }
//        qDebug() << indexList;
    }

    return indexList;
}

QList<QVariant> Middle::getWordByPinyinBihua(QString pinyin, QString bihua, QString grade, QString level)
{
    QByteArray ret;
    QList<QVariant> wordList;
    QString sql;
    QString pinyin_ = pinyin.trimmed();
    QString bihua_ = bihua.trimmed();

    if (offline) {
        if (grade.length()>0 && level.length()>0) {
            sql = "select * from g_word w where w.wd_pinyin='" + pinyin_ + "' and w.zi_bihua='" + bihua_ + "' and w.zi in (select cw.zi from g_course_word cw where cw.course_code in "
                  "(select code from g_course c where c.version='" + version +"' and c.grade=" + grade +
                    " and c.level=" + level +")group by cw.zi) group by w.zi";
        } else {
            sql = "select * from g_word w where w.wd_pinyin='" + pinyin_ + "' and w.zi_bihua='" + bihua_ + "' and w.zi in (select cw.zi from g_course_word cw where cw.course_code in "
                  "(select code from g_course c where c.version='" + version +"')group by cw.zi) group by w.zi";
        }
        qDebug() << "getWordByPinyinBihua sql" << sql;
        wordList = dbGetList(sql);
//        qDebug() << indexList;
    } else {
        ret = restAPI->getWordByPinyinBihua(version, grade, level, pinyin, bihua);
        if (ret.length() > 0) {
            wordList = tool->jsonParser(ret, "word");
        } else {
            offline = true;
            return wordList;
        }
    }

    return wordList;
}

// 此处笔画为外部笔画
QList<QVariant> Middle::getWordByBushouBihua(QString bushou, QString bihua, QString grade, QString level)
{
    QByteArray ret;
    QList<QVariant> wordList;
    QString sql;
    QString bushou_ = bushou.trimmed();
    QString bihua_ = bihua.trimmed();

    if (offline) {
        if (grade.length()>0 && level.length()>0) {
            sql = "select * from g_word w where w.bushou='" + bushou_ + "' and w.bw_bihua='" + bihua_ + "' and w.zi in (select cw.zi from g_course_word cw where cw.course_code in "
                  "(select code from g_course c where c.version='" + version +"' and c.grade=" + grade +
                    " and c.level=" + level +")group by cw.zi) group by w.zi";
        } else {
            sql = "select * from g_word w where w.bushou='" + bushou_ + "' and w.bw_bihua='" + bihua_ + "' and w.zi in (select cw.zi from g_course_word cw where cw.course_code in "
                  "(select code from g_course c where c.version='" + version +"')group by cw.zi) group by w.zi";
        }
        qDebug() << "getWordByBushouBihua sql" << sql;
        wordList = dbGetList(sql);
//        qDebug() << indexList;
    } else {
        ret = restAPI->getWordByBushouBihua(version, grade, level, bushou, bihua);
        if (ret.length() > 0) {
            wordList = tool->jsonParser(ret, "word");
        } else {
            offline = true;
            return wordList;
        }
    }

    return wordList;
}

QList<QVariant> Middle::getWordByPinyin(QString pinyin, QString grade, QString level)
{
    QByteArray ret;
    QList<QVariant> wordList;
    QString sql;

    if (offline) {
        sql = "select * from g_word w where w.wd_pinyin='" + pinyin + "' and w.zi in (select cw.zi from g_course_word cw where cw.course_code in "
              "(select code from g_course c where c.version='" + version +"')group by cw.zi) group by w.zi";

        wordList = dbGetList(sql);
    } else {
        ret = restAPI->getWordByPinyin(version, grade, level, pinyin);
        if (ret.length() > 0) {
            wordList = tool->jsonParser(ret, "word");
        } else {
            offline = true;
            return wordList;
        }
    }

    return wordList;
}

QList<QVariant> Middle::getWordByBushou(QString bushou, QString grade, QString level)
{
    QByteArray ret;
    QList<QVariant> wordList;
    QString sql;
    QString bushou_ = bushou.trimmed();

    if (offline) {
        sql = "select * from g_word w where w.bushou='" + bushou_ + "' and w.zi in (select cw.zi from g_course_word cw where cw.course_code in "
              "(select code from g_course c where c.version='" + version +"')group by cw.zi) group by w.zi";

        qDebug() << "sql: " << sql;
        wordList = dbGetList(sql);
    } else {
        ret = restAPI->getWordByBushou(version, grade, level, bushou);
        if (ret.length() > 0) {
            wordList = tool->jsonParser(ret, "word");
        } else {
            offline = true;
            return wordList;
        }
    }

    return wordList;
}
//------------------------- 生字表 ------------------------------------------

//QString Middle::getOption(QString key)
//{
//    QJsonObject json;

//    json = restAPI->getOption(key);
//    QString data(QJsonDocument(json).toJson());

//    qDebug() << data;

//    return data.toUtf8();
//}

QString Middle::getBaseUrl()
{
    return baseUrl;
}

QString Middle::getTrainUrl()
{
    return trainUrl;
}

QString Middle::getResUrl()
{
    QString url = getOption("res_web_server");

    qDebug() << __FUNCTION__ << url;

    return url;
}

QString Middle::getTrainResUrl()
{
    QString url = getOption("train_web_server");

    qDebug() << __FUNCTION__ << url;

    return url;
}

void Middle::setDownloadInfo(QString type, QString version, QString grade, QString level)
{
    tool->setDownloadInfo(type, this->versionMap(version), grade, level);
}

void Middle::unSetDownloadInfo(QString type, QString version, QString grade, QString level)
{
    tool->unSetDownloadInfo(type, this->versionMap(version), grade, level);
}

bool Middle::getDownloadStatus(QString type, QString version, QString grade, QString level)
{
    QList< QList<QString> > downloadInfo = tool->getDownloadInfo();
    QString type_, version_, grade_, level_;

    for (int i=0; i<downloadInfo.count(); i++) {
        QList<QString> tmp = downloadInfo[i];
        type_ = tmp[0];
        version_ = tmp[1];
        grade_ = tmp[2];
        level_ = tmp[3];

        if (version.length() == 0) {
            if (type_ == type && version_.length() == 0) {
                return true;
            }
        }
        else if (grade.length() == 0) {
            if (type_ == type && version_ == this->versionMap(version) && grade_.length() == 0) {
                return true;
            }
        }
        else if (level.length() == 0) {
            if (type_ == type && version_ == this->versionMap(version) && grade_ == grade && level_.length() == 0) {
                return true;
            }
        }
        else {
            if (type_ == type && version_ == this->versionMap(version) && grade_ == grade && level_ == level) {
                return true;
            }
        }
    }

    return false;
}

//void Middle::download(QString version, QString grade, QString level, QString all)
//{
//    QString ver = this->versionMap(version);

//    qDebug() << ver << grade << level;
//    sync->create();
//}

QString Middle::getVideoPath(QString type, QString url, int size)
{
    QString name = "";
    QString webFile;

    if (type == "ggxz")
        webFile = this->getBaseUrl() + url;
    if (type == "dxpy")
        webFile = this->getTrainUrl() + url;

    if (url.length() > 0) {
        QStringList list = url.split("/");
        name = list[list.length()-1];
    }

    QString tmp = name;
    QByteArray encodeName = name.toUtf8().toPercentEncoding();
    webFile.replace(name, QString(encodeName));
    qDebug() << __FUNCTION__ << "webFile" << webFile;

    QString localFile = QCoreApplication::applicationDirPath() + "/cache/" + tmp;
    qDebug() << __FUNCTION__ << "本地文件路径" << localFile;
    QFileInfo fi(localFile);
    QFile file(localFile);
    qDebug() << __FUNCTION__ << "file.size():" << file.size() << "size:" << size;

    // 离线模式下不再检测文件大小
    if (offline && fi.isFile()) {
        localFile.replace("/", "\\");
        qDebug() << __FUNCTION__ << "离线模式 本地文件路径" << localFile;
        return localFile;
    }

    if (size == -1)
        return webFile;

    if (fi.isFile() && file.size() == size && file.size() > 0) {
    // 只要本地有视频就播放本地，不再检测是否是最新视频
//    if (fi.isFile() && file.size() > 0) {
        localFile.replace("/", "\\");
        qDebug() << __FUNCTION__ << "在线模式 本地文件路径" << localFile;
        return localFile;
    }

    return webFile;
}

void Middle::myQuit()
{
    qDebug() << "quit!";
    restAPI->logout();
}

void Middle::errorMBox(QString text)
{
    QMessageBox msgBox;
    msgBox.setWindowTitle("错误");
    msgBox.setFixedSize(400,300);
    msgBox.setText(text);
    msgBox.setIcon(QMessageBox::Critical);
    msgBox.setStyleSheet("font: 15pt");
    msgBox.addButton("确定",QMessageBox::ActionRole);
    msgBox.exec();
}

QByteArray Middle::getImageData(QString url)
{
    QByteArray data;
    if (offline) {
        return data;
    } else {
       // TODO baseUrl + url
       url = this->getBaseUrl() + url;
       // del
       //url = this->getBaseUrlOld() + url;
       qDebug() << __FUNCTION__ << "image url" << url;
       data = restAPI->getReply(url);
    }

   return data;
}

QByteArray Middle::getTrainImageData(QString url)
{
   url = this->getTrainUrl() + url;
   qDebug() << __FUNCTION__ << "image url" << url;
   QByteArray data = restAPI->getReply(url);

   return data;
}

QString Middle::getImageFile(QString url, QString prefix)
{
    QString filePath, fileName;
    QStringList list = url.split("/");
    fileName = list[list.length()-1];

    filePath = QCoreApplication::applicationDirPath()+ prefix + fileName;
    QFile fileTmp(filePath);

    if(!fileTmp.exists()) {
        filePath = "";
    }

    qDebug() << "getImageFile filePath" << filePath;
    return filePath;
}

QString Middle::getImageFile2(QString url, QString prefix)
{

    QStringList list = url.split("/");
    QString fileName = list[list.length()-2] + "/" + list[list.length()-1];

    return QCoreApplication::applicationDirPath()+ prefix + fileName;
}

QList<QVariant> Middle::getSwjzInfo(QString word)
{
    QList<QVariant> wordList;
    QByteArray ret;
    QString sql;

    if (offline) {
        sql = "select * from g_calligraphy_word where zi='" + word + "'";
        wordList = dbGetList(sql);
    } else {
        ret = restAPI->getSwjzInfo(word.toUtf8());
        if (ret.length() > 0) {
            wordList = tool->jsonParser(ret, "swjz");
        } else {
            offline = true;
            return wordList;
        }
    }

    return wordList;
}

QString Middle::getClientVersion()
{
    return "2.0";
}

void Middle::setVolume(int vol)
{
    tool->writeIni("Volume", QString::number(vol, 10));
}

int Middle::getVolume()
{
    // 返回 100
    return 100;
    int vol;
    bool ok;
    QString tmp = tool->readIni("Volume");

    if (tmp.length() == 0)
        tmp = "50";

    vol = tmp.toInt(&ok, 10);

    return vol;
}

bool Middle::downloadFile(QString url_, int size)
{
    // del
    QUrl url = this->getBaseUrl() + url_;

    QString path = QCoreApplication::applicationDirPath();
    QFileInfo info(url.path());
    QString fileName(info.fileName());
    QString filePath = path + "/cache/" + fileName;
    qDebug() << __FUNCTION__ << "downloadFile filePath" << filePath << "size:" << size;

    QFile fileTmp(filePath);
    if (fileTmp.exists() && fileTmp.size() == size) {
        return false;
    }

    return true;
}

void Middle::startDownload()
{
    QList<QString> fileList;
    qDebug() << __FUNCTION__ << "before download" << download_list;

    if (!downloading && download_list.count() > 0) {
        QString path = QCoreApplication::applicationDirPath();
        fileList = download_list[0];
        QUrl url = fileList[0];
        int size = fileList[1].toInt();

        QFileInfo info(url.path());
        QString fileName(info.fileName());
        QString filePath = path + "/cache/" + fileName;
        qDebug() << __FUNCTION__ << "filePath" << filePath << "size:" << size;

        QFile fileTmp(filePath);
        if (fileTmp.exists() && fileTmp.size() == size && fileTmp.size() > 0) {
            download_list.removeAt(0);
            startDownload();
            return;
        }

        file = new QFile(filePath);
        if (!file->open(QIODevice::WriteOnly)) {
            qDebug() << "open file error";
            delete file;
            file = 0;
            return;
        }

        qDebug() << __FUNCTION__ << "start download" << download_list[0];
        startRequest(url);
        download_list.removeAt(0);
        downloading = true;
        qDebug() << __FUNCTION__ << "after remove" << download_list;
    }
}

void Middle::download(QString url, int size)
{
    QList<QString> fileList;

    if (getClientMode())
        return;

    if (size == -1)
        return;
    if (tool->readIni("Cache") == "false" && downloadFile(url, size) == false)
        return;

    // del
    url = this->getBaseUrl() + url;
    fileList.append(url);
    fileList.append(QString::number(size, 10));
    download_list.append(fileList);
    startDownload();
}

void Middle::startRequest(QUrl url)
{
    reply = manager->get(QNetworkRequest(url));
    connect(reply,SIGNAL(finished()),this,SLOT(httpFinished()));
    connect(reply,SIGNAL(readyRead()),this,SLOT(httpReadyRead()));
}

void Middle::httpFinished()
{
    file->flush();
    file->close();
    reply->deleteLater();
    reply = 0;
    delete file;
    file = 0;
    downloading = false;
    startDownload();
}

void Middle::httpReadyRead()
{
    if (file) {
        file->write(reply->readAll());
    }
}

void Middle::initCache()
{
    QString status = tool->readIni("Cache");

    qDebug() << __FUNCTION__ << "status:" << status;
    if (status.length() == 0) {
        tool->writeIni("Cache", "true");
    }
}

// 初始化数据库连接
void Middle::initDatabase()
{
    QString version = "2";
    if (getCurrentDBVersion().length() == 0 || getCurrentDBVersion().toInt() < version.toInt()) {
        setCurrentDBVersion(version);
    }
    db = QSqlDatabase::addDatabase("QSQLITE");  //使用sqlite数据库驱动
    db.setDatabaseName(QCoreApplication::applicationDirPath() + "/data.db");  //我们之前建立的数据库
    bool ok = db.open();  //尝试连接数据库
    qDebug() << __FUNCTION__ << ok;
    if (ok)
        dbOk = true;
    else {
        dbOk = false;
        qDebug() << __FUNCTION__  << "open databse error!";
    }
}

QList<QVariant> Middle::dbGetList(QString sql)
{
    QSqlQuery query(db);
    QMap<QString ,QVariant> map;
    QList<QVariant> list;

    if (query.exec(sql))   //尝试列出 student 表的所有记录
    {  //本次查询成功
        while(query.next())
        {  //定位结果到下一条记录
            for(int x=0; x < query.record().count(); x++) {
                map.insert(query.record().fieldName(x), query.value(x));
            }
            list.append(map);
            map.clear();
        }
    }
//    qDebug() << list;

    return list;
}

QString Middle::dbGetString(QString tableName, QString dbValue, QString dbKey, QString key)
{
    QSqlQuery query(db);
    QString sql;
    QString ret;

    sql = "select " + dbValue + " from " + tableName + " where " + dbKey + " = '" + key + "'";
//    qDebug() << sql;
    if (query.exec(sql)) {
        while(query.next())
        {  //定位结果到下一条记录
            ret = query.value(0).toString();
        }
    }
//    qDebug() << ret;
    return ret;
}

bool Middle::getClientMode()
{
    return offline;
}

void Middle::setClientMode(bool value)
{
    offline = value;
}

int Middle::verCap(QString v1, QString v2) {
    if (!v1.compare(v2)) {
        return 0;
    }

    QStringList vList1 = v1.split(".");
    QStringList vList2 = v2.split(".");
    int vCount1 = vList1.count();
    int vCount2 = vList2.count();
    int nCount = vCount1 > vCount2 ? vCount1 : vCount2;

    for (int i=0; i<nCount; i++) {
        if (i < vCount1 && i < vCount2) {
            int n1 = vList1[i].toInt();
            int n2 = vList2[i].toInt();

            if (n1 == n2)
                continue;
            if (n1 > n2)
                return 1;
            if (n1 < n2)
                return -1;
        } else if (i < vCount1) {
            int n = vList1[i].toInt();
            if (n > 0)
                return 1;
            if (n == 0)
                continue;
        } else {
            int n = vList2[i].toInt();
            if (n > 0)
                return -1;
            if (n == 0)
                continue;
        }
    }

    return 0;
}

Middle::~Middle()
{
    if(sync != NULL)
        delete sync;
}
