#include "querysubpage.h"
#include <QDebug>

QuerySubPage* QuerySubPage::querySubInstance = NULL;

QuerySubPage::QuerySubPage(QWidget *parent) : QWidget(parent)
{
    middle = Middle::getMiddleInstance();
    wordPage = WordPage::getWordPageInstance();
    queryPlayPage = QueryPlayPage::getQueryPlayPageInstance();
}

QuerySubPage* QuerySubPage::getQuerySubInstance()
{
    if (querySubInstance == NULL)
        querySubInstance = new QuerySubPage();

    return querySubInstance;
}

void QuerySubPage::setWidget(QWidget *querySub, QStackedWidget *stack, QWidget *next, QWidget *nextPlay)
{
    querySubPage = querySub;
    nextPage = next;
    nextPlayPage = nextPlay;
    stackWidget = stack;

    // 页面元素初始化
    titleLabel = new QLabel(querySubPage);
    btnBack = new ImageButton(querySubPage);
    btnPlay = new ImageButton(querySubPage);
    imageList = new QListWidget(querySubPage);
    imageList->hide();

    connect(imageList,SIGNAL(clicked(QModelIndex)),this,SLOT(itemClicked(QModelIndex)));
    connect(btnBack, SIGNAL(clicked()), this, SLOT(onBackClick()));
    connect(btnPlay, SIGNAL(clicked()), this, SLOT(onPlayClick()));
}

void QuerySubPage::setQuerySubPage(QWidget *parent, ImageButton *btnP, QList< QList<QString> > wordList)
{
    parentPage = parent;
    btnParent = btnP;
    words = wordList;

    setBackground();
    setButtons();
    setListView(wordList);
}

void QuerySubPage::setBackground()
{
    querySubPage->setStyleSheet("QWidget#querySubPage{image:url(:/chaxun/bj_17.jpg)}");

    titleLabel->setGeometry(433,40,200,40);
    titleLabel->setText("查询结果");
    titleLabel->setStyleSheet("font-size: 30px; font-family:宋体");
}

void QuerySubPage::setListView(QList< QList<QString> > wordList)
{
    imageList->clear();
    imageList->setGeometry(170,250,685,300);
    imageList->show();
    //设置QListWidget的显示模式
    imageList->setViewMode(QListView::IconMode);
    //设置QListWidget中单元项的图片大小
    imageList->setIconSize(QSize(80,80));
    //设置QListWidget中单元项的间距
    imageList->setSpacing(12);
    //设置自动适应布局调整（Adjust适应，Fixed不适应），默认不适应
    imageList->setResizeMode(QListWidget::Adjust);
    //设置不能移动
    imageList->setMovement(QListWidget::Static);
    //去掉边框
    imageList->setFrameStyle(0);
    //设置鼠标样式
    imageList->setCursor(QCursor(Qt::PointingHandCursor));

    for (int j=0; j<wordList.count(); j++) {
        //定义QListWidgetItem对象
        QListWidgetItem *imageItem = new QListWidgetItem;
        //为单元项设置属性
        QPixmap image;
        //TODO
        // 添加BaseUrl
//        image.loadFromData(middle->getImageData(wordList[j][0]));
        QString imageFile = middle->getImageFile(wordList[j][0], "/zi/");
        if (imageFile.length() > 0)
            image.load(imageFile);
        else
            image.loadFromData(middle->getImageData(wordList[j][0]));
        imageItem->setIcon(QIcon(image));
        imageItem->setData(Qt::WhatsThisRole, wordList[j][1]);
        //重新设置单元项图片的宽度和高度
        imageItem->setSizeHint(QSize(80,80));
        //将单元项添加到QListWidget中
        imageList->addItem(imageItem);
    }
}

void QuerySubPage::setButtons()
{
    // 循环播放按钮
    btnPlay->setGeometry(852,380,109,101);
    QPixmap playP1(":/chaxun/xhbf_01.png");
    QPixmap playP2(":/chaxun/xhbf_02.png");
    btnPlay->setNormalImage(playP1);
    btnPlay->setPressImage(playP2);
    btnPlay->setCursor(QCursor(Qt::PointingHandCursor));

    btnBack->setGeometry(852,525,109,101);
    QPixmap backP1(":/chaxun/fh_04.png");
    QPixmap backP2(":/chaxun/fh_03.png");
    btnBack->setNormalImage(backP1);
    btnBack->setPressImage(backP2);
    btnBack->setCursor(QCursor(Qt::PointingHandCursor));
}

void QuerySubPage::itemClicked(QModelIndex index)
{
    imageList->blockSignals(true);
    QString word = index.data(Qt::WhatsThisRole).toString();

    wordPage->setWordPage(querySubPage, word, imageList, "", "");

    stackWidget->setCurrentWidget(nextPage);
}

void QuerySubPage::onBackClick()
{
    btnParent->blockSignals(false);
    stackWidget->setCurrentWidget(parentPage);
}

void QuerySubPage::onPlayClick()
{
    btnPlay->blockSignals(true);
    queryPlayPage->setQueryPlayPage(querySubPage, btnPlay, words);
    stackWidget->setCurrentWidget(nextPlayPage);
}
