#include "sfxspage.h"
#include <qmath.h>
#include <QDebug>

SfxsPage* SfxsPage::sfxsInstance = NULL;

SfxsPage::SfxsPage(QWidget *parent) : QWidget(parent)
{
    middle = Middle::getMiddleInstance();
}

SfxsPage* SfxsPage::getSfxsInstance()
{
    if (sfxsInstance == NULL)
        sfxsInstance = new SfxsPage();

    return sfxsInstance;
}

void SfxsPage::setWidget(QWidget *sfxs, QStackedWidget *stack, QWidget *next)
{
    sfxsPage = sfxs;
    nextPage = next;
    stackWidget = stack;

    table = new QTableWidget(sfxsPage);
    table->hide();
    btnBack = new ImageButton(sfxsPage);
    zsLabel = new QLabel(sfxsPage);
    lsLabel = new QLabel(sfxsPage);
    csLabel = new QLabel(sfxsPage);
    ksLabel = new QLabel(sfxsPage);
    xsLabel = new QLabel(sfxsPage);
}

void SfxsPage::setSfxsPage(QWidget *parent, ImageButton *btnP, QString word)
{
    parentPage = parent;
    btnParent = btnP;

    setBackGround();
    setButtons();
    setList(word);
}

void SfxsPage::setBackGround()
{
    sfxsPage->setStyleSheet("QWidget#sfxsPage{background-color: rgb(73, 127, 191);}");

    zsLabel->setGeometry(31,30,183,30);
    zsLabel->setStyleSheet("image:url(:/nianji/shufazidian/nj_zs.png)");

    lsLabel->setGeometry(214,30,183,30);
    lsLabel->setStyleSheet("image:url(:/nianji/shufazidian/nj_ls.png)");

    csLabel->setGeometry(397,30,183,30);
    csLabel->setStyleSheet("image:url(:/nianji/shufazidian/nj_cs.png)");

    ksLabel->setGeometry(580,30,183,30);
    ksLabel->setStyleSheet("image:url(:/nianji/shufazidian/nj_ks.png)");

    xsLabel->setGeometry(763,30,183,30);
    xsLabel->setStyleSheet("image:url(:/nianji/shufazidian/nj_xs.png)");
}

void SfxsPage::setList(QString word)
{
    QList<QVariant> list;
    QList<QString> zsList;
    QList<QString> lsList;
    QList<QString> csList;
    QList<QString> ksList;
    QList<QString> xsList;
    QList<int> count;

    list = middle->getCalligraphyWord(word);

    foreach (QVariant tmp, list) {
        QString type = tmp.toMap()["type"].toString();
        QString imageUrl = tmp.toMap()["img_url"].toString();
        if (type == "1") {
            ksList.append(imageUrl);
        } else if (type == "2") {
            xsList.append(imageUrl);
        } else if (type == "3") {
            csList.append(imageUrl);
        } else if (type == "4") {
            lsList.append(imageUrl);
        } else if (type == "5") {
            zsList.append(imageUrl);
        }
    }

    count.append(ksList.count());
    count.append(xsList.count());
    count.append(csList.count());
    count.append(lsList.count());
    count.append(zsList.count());

    qDebug() << __FUNCTION__ << count;
    qSort(count.begin(), count.end());
    qDebug() << __FUNCTION__ << count[4];

    setTableWidget(zsList, lsList, csList, ksList, xsList, count[4]);
}

void SfxsPage::setTableWidget(QList<QString> zsList, QList<QString> lsList, QList<QString> csList,
                              QList<QString> ksList, QList<QString> xsList, int count)
{
    table->clear();
    table->setGeometry(31,62,940,555);
    table->show();

    QHeaderView* headerViewV = table->verticalHeader();
    QHeaderView* headerViewH = table->horizontalHeader();
    headerViewV->setHidden(true);
    headerViewH->setHidden(true);

    table->setColumnCount(5);
    table->setRowCount(count);
    table->setColumnWidth(0,183);
    table->setColumnWidth(1,183);
    table->setColumnWidth(2,183);
    table->setColumnWidth(3,183);
    table->setColumnWidth(4,183);

    for (int i=0; i<zsList.count(); i++) {
        QLabel *imageLabel = new QLabel();
        imageLabel->setFixedSize(183,183);
        QPixmap image;
//        qDebug() << __FUNCTION__  << zsList[i];
        image.loadFromData(middle->getImageData(zsList[i]));
        image = image.scaled(QSize(183,183),Qt::KeepAspectRatio);
        imageLabel->setPixmap(image);

        table->setRowHeight(i, 183);
        table->setCellWidget(i, 0, imageLabel);
    }

    for (int i=0; i<lsList.count(); i++) {
        QLabel *imageLabel = new QLabel();
        imageLabel->setFixedSize(183,183);
        QPixmap image;
//        qDebug() << __FUNCTION__  << lsList[i];
        image.loadFromData(middle->getImageData(lsList[i]));
        image = image.scaled(QSize(183,183),Qt::KeepAspectRatio);
        imageLabel->setPixmap(image);

        table->setRowHeight(i, 183);
        table->setCellWidget(i, 1, imageLabel);
    }

    for (int i=0; i<csList.count(); i++) {
        QLabel *imageLabel = new QLabel();
        imageLabel->setFixedSize(183,183);
        QPixmap image;
//        qDebug() << __FUNCTION__  << csList[i];
        image.loadFromData(middle->getImageData(csList[i]));
        image = image.scaled(QSize(183,183),Qt::KeepAspectRatio);
        imageLabel->setPixmap(image);

        table->setRowHeight(i, 183);
        table->setCellWidget(i, 2, imageLabel);
    }

    for (int i=0; i<ksList.count(); i++) {
        QLabel *imageLabel = new QLabel();
        imageLabel->setFixedSize(183,183);
        QPixmap image;
//        qDebug() << __FUNCTION__  << ksList[i];
        image.loadFromData(middle->getImageData(ksList[i]));
        image = image.scaled(QSize(183,183),Qt::KeepAspectRatio);
        imageLabel->setPixmap(image);

        table->setRowHeight(i, 183);
        table->setCellWidget(i, 3, imageLabel);
    }

    for (int i=0; i<xsList.count(); i++) {
        QLabel *imageLabel = new QLabel();
        imageLabel->setFixedSize(183,183);
        QPixmap image;
//        qDebug() << __FUNCTION__  << xsList[i];
        image.loadFromData(middle->getImageData(xsList[i]));
        image = image.scaled(QSize(183,183),Qt::KeepAspectRatio);
        imageLabel->setPixmap(image);

        table->setRowHeight(i, 183);
        table->setCellWidget(i, 4, imageLabel);
    }

    connect(table, SIGNAL(clicked(QModelIndex)), this, SLOT(onIndexItemClicked(QModelIndex)));
}

void SfxsPage::setButtons()
{
    btnBack->setGeometry(855,590,109,101);
    QPixmap backP1(":/nianji/yiji/fh_03.png");
    QPixmap backP2(":/nianji/yiji/fh_04.png");
    btnBack->setNormalImage(backP1);
    btnBack->setPressImage(backP2);
    btnBack->setCursor(QCursor(Qt::PointingHandCursor));
    QObject::connect(btnBack, SIGNAL(clicked()), this, SLOT(onBackClick()));
}

void SfxsPage::onBackClick()
{
    btnParent->blockSignals(false);
    stackWidget->setCurrentWidget(parentPage);
}

void SfxsPage::onIndexItemClicked(QModelIndex index)
{
    QString str = index.data().toString();

    qDebug() << __FUNCTION__  << "str : " << str;
}
