#include "swjzpage.h"
#include <QDebug>
#include <qmath.h>
#include <QMessageBox>
#include <QTextCodec>
#include <QScrollBar>

SwjzPage* SwjzPage::swjzInstance = NULL;

SwjzPage::SwjzPage(QWidget *parent) : QWidget(parent)
{
    middle = Middle::getMiddleInstance();
    swjzImage = SwjzImage::getSwjzImageInstance();
    moo = new Phonon::MediaObject;
    device = new Phonon::AudioOutput;

    Phonon::createPath(moo, device);
}

SwjzPage* SwjzPage::getSwjzInstance()
{
    if (swjzInstance == NULL)
        swjzInstance = new SwjzPage();

    return swjzInstance;
}

void SwjzPage::setWidget(QWidget *swjz, QStackedWidget *stack, QWidget *next)
{
    swjzPage = swjz;
    nextPage = next;
    stackWidget = stack;

    // 页面元素初始化
    bgLabel = new QLabel(swjzPage);
    topImageLabel = new QLabel(swjzPage);
    pinyinLabel = new QLabel(swjzPage);
    ziLabel = new QLabel(swjzPage);

    zsText = new QLabel(swjzPage);
    zsImage = new MyLabel(swjzPage);
    lsText = new QLabel(swjzPage);
    lsImage = new MyLabel(swjzPage);
    ksText = new QLabel(swjzPage);
    ksImage = new MyLabel(swjzPage);
    xsText = new QLabel(swjzPage);
    xsImage = new MyLabel(swjzPage);
    csText = new QLabel(swjzPage);
    csImage = new MyLabel(swjzPage);
    descriptionImage = new MyLabel(swjzPage);
    zhushi = new QLabel(swjzPage);

    btnPlay = new ImageButton(swjzPage);
    btnBack = new ImageButton(swjzPage);

    connect(btnBack, SIGNAL(clicked()), this, SLOT(onBackClick()));
    connect(btnPlay, SIGNAL(clicked()), this, SLOT(onPlayClick()));
    connect(zsImage, SIGNAL(LabelClicked()), this, SLOT(onZsClick()));
    connect(lsImage, SIGNAL(LabelClicked()), this, SLOT(onLsClick()));
    connect(ksImage, SIGNAL(LabelClicked()), this, SLOT(onKsClick()));
    connect(xsImage, SIGNAL(LabelClicked()), this, SLOT(onXsClick()));
    connect(csImage, SIGNAL(LabelClicked()), this, SLOT(onCsClick()));
    connect(moo, SIGNAL(stateChanged(Phonon::State,Phonon::State)), this, SLOT(stateChanged(Phonon::State,Phonon::State)));
    connect(moo, SIGNAL(finished()), this, SLOT(finished()));
}

void SwjzPage::setSwjzPage(QWidget *parent, QString word, ImageButton *btnP,
                           MyLabel *videoLabel, QLabel *videoBgLabel, bool show_, QAxWidget *flash, bool play_)
{
    parentPage = parent;
    btnParent = btnP;
    parentVideoLabel = videoLabel;
    parentVideoBgLabel = videoBgLabel;
    show = show_;
    flashP = flash;
    play = play_;
    descriptionText = new QLabel(swjzPage);

    setBackGround();
    setButtons();
    setItems(word);
}

void SwjzPage::setBackGround()
{
    swjzPage->setStyleSheet("QWidget#swjzPage{background-color: rgb(73, 127, 191);}");
    topImageLabel->setGeometry(354,8,290,87);
    topImageLabel->setStyleSheet("image:url(:/nianji/swjz/swjz_tb.png)");

    bgLabel->setGeometry(40,50,920,540);
    bgLabel->setStyleSheet("background-color:white");

    QScrollArea *scrollArea = new QScrollArea(swjzPage);
    scrollArea->setGeometry(355,370,560,170);
    scrollArea->setFrameShape(QFrame::NoFrame);
    scrollArea->setStyleSheet("background-color:white");
    scrollArea->verticalScrollBar()->setStyleSheet("QScrollBar:vertical{width:6px;background:transparent;background-color:rgb(248, 248, 248);margin:0px,0px,0px,0px;padding-top:10px;padding-bottom:10px;}"
                       "QScrollBar::handle:vertical{width:6px;background:lightgray ;border-radius:5px;min-height:20px;}"
                       "QScrollBar::handle:vertical:hover{width:6px;background:gray;border-radius:5px;min-height:20px;}"
                       "QScrollBar::add-line:vertical{height:10px;width:10px;border-image:url(:/button/images/button/down.png);subcontrol-position:bottom;}"
                       "QScrollBar::sub-line:vertical{height:10px;width:10px;border-image:url(:/button/images/button/up.png);subcontrol-position:top;}"
                       "QScrollBar::add-line:vertical:hover{height:10px;width:10px;border-image:url(:/button/images/button/down_mouseDown.png);subcontrol-position:bottom;}"
                       "QScrollBar::sub-line:vertical:hover{height:10px;width:10px;border-image:url(:/button/images/button/up_mouseDown.png);subcontrol-position:top;}"
                       "QScrollBar::add-page:vertical,QScrollBar::sub-page:vertical{background:transparent;border-radius:5px;}");
    scrollArea->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);//去掉水平滚动条
    scrollArea->setWidget(descriptionText);
}

void SwjzPage::setItems(QString word)
{
    QList<QVariant> wordInfo = middle->getSwjzInfo(word);
    if (wordInfo.count() == 0)
        return;
    QString pinyin = wordInfo[0].toMap()["pinyin"].toString();
    zsUrl = wordInfo[0].toMap()["zhuanpicurl"].toString();
    lsUrl = wordInfo[0].toMap()["lipicurl"].toString();
    ksUrl = wordInfo[0].toMap()["kaipicurl"].toString();
    xsUrl = wordInfo[0].toMap()["xingpicurl"].toString();
    cSurl = wordInfo[0].toMap()["caopicurl"].toString();
    msUrl = wordInfo[0].toMap()["bigpicurl"].toString();
    QString description = wordInfo[0].toMap()["explain"].toString();
    QString url = wordInfo[0].toMap()["listen"].toString();
    // del
    url = middle->getBaseUrl() + url;
    audioUrl = encodeUrl(url);

    qDebug() << __FUNCTION__  << zsUrl << lsUrl << ksUrl<< xsUrl << cSurl << msUrl <<description;

    setWord(pinyin, word);
    setImages(zsUrl, lsUrl, ksUrl, xsUrl, cSurl, msUrl);
    setDescription(description);
}

void SwjzPage::setWord(QString pinyin, QString word)
{
    QFont fontPinyin, fontWord;

    fontPinyin.setFamily("HaiCi Dict");
    fontPinyin.setPixelSize(35);
    fontPinyin.setLetterSpacing(QFont::AbsoluteSpacing,1);
    fontPinyin.setBold(true);

    fontWord.setFamily("楷体");
    fontWord.setPixelSize(130);
    fontWord.setLetterSpacing(QFont::AbsoluteSpacing,1);
    fontWord.setWeight(40);

    pinyinLabel->setGeometry(114,118,170,40);
    pinyinLabel->setText(pinyin);
    pinyinLabel->setFont(fontPinyin);
    pinyinLabel->setAlignment(Qt::AlignCenter);

    ziLabel->setGeometry(114,170,170,160);
    ziLabel->setText(word);
    ziLabel->setFont(fontWord);
    ziLabel->setAlignment(Qt::AlignCenter);
}

void SwjzPage::setImages(QString zs, QString ls, QString ks, QString xs, QString cs, QString ms)
{
    QFont font;

    font.setFamily("楷体");
    font.setPixelSize(24);
    font.setLetterSpacing(QFont::AbsoluteSpacing,1);
    font.setWeight(40);

    zsImage->setGeometry(330,150,110,110);
    zsImage->setStyleSheet("border-width: 1px; border-style: solid; border-color: rgb(201,201,202);");
    QPixmap pixzs;
    if (middle->getClientMode()) {
        pixzs.load(middle->getImageFile(zs, "/swjz/"));
    } else {
        pixzs.loadFromData(middle->getImageData(zs));
    }
    pixzs = pixzs.scaled(110,110,Qt::IgnoreAspectRatio);
    zsImage->setPixmap(pixzs);
//    zsImage->setStyleSheet("image:url(:/nianji/swjz/zs.png)");
    zsText->setGeometry(330,270,110,50);
    zsText->setText("篆书");
    zsText->setAlignment(Qt::AlignCenter);
    zsText->setFont(font);

    lsImage->setGeometry(450,150,110,110);
    lsImage->setStyleSheet("border-width: 1px; border-style: solid; border-color: rgb(201,201,202);");
    QPixmap pixls;
    if (middle->getClientMode()) {
        pixls.load(middle->getImageFile(ls, "/swjz/"));
    } else {
        pixls.loadFromData(middle->getImageData(ls));
    }
    pixls = pixls.scaled(110,110,Qt::IgnoreAspectRatio);
    lsImage->setPixmap(pixls);
//    lsImage->setStyleSheet("image:url(:/nianji/swjz/ls.png)");
    lsText->setGeometry(450,270,110,50);
    lsText->setText("隶书");
    lsText->setAlignment(Qt::AlignCenter);
    lsText->setFont(font);

    ksImage->setGeometry(570,150,110,110);
    ksImage->setStyleSheet("border-width: 1px; border-style: solid; border-color: rgb(201,201,202);");
    QPixmap pixks;
    if (middle->getClientMode()) {
        pixks.load(middle->getImageFile(ks, "/swjz/"));
    } else {
        pixks.loadFromData(middle->getImageData(ks));
    }
    pixks = pixks.scaled(110,110,Qt::IgnoreAspectRatio);
    ksImage->setPixmap(pixks);
//    ksImage->setStyleSheet("image:url(:/nianji/swjz/ks.png)");
    ksText->setGeometry(570,270,110,50);
    ksText->setText("楷书");
    ksText->setAlignment(Qt::AlignCenter);
    ksText->setFont(font);

    xsImage->setGeometry(690,150,110,110);
    xsImage->setStyleSheet("border-width: 1px; border-style: solid; border-color: rgb(201,201,202);");
    QPixmap pixxs;
    if (middle->getClientMode()) {
        pixxs.load(middle->getImageFile(xs, "/swjz/"));
    } else {
        pixxs.loadFromData(middle->getImageData(xs));
    }
    pixxs = pixxs.scaled(110,110,Qt::IgnoreAspectRatio);
    xsImage->setPixmap(pixxs);
//    xsImage->setStyleSheet("image:url(:/nianji/swjz/xs.png)");
    xsText->setGeometry(690,270,110,50);
    xsText->setText("行书");
    xsText->setAlignment(Qt::AlignCenter);
    xsText->setFont(font);

    csImage->setGeometry(810,150,110,110);
    csImage->setStyleSheet("border-width: 1px; border-style: solid; border-color: rgb(201,201,202);");
    QPixmap pixcs;
    if (middle->getClientMode()) {
        pixcs.load(middle->getImageFile(cs, "/swjz/"));
    } else {
        pixcs.loadFromData(middle->getImageData(cs));
    }
    pixcs = pixcs.scaled(110,110,Qt::IgnoreAspectRatio);
    csImage->setPixmap(pixcs);
//    csImage->setStyleSheet("image:url(:/nianji/swjz/cs.png)");
    csText->setGeometry(810,270,110,50);
    csText->setText("草书");
    csText->setAlignment(Qt::AlignCenter);
    csText->setFont(font);

    descriptionImage->setGeometry(60,350,270,230);
    descriptionImage->setStyleSheet("border-width: 1px; border-style: solid; border-color: rgb(201,201,202);");
    QPixmap pixms;
    if (middle->getClientMode()) {
        pixms.load(middle->getImageFile(ms, "/swjz/"));
    } else {
        pixms.loadFromData(middle->getImageData(ms));
    }
    pixms = pixms.scaled(270,230,Qt::IgnoreAspectRatio);
    descriptionImage->setPixmap(pixms);
//    descriptionImage->setStyleSheet("image:url(:/nianji/swjz/ms.png)");
}

void SwjzPage::setDescription(QString description)
{
    QFont font, fontZhushi;

    font.setFamily("楷体");
    font.setPixelSize(25);
    font.setLetterSpacing(QFont::AbsoluteSpacing,1);
    font.setWeight(40);

    fontZhushi.setFamily("楷体");
    fontZhushi.setPixelSize(28);
    fontZhushi.setLetterSpacing(QFont::AbsoluteSpacing,1);
    fontZhushi.setWeight(50);

    zhushi->setGeometry(355,340,80,30);
    zhushi->setText("注释:");
    zhushi->setFont(fontZhushi);
    zhushi->raise();

    //description = "从古文字形上来看，这个字是由前后两个人构成的。所以，这个字本义是\"跟随\",\"从\"还有顺从、依从之义。后来，又给这个字加上了“彳”“止”，用来表示行动。在古书中，“从”是“纵”的通假，表示合纵。现代汉语中，“从”的本义未变，如随从、服从，都是要人们按照一定的规矩办事情。";

    int line =qCeil(double(description.length())/20);
    descriptionText->setGeometry(QRect(355, 370, 540, 35*line));
    descriptionText->setScaledContents(true);

    descriptionText->setWordWrap(true);
    descriptionText->setAlignment(Qt::AlignTop);
    descriptionText->setFont(font);
    descriptionText->setText(description);
}

void SwjzPage::setButtons()
{
    // 播放音频
    btnPlay->setGeometry(430,345,20,19);
    QPixmap playP(":/nianji/swjz/swjz_lb01.png");
    btnPlay->setNormalImage(playP);
    btnPlay->setPressImage(playP);
    btnPlay->setCursor(QCursor(Qt::PointingHandCursor));

    // 返回
    btnBack->setGeometry(825,540,109,101);
    QPixmap backP1(":/nianji/yiji/fh_03.png");
    QPixmap backP2(":/nianji/yiji/fh_04.png");
    btnBack->setNormalImage(backP1);
    btnBack->setPressImage(backP2);
    btnBack->setCursor(QCursor(Qt::PointingHandCursor));
}

void SwjzPage::onBackClick()
{
    moo->stop();
    btnParent->blockSignals(false);
    stackWidget->setCurrentWidget(parentPage);
    if (show) {
        parentVideoLabel->hide();
        parentVideoBgLabel->hide();
    }

    flashP->show();
    flashP->dynamicCall("Play()");

    pinyinLabel->clear();
    ziLabel->clear();
    zsText->clear();
    zsImage->clear();
    lsText->clear();
    lsImage->clear();
    ksText->clear();
    ksImage->clear();
    xsText->clear();
    xsImage->clear();
    csText->clear();
    csImage->clear();
    descriptionText->clear();
    descriptionImage->clear();
    zhushi->clear();
    delete descriptionText;
    descriptionText = NULL;
}

void SwjzPage::onPlayClick()
{
    qDebug() << __FUNCTION__  << moo->state() << Phonon::StoppedState << Phonon::LoadingState;
    qDebug() << __FUNCTION__  << audioUrl;

    if (moo->state() == Phonon::LoadingState || moo->state() == Phonon::StoppedState) {
        qDebug() << __FUNCTION__  << audioUrl;
        moo->setCurrentSource(audioUrl);
//        moo->setCurrentSource(QUrl("http://www.cguoguo.org.cn/jfx-web/attachment/ggxzvideo/zi/listen/%E5%8E%82.mp3"));
    }
    qDebug() << __FUNCTION__  << moo->state();
    if (moo->state() == Phonon::PlayingState) {
        moo->pause();
    } else {
        moo->play();
    }
}

void SwjzPage::stateChanged(Phonon::State newstate, Phonon::State oldstate)
{
    Q_UNUSED(oldstate);
    QPixmap P1(":/nianji/swjz/swjz_lb01.png");
    QPixmap P2(":/nianji/swjz/swjz_lb02.png");

    switch (newstate) {
        case Phonon::ErrorState:
            QMessageBox::warning(this, "播放异常", moo->errorString(), QMessageBox::Close);
            break;

        case Phonon::StoppedState:
            qDebug() << __FUNCTION__  << "stopped";
            btnPlay->setNormalImage(P1);
            btnPlay->setPressImage(P1);
            btnPlay->repaint();
            break;
        case Phonon::PlayingState:
            btnPlay->setNormalImage(P2);
            btnPlay->setPressImage(P2);
            btnPlay->repaint();
            break;
        case Phonon::PausedState:
            btnPlay->setNormalImage(P1);
            btnPlay->setPressImage(P1);
            btnPlay->repaint();
            break;
        case Phonon::BufferingState:
            break;
        case Phonon::LoadingState:
            break;
    }
}

void SwjzPage::finished()
{
    qDebug() << __FUNCTION__  << "finished";
    qDebug() << __FUNCTION__  << audioUrl;
    moo->setCurrentSource(audioUrl);
//    moo->setCurrentSource(QUrl("http://www.cguoguo.org.cn/jfx-web/attachment/ggxzvideo/zi/listen/%E5%8E%82.mp3"));
}

void SwjzPage::onZsClick()
{
    swjzImage->setSwjzImagePage(zsUrl);
    swjzImage->show();
}

void SwjzPage::onLsClick()
{
    swjzImage->setSwjzImagePage(lsUrl);
    swjzImage->show();
}

void SwjzPage::onKsClick()
{
    swjzImage->setSwjzImagePage(ksUrl);
    swjzImage->show();
}

void SwjzPage::onXsClick()
{
    swjzImage->setSwjzImagePage(xsUrl);
    swjzImage->show();
}

void SwjzPage::onCsClick()
{
    swjzImage->setSwjzImagePage(cSurl);
    swjzImage->show();
}

QString SwjzPage::encodeUrl(QString url)
{
    QStringList list = url.split("/");
    QString name = list[list.length()-1];

    QTextCodec *code = QTextCodec::codecForName("utf-8");//解决中文路径问题
    QByteArray byte = code->fromUnicode(name);
    byte = byte.toPercentEncoding();

    url = url.replace(name, byte);

    qDebug() << __FUNCTION__ << "url"  << url;
    return url;
}
