#include "syncthread.h"
#include <QDir>
#include <QCoreApplication>

#include <QIODevice>
#include <QNetworkRequest>
#include <QUrl>
#include <QEventLoop>
#include <QThread>
#include <QFile>
#include <QtScript>
#include <QList>
#include <windows.h>

#include "tool.h"

SyncThread::SyncThread()
{
    enabled = true;
    m_running = false;
    m_accessManager= new QNetworkAccessManager(this);
    m_NetManager = new QNetworkAccessManager(this);
    m_CookieJar = new QNetworkCookieJar(this);
    m_url_server = Tool::readXml("BaseUrl") + "/admin/";

    tool = Tool::getToolInstance();

    m_path_cache = QCoreApplication::applicationDirPath() + "/cache/";
    QDir dir_cache(m_path_cache);
    if(!dir_cache.exists())
    {
        dir_cache.mkpath(m_path_cache);
    }
}

void SyncThread::create()
{
    loginAPIServer("admin","123456");
    if(m_running)
    {
        return;
    }
    m_running =true;
    Tool tool;
    while(true)
    {
        if (!enabled)
        {
//            QThread::sleep(10);
            Sleep(10000);
            continue;
        }
        QList< QList<QString> > downloadInfo = tool.getDownloadInfo();
        qDebug() << "======downloadInfo:" <<downloadInfo;
        for(int i=0;i <downloadInfo.count(); i++)
        {
            QList<QString> tmp = downloadInfo[i];
            QString type = tmp[0];
            QString version = tmp[1];
            QString grade = tmp[2];
            QString level = tmp[3];
            if(type == "course")
            {
                if(grade == "")
                {
                    _sync_word(version, "1", "0");
                    _sync_word(version, "1", "1");
                    _sync_word(version, "2", "0");
                    _sync_word(version, "2", "1");
                    _sync_word(version, "3", "0");
                    _sync_word(version, "3", "1");
                    _sync_word(version, "4", "0");
                    _sync_word(version, "4", "1");
                    _sync_word(version, "5", "0");
                    _sync_word(version, "5", "1");
                    _sync_word(version, "6", "0");
                    _sync_word(version, "6", "1");
                    _sync_course(version, "1", "0");
                    _sync_course(version, "1", "1");
                    _sync_course(version, "2", "0");
                    _sync_course(version, "2", "1");
                    _sync_course(version, "3", "0");
                    _sync_course(version, "3", "1");
                    _sync_course(version, "4", "0");
                    _sync_course(version, "4", "1");
                    _sync_course(version, "5", "0");
                    _sync_course(version, "5", "1");
                    _sync_course(version, "6", "0");
                    _sync_course(version, "6", "1");
                }
                else if(level == "")
                {
                    _sync_word(version, grade, "0");
                    _sync_word(version, grade, "1");
                    _sync_course(version, grade, "0");
                    _sync_course(version, grade, "1");
                }
                else
                {
                    _sync_word(version, grade, level);
                    _sync_course(version, grade, level);
                }
            }
            else if(type == "sfkt")
                _sync_calligraphylesson(grade, level);
            else if(type == "xzzs")
                _sync_xzzs();
            else if(type == "ppbs")
                _sync_bushou();
            else if(type == "jbbh")
                _sync_strokes();
            else if(type == "sxzf")
                _sync_poemitem(grade);
        }
//        QThread::sleep(60);
        Sleep(60000);
    }
}

void SyncThread::enable()
{
     enabled = true;
}

void SyncThread::disable()
{
    enabled = false;
}

int SyncThread::loginAPIServer(QString user, QString passwd)
{
    QString url = m_url_server + "login?username=" + user + "&password=" + passwd;
    m_NetManager->setCookieJar(m_CookieJar);

    QByteArray json = getReply(url);
    QList<QNetworkCookie>  cookies = m_NetManager->cookieJar()->cookiesForUrl(QUrl(url));

    return 0;
}

QByteArray SyncThread::getReply(QString url)
{
    QNetworkRequest request;
    QByteArray line;
    QEventLoop eventLoop;
    request.setUrl(QUrl(url));
    request.setHeader(QNetworkRequest::ContentTypeHeader,"application/json");
    QNetworkReply* reply = m_NetManager->get(request);

    QObject::connect(m_NetManager, SIGNAL(finished(QNetworkReply*)), &eventLoop, SLOT(quit()));
    eventLoop.exec();

    line=reply->readAll();
    //qDebug() << "======result:" <<line;

    return line;
}

QByteArray SyncThread::getWord(QString version, QString grade, QString level)
{
    QString url = m_url_server + "word/getCacheList?" + "version=" + version + "&grade=" + grade + "&level="+level;
    QByteArray json = getReply(url);

    return json;
}

QByteArray SyncThread::getCourse(QString version, QString grade, QString level)
{
    QString url = m_url_server + "course/getCacheList?" + "version=" + version + "&grade=" + grade + "&level="+level;
    QByteArray json = getReply(url);

    return json;
}

QByteArray SyncThread::getCalligraphylesson(QString grade, QString level, QString type)
{
    QString url = m_url_server + "calligraphylesson/getCacheList?" + "&grade=" + grade + "&level="+level + "&type="+type;
    QByteArray json = getReply(url);

    return json;
}

QByteArray SyncThread::getXzzs()
{
    QString url = m_url_server + "xzzs/getCacheList";
    qDebug() << "======url:"<<url;
    QByteArray json = getReply(url);

    return json;
}

QByteArray SyncThread::getBushou()
{
    QString url = m_url_server + "bushou/getCacheList";
    QByteArray json = getReply(url);

    return json;
}

QByteArray SyncThread::getStrokes()
{
    QString url = m_url_server + "strokes/getCacheList";
    QByteArray json = getReply(url);

    return json;
}

QByteArray SyncThread::getPoemitem(QString grade)
{
    QString url = m_url_server + "poemitem/getCacheList?" + "&grade=" + grade;
    QByteArray json = getReply(url);

    return json;
}

void SyncThread::_sync_word(QString version, QString grade, QString level)
{
    qDebug() << "======start _sync_word" << version << grade << level;
    QByteArray ret;
    QList<QVariant> list;

    ret = getWord(version, grade, level);

    if (ret.length() > 0) {
        list = tool->jsonParser(ret, "CacheList");
    }

    for(int i=0;i <list.count(); i++)
    {
        QVariant tmp = list[i];

        QString url = tmp.toMap()["bs_video_url"].toString();
        int size = tmp.toMap()["bs_video_url_size"].toInt();
        int first = url.lastIndexOf ("/");
        QString filename = url.right(url.length()-first-1);

        if(filename != "") _download(url, m_path_cache+filename, size);

        url = tmp.toMap()["gb_video_url"].toString();
        size = tmp.toMap()["gb_video_url_size"].toInt();
        first = url.lastIndexOf ("/");
        filename = url.right(url.length()-first-1);

        if(filename != "") _download(url, m_path_cache+filename, size);

        url = tmp.toMap()["mb_video_url"].toString();
        size = tmp.toMap()["mb_video_url_size"].toInt();
        first = url.lastIndexOf ("/");
        filename = url.right(url.length()-first-1);

        if(filename != "") _download(url, m_path_cache+filename, size);

        url = tmp.toMap()["qb_video_url"].toString();
        size = tmp.toMap()["qb_video_url_size"].toInt();
        first = url.lastIndexOf ("/");
        filename = url.right(url.length()-first-1);

        if(filename != "") _download(url, m_path_cache+filename, size);
    }
}

void SyncThread::_sync_course(QString version, QString grade, QString level)
{
    qDebug() << "======start _sync_course";
    QByteArray ret;
    QList<QVariant> list;

    ret = getCourse(version, grade, level);
    if (ret.length() > 0) {
        list = tool->jsonParser(ret, "CacheList");
    }

    for(int i=0; i<list.count(); i++)
    {
        QVariant tmp = list[i];

        QString url = tmp.toMap()["qwfd_video"].toString();
        int size = tmp.toMap()["qwfd_video_size"].toInt();
        int first = url.lastIndexOf ("/");
        QString filename = url.right(url.length()-first-1);

        if(filename != "") _download(url, m_path_cache+filename, size);

        url = tmp.toMap()["qwgd_video"].toString();
        size = tmp.toMap()["qwgd_video_size"].toInt();
        first = url.lastIndexOf ("/");
        filename = url.right(url.length()-first-1);

        if(filename != "") _download(url, m_path_cache+filename, size);

        url = tmp.toMap()["qwyd_video"].toString();
        size = tmp.toMap()["qwyd_video_size"].toInt();
        first = url.lastIndexOf ("/");
        filename = url.right(url.length()-first-1);

        if(filename != "") _download(url, m_path_cache+filename, size);
    }
}

void SyncThread::_sync_calligraphylesson(QString grade, QString level)
{
    qDebug() << "======start _sync_calligraphylesson";
    QByteArray ret;
    QList<QVariant> list;

    ret = getCalligraphylesson(grade, level, "0");
    if (ret.length() > 0) {
        list = tool->jsonParser(ret, "CacheList");
    }

    for(int i=0; i<list.count(); i++)
    {
        QVariant tmp = list[i];

        QString url = tmp.toMap()["video_url"].toString();
        int size = tmp.toMap()["video_size"].toInt();
        int first = url.lastIndexOf ("/");
        QString filename = url.right(url.length()-first-1);

        if(filename != "") _download(url, m_path_cache+filename, size);
    }

    ret = getCalligraphylesson(grade, level, "1");
    if (ret.length() > 0) {
        list = tool->jsonParser(ret, "CacheList");
    }

    for(int i=0; i<list.count(); i++)
    {
        QVariant tmp = list[i];

        QString url = tmp.toMap()["video_url"].toString();
        int size = tmp.toMap()["video_size"].toInt();
        int first = url.lastIndexOf ("/");
        QString filename = url.right(url.length()-first-1);

        if(filename != "") _download(url, m_path_cache+filename, size);
    }
}
void SyncThread::_sync_xzzs()
{
    qDebug() << "======start _sync_xzzs";
    QByteArray ret;
    QList<QVariant> list;

    ret = getXzzs();
    if (ret.length() > 0) {
        list = tool->jsonParser(ret, "CacheList");
    }
    qDebug() << list;

    for(int i=0; i<list.count(); i++)
    {
        QVariant tmp = list[i];

        QString url = tmp.toMap()["video_url"].toString();
        int size = tmp.toMap()["video_size"].toInt();
        int first = url.lastIndexOf ("/");
        QString filename = url.right(url.length()-first-1);

        if(filename != "") _download(url, m_path_cache+filename, size);
    }
}

void SyncThread::_sync_bushou()
{
    qDebug() << "======start _sync_bushou";

    QByteArray ret;
    QList<QVariant> list;

    ret = getBushou();
    if (ret.length() > 0) {
        list = tool->jsonParser(ret, "CacheList");
    }
    qDebug() << list;

    for(int i=0; i<list.count(); i++)
    {
        QVariant tmp = list[i];

        QString url = tmp.toMap()["video_url"].toString();
        int size = tmp.toMap()["video_size"].toInt();
        int first = url.lastIndexOf ("/");
        QString filename = url.right(url.length()-first-1);

        if(filename != "") _download(url, m_path_cache+filename, size);
    }
}

void SyncThread::_sync_strokes()
{
    qDebug() << "======start _sync_strokes";
    QByteArray ret;
    QList<QVariant> list;

    ret = getStrokes();
    if (ret.length() > 0) {
        list = tool->jsonParser(ret, "CacheList");
    }
    qDebug() << list;

    for(int i=0; i<list.count(); i++)
    {
        QVariant tmp = list[i];

        QString url = tmp.toMap()["fb_video_url"].toString();
        int size = tmp.toMap()["fb_video_url_size"].toInt();
        int first = url.lastIndexOf ("/");
        QString filename = url.right(url.length()-first-1);

        if(filename != "") _download(url, m_path_cache+filename, size);

        url = tmp.toMap()["gb_video_url"].toString();
        size = tmp.toMap()["gb_video_url_size"].toInt();
        first = url.lastIndexOf ("/");
        filename = url.right(url.length()-first-1);

        if(filename != "") _download(url, m_path_cache+filename, size);

        url = tmp.toMap()["mb_video_url"].toString();
        size = tmp.toMap()["mb_video_url_size"].toInt();
        first = url.lastIndexOf ("/");
        filename = url.right(url.length()-first-1);

        if(filename != "") _download(url, m_path_cache+filename, size);
    }
}
void SyncThread::_sync_poemitem(QString grade)
{
    qDebug() << "======start _sync_poemitem";
    QByteArray ret;
    QList<QVariant> list;

    ret = getPoemitem(grade);
    if (ret.length() > 0) {
        list = tool->jsonParser(ret, "CacheList");
    }
    qDebug() << list;

    for(int i=0; i<list.count(); i++)
    {
        QVariant tmp = list[i];

        QString url = tmp.toMap()["video_url"].toString();
        int size = tmp.toMap()["video_size"].toInt();
        int first = url.lastIndexOf ("/");
        QString filename = url.right(url.length()-first-1);

        if(filename != "") _download(url, m_path_cache+filename, size);
    }
}

void SyncThread::_download(QString strurl, QString strpath, int size)
{
    if (!enabled) return;
    if (size <= 0) return;
    QFile file(strpath);
    if (file.exists() && file.size() == size) return;

    if(strurl.startsWith("/")) strurl = strurl.right(strurl.length()-1);
    QString server_url = m_url_server;
    if(server_url.endsWith("admin/")) server_url = server_url.left(server_url.length()-6);

    QUrl url = QUrl(server_url + strurl);
    qDebug() << "======start download:" << url;
    QNetworkRequest request;
    request.setUrl(url);
    m_reply=m_accessManager->get(request);
    QEventLoop eventLoop;

//    connect(m_reply, &QNetworkReply::finished, &eventLoop, &QEventLoop::quit);
//    connect(m_reply, SIGNAL(finished(QNetworkReply*)), &eventLoop, SLOT(quit()));
//    QObject::connect(m_reply, SIGNAL(finished(QNetworkReply*)), &eventLoop, SLOT(quit()));
    connect(m_reply, SIGNAL(finished()), &eventLoop, SLOT(quit()));
    eventLoop.exec(QEventLoop::ExcludeUserInputEvents);

//    QNetworkRequest request;
//    QByteArray line;
//    QEventLoop eventLoop;
//    request.setUrl(QUrl(url));
//    request.setHeader(QNetworkRequest::ContentTypeHeader,"application/json");
//    QNetworkReply* reply = m_NetManager->get(request);

//    QObject::connect(m_NetManager, SIGNAL(finished(QNetworkReply*)), &eventLoop, SLOT(quit()));
//    eventLoop.exec();

    file.remove();
    file.open(QIODevice::WriteOnly);
    file.write(m_reply->readAll());
    file.close();
    m_reply->deleteLater();
    m_reply = NULL;
}


