#include "szzbpage.h"
#include <QDebug>
#include <QHBoxLayout>
#include <QPushButton>
#include <QtCore/qmath.h>

SzzbPage* SzzbPage::szzbInstance = NULL;

SzzbPage::SzzbPage(QObject *parent) : QObject(parent)
{
    middle = Middle::getMiddleInstance();
    szzbSub = SzzbSubPage::getSzzbSubInstance();
    load = new Loading();
}

SzzbPage* SzzbPage::getSzzbInstance()
{
    if (szzbInstance == NULL)
        szzbInstance = new SzzbPage();

    return szzbInstance;
}

void SzzbPage::setWidget(QWidget *szzb, QStackedWidget *stack, QWidget *next)
{
    szzbPage = szzb;
    nextPage = next;
    stackWidget = stack;

    btnBack = new ImageButton(szzbPage);
    indexWidget = new QTableWidget(szzbPage);
    indexWidget->hide();
    listWidget = new QListWidget(szzbPage);
    listWidget->hide();
    btnPinyin = new ImageButton(szzbPage);
    btnBushou = new ImageButton(szzbPage);

    backGroundLabel = new QLabel(szzbPage);
    btnQuery = new ImageButton(szzbPage);
    rectLabel = new QLabel(szzbPage);
    queryLine = new QLineEdit(szzbPage);

    connect(indexWidget, SIGNAL(clicked(QModelIndex)), this, SLOT(onIndexItemClicked(QModelIndex)));
    connect(queryLine, SIGNAL(returnPressed()), this, SLOT(onQueryClick()));
    connect(btnPinyin, SIGNAL(clicked()), this, SLOT(onPinyinClick()));
    connect(btnBushou, SIGNAL(clicked()), this, SLOT(onBushouClick()));
    connect(btnBack, SIGNAL(clicked()), this, SLOT(onBackClick()));
    connect(btnQuery, SIGNAL(clicked()), this, SLOT(onQueryClick()));
}

void SzzbPage::setSzzbPage(QWidget *parent, ImageButton *btnP, QString grade_, QString level_)
{
    parentPage = parent;
    btnParent = btnP;
    parentPage = parent;
    grade = grade_;
    level = level_;
    type = "pinyin";

    if (grade_.length() != 0) {
        hideQueryItem();
    } else {
        showQueryItem();
    }

    setBackGround();
    setQueryItem();
    setButtons();
    setIndexList();
    setItemStyle();
    setItems("");
}

void SzzbPage::setBackGround()
{
    szzbPage->setStyleSheet("QWidget#szzbPage{background-color: rgb(73, 127, 191);}");
//    szzbPage->setStyleSheet("background-color: rgb(73, 127, 191);");
//    backGroundLabel->setGeometry(0,0,1000,700);
//    backGroundLabel->setStyleSheet("QLabel#backGroundLabel{background-color: rgb(73, 127, 191);}");
}

void SzzbPage::setIndexList()
{
    if (type == "pinyin"){
        indexListPinyin = middle->getPinyinIndexList(grade, level);
    } else {
        indexListBushou = middle->getBushouIndexList(grade, level);
        qDebug() << __FUNCTION__  << "indexList:" << indexListBushou;
    }

    indexWidget->setGeometry(0,0,50,640);
    indexWidget->show();
    indexWidget->clearSelection();

    QHeaderView* headerViewV = indexWidget->verticalHeader();
    QHeaderView* headerViewH = indexWidget->horizontalHeader();
    headerViewV->setHidden(true);
    headerViewH->setHidden(true);

    if (type == "pinyin"){
        indexWidget->setRowCount(indexListPinyin.count());
    } else {
        indexWidget->setRowCount(indexListBushou.count());
    }
    indexWidget->setColumnCount(1);
    indexWidget->setColumnWidth(0,50);

    indexWidget->setFrameShape(QFrame::NoFrame); //设置边框
    indexWidget->setEditTriggers(QAbstractItemView::NoEditTriggers);
    indexWidget->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);//去掉水平滚动条
    indexWidget->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);  //去掉垂直滚动条
    indexWidget->setAlternatingRowColors(true);
    indexWidget->setFocusPolicy(Qt::NoFocus); //去除选中虚线框
    indexWidget->setCursor(QCursor(Qt::PointingHandCursor));

    indexWidget->setStyleSheet("QTableWidget{font-size: 20px; font-family:宋体}"
                               "QTableWidget::item:hover{background-color: rgb(251,151,5)}"
                               "QTableWidget::item:hover{color: white}"
                               "QTableWidget{background-color:white}");

    if (type == "pinyin") {
        for (int i=0; i<indexListPinyin.count(); i++) {
            indexWidget->setRowHeight(i, 46);
            QVariant tmp = indexListPinyin[i];
            indexWidget->setItem(i, 0, new QTableWidgetItem(tmp.toMap()["index"].toString()));
            indexWidget->item(i,0)->setTextAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
        }
    } else {
        for (int i=0; i<indexListBushou.count(); i++) {
            indexWidget->setRowHeight(i, 46);
            QVariant tmp = indexListBushou[i];
            indexWidget->setItem(i, 0, new QTableWidgetItem(tmp.toMap()["bs_bihua"].toString() + "画"));
            indexWidget->item(i,0)->setTextAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
        }
    }
}

void SzzbPage::setItemStyle()
{
    listWidget->setGeometry(100,60,850,535);
    listWidget->show();
    listWidget->setFocusPolicy(Qt::NoFocus);
    listWidget->setMovement(QListWidget::Static);
    listWidget->setSpacing(5);
    // 去掉滚动条
    listWidget->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    listWidget->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    //去掉选中的背景框
    listWidget->setSelectionMode(QAbstractItemView::NoSelection);
    //去掉边框
    listWidget->setFrameStyle(0);
    listWidget->setStyleSheet("background-color:transparent");
}

void SzzbPage::setItems(QString index)
{
    listWidget->clear();
    load->show();

    QList<QVariant> tmp;
    QStringList list;

    if (index.length() > 0) {
        if (type == "pinyin") {
            tmp = middle->getPinyinList(index, grade, level);

            list.clear();
            foreach (QVariant index, tmp) {
                list.append(index.toMap()["wd_pinyin"].toString());
            }
        }
        if (type == "bushou") {
            tmp = middle->getBushouList(index, grade, level);

            list.clear();
            foreach (QVariant index, tmp) {
                list.append(index.toMap()["bushou"].toString());
            }
        }

        setSubItem(list, index);
    } else {
        if (type == "pinyin") {
            QString index_;
            for (int i=0; i< indexListPinyin.count(); i++) {
                index_ = indexListPinyin[i].toMap()["index"].toString();
                tmp = middle->getPinyinList(index_, grade, level);
                list.clear();
                foreach (QVariant index, tmp) {
                    list.append(index.toMap()["wd_pinyin"].toString());
                }

                setSubItem(list, index_);
            }
        } else {
            QString index_;
            for (int i=0; i< indexListBushou.count(); i++) {
                index_ = indexListBushou[i].toMap()["bs_bihua"].toString();
                tmp = middle->getBushouList(index_, grade, level);
                list.clear();
                foreach (QVariant index, tmp) {
                    list.append(index.toMap()["bushou"].toString());
                }

                setSubItem(list, index_);
            }
        }
    }

    load->close();
}

void SzzbPage::setSubItem(QStringList list, QString index)
{
    int height;
    int tmp = qCeil(double(list.count())/7);
    QListWidgetItem *listItem = new QListWidgetItem();
    ItemWidget *widget = new ItemWidget;

    if (tmp == 0)
        height = 0;
    else
        height = 60 + tmp*55;

    widget->setWidget(840, height, list, type, grade, level, index, szzbPage, nextPage, stackWidget);
    listItem->setSizeHint(QSize(840,height));

    listWidget->addItem(listItem);
    listWidget->setItemWidget(listItem, widget);
}

void SzzbPage::setQueryItem()
{
    QPalette pa;
    pa.setColor(QPalette::Text,QColor(255,255,255));

    btnQuery->setGeometry(770,37,19,19);
    btnQuery->raise();
    QPixmap queryP1(":szzb/ss.png");
    btnQuery->setNormalImage(queryP1);
    btnQuery->setPressImage(queryP1);
    btnQuery->setCursor(QCursor(Qt::PointingHandCursor));

    queryLine->setGeometry(570,34,180,25);
    queryLine->setFont(QFont( "楷体" , 11 , QFont::Bold));
//    queryLine->setPalette(pa);
    queryLine->setPlaceholderText("请输入要查询的拼音");
    queryLine->setStyleSheet("background-color:transparent;border-width:0;border-style:outset");

    rectLabel->setGeometry(530,30,300,33);
    rectLabel->setStyleSheet("border-width: 2px; border-style: solid; border-radius:12px; border-color: white;");
}

void SzzbPage::setButtons()
{
    // 拼音查字
    btnPinyin->setGeometry(115,27,120,40);
    QPixmap pinyinP1(":szzb/pycz02.png");
    QPixmap pinyinP2(":szzb/pycz02.png");
    btnPinyin->setNormalImage(pinyinP1);
    btnPinyin->setPressImage(pinyinP2);
    btnPinyin->setCursor(QCursor(Qt::PointingHandCursor));
    btnPinyin->setFocus();

    // 部首查字
    btnBushou->setGeometry(250,34,120,33);
    QPixmap bushouP1(":szzb/bscz01.png");
    QPixmap bushouP2(":szzb/bscz01.png");
    btnBushou->setNormalImage(bushouP1);
    btnBushou->setPressImage(bushouP2);
    btnBushou->setCursor(QCursor(Qt::PointingHandCursor));

    // 返回
    btnBack->setGeometry(935,470,35,103);
    QPixmap backP1(":szzb/fh.png");
    QPixmap backP2(":szzb/fh2.png");
    btnBack->setNormalImage(backP1);
    btnBack->setPressImage(backP2);
    btnBack->setCursor(QCursor(Qt::PointingHandCursor));
}

void SzzbPage::onIndexItemClicked(QModelIndex index)
{
    QString index_;
    QString str = index.data().toString();
//    qDebug() << __FUNCTION__  << str.mid(0,str.length()-1);
    if (str.length() == 1) {
        index_ = str;
    } else {
        index_ = str.mid(0,str.length()-1);
    }
    qDebug() << __FUNCTION__  << "index:" << index_;
    setItems(QString(index_));
}

void SzzbPage::onPinyinClick()
{
    QPixmap pinyinP(":szzb/pycz02.png");
    QPixmap bushouP(":szzb/bscz01.png");

    btnPinyin->setGeometry(115,27,120,40);
    btnPinyin->setNormalImage(pinyinP);
    btnPinyin->setPressImage(pinyinP);
    btnPinyin->repaint();

    btnBushou->setGeometry(250,34,120,33);
    btnBushou->setNormalImage(bushouP);
    btnBushou->setPressImage(bushouP);
    btnBushou->repaint();

//    btnPinyin->blockSignals(true);
//    btnBushou->blockSignals(false);
    queryLine->clear();
    queryLine->clearFocus();
    queryLine->setPlaceholderText("请输入要查询的拼音");
    type = "pinyin";
    setIndexList();
    setItems("");
}

void SzzbPage::onBushouClick()
{
    QPixmap pinyinP(":szzb/pycz01.png");
    QPixmap bushouP(":szzb/bscz02.png");

    btnPinyin->setGeometry(115,34,120,33);
    btnPinyin->setNormalImage(pinyinP);
    btnPinyin->setPressImage(pinyinP);
    btnPinyin->repaint();

    btnBushou->setGeometry(250,27,120,40);
    btnBushou->setNormalImage(bushouP);
    btnBushou->setPressImage(bushouP);
    btnBushou->repaint();

//    btnBushou->blockSignals(true);
//    btnPinyin->blockSignals(false);
    queryLine->clear();
    queryLine->clearFocus();
    queryLine->setPlaceholderText("请输入要查询的部首");
    type = "bushou";
    setIndexList();
    setItems("");
}

void SzzbPage::onQueryClick()
{
    QList<QVariant> indexList;
    QString text = queryLine->text();
    text.remove(QRegExp("\\s"));

    if (text.length() == 0) {
        if (type == "pinyin")
            middle->errorMBox("请输入拼音");
        else
            middle->errorMBox("请输入部首");
        return;
    }

    if (type == "pinyin"){
        indexList = middle->getZiBihuaIndexList(text, grade, level);
    } else {
        indexList = middle->getWbBihuaIndexList(text, grade, level);
    }

    if (indexList.count() == 0) {
        middle->errorMBox("未查询到: " + text);
        return;
    }

    stackWidget->setCurrentWidget(nextPage);
    szzbSub->setSzzbSubPage(szzbPage, listWidget, text, type, grade, level);
}

void SzzbPage::onBackClick()
{
    btnParent->blockSignals(false);
    btnPinyin->blockSignals(false);
    btnBushou->blockSignals(false);
    queryLine->clear();
    stackWidget->setCurrentWidget(parentPage);
}

void SzzbPage::hideQueryItem()
{
    btnBushou->hide();
    btnPinyin->hide();
    btnQuery->hide();
    rectLabel->hide();
    queryLine->hide();
}

void SzzbPage::showQueryItem()
{
    btnBushou->show();
    btnPinyin->show();
    btnQuery->show();
    rectLabel->show();
    queryLine->show();
}
