#include "tool.h"
//#include "parser.h"
#include <QtNetwork/QtNetwork>
#include <QtNetwork/QNetworkInterface>
#include <QtXml/QDomDocument>
#include <QtScript>
#include <QList>
#include <QSettings>
#include <QDesktopServices>
#include <QDebug>

Tool* Tool::toolInstance = NULL;

Tool::Tool(QObject *parent) : QObject(parent)
{
}

Tool* Tool::getToolInstance()
{
    if (toolInstance == NULL) {
        toolInstance = new Tool();
    }
    return toolInstance;
}

QList<QVariant> Tool::jsonParser(QByteArray sourse, QString key)
{
    QList<QVariant> ret;

    QScriptEngine engine;
    QScriptValue sc =  engine.evaluate("value=" + QString(sourse));

    if (sc.property(key).isArray())
    {
        ret = sc.property(key).toVariant().toList();
    }

    return ret;
}

QString Tool::jsonParserString(QByteArray sourse, QString key)
{
    QScriptEngine engine;
    QScriptValue sc =  engine.evaluate("value=" + QString(sourse));
    QString result = sc.property(key).toString();
    qDebug() << __FUNCTION__  << "jsonParserString" << result;

    if (result.length() > 0)
        return result;

    return "-1";
}

QString Tool::getMacAddress()
{
    QList<QNetworkInterface> nets = QNetworkInterface::allInterfaces();
    qDebug() << __FUNCTION__  << nets;
    int i = 0;
    QString macAddr = "";
    foreach(QNetworkInterface ni,nets)
    {
        i++;
        //qDebug() << __FUNCTION__ <<i<<ni.name()<<ni.hardwareAddress()<<ni.humanReadableName();
        macAddr = ni.hardwareAddress();
        qDebug() << __FUNCTION__  << ni.name();
        qDebug() << __FUNCTION__  << ni.humanReadableName();
        qDebug() << __FUNCTION__  << ni.isValid();
//        qDebug() << __FUNCTION__  << ni.addressEntries();
//        if(ni.name().toLower().contains("ethernet") && !ni.humanReadableName().toLower().contains("vmware"))
//        if(!ni.humanReadableName().toLower().contains("Bluetooth") && !ni.humanReadableName().toLower().contains("Loopback") &&
//                !ni.humanReadableName().toLower().contains("vmware"))
//        {
//            qDebug() << __FUNCTION__ << "This is our wannt " << ni.name()<<ni.hardwareAddress()<<ni.humanReadableName();
//            break;
//        }
        if (macAddr.length() == 17)
            break;
    }
    return macAddr;
}

QString Tool::getMacAddressList()
{
    QList<QNetworkInterface> nets = QNetworkInterface::allInterfaces();
    int i = 0;
    QString macAddr = "";
    QString macAddrList = "";
    foreach(QNetworkInterface ni,nets)
    {
        i++;
        macAddr = ni.hardwareAddress();
        if (macAddr.length() == 17) {
            macAddrList = macAddrList + macAddr + ",";
        }
    }

    return macAddrList.mid(0,macAddrList.length()-1);
}

void Tool::writeXml(QString tag, QString value) {
    QFile file(QCoreApplication::applicationDirPath()+"/gup.xml");
    if(!file.open(QFile::ReadOnly))
        return;

    //增加一个一级子节点以及元素
    QDomDocument doc;
    if(!doc.setContent(&file))
    {
        file.close();
        return;
    }
    file.close();

    QDomElement root=doc.documentElement();
    QDomElement url=doc.createElement(tag);
    QDomText text;
    text=doc.createTextNode(value);
    url.appendChild(text);
    root.appendChild(url);

    if(!file.open(QFile::WriteOnly|QFile::Truncate)) //先读进来，再重写，如果不用truncate就是在后面追加内容，就无效了
        return;
    //输出到文件
    QTextStream out_stream(&file);
    doc.save(out_stream,4); //缩进4格
    file.close();
}

QString Tool::readXml(QString tag)
{
    QFile file(QCoreApplication::applicationDirPath()+"/gup.xml");
    if(!file.open(QFile::ReadOnly))
        return NULL;

    QDomDocument doc;
    if(!doc.setContent(&file))
    {
        file.close();
        return NULL;
    }
    file.close();

    QDomElement root=doc.documentElement(); //返回根节点
    qDebug() << __FUNCTION__ <<root.nodeName();
    QDomNode node=root.firstChild(); //获得第一个子节点
    while(!node.isNull())  //如果节点不空
    {
        if(node.isElement()) //如果节点是元素
        {
            QDomElement e=node.toElement(); //转换为元素，注意元素和节点是两个数据结构，其实差不多
            if (e.tagName() == tag) {
                return e.text();
            }
        }
        node=node.nextSibling(); //下一个兄弟节点,nextSiblingElement()是下一个兄弟元素，都差不多
    }

    return NULL;
}

void Tool::setDownloadInfo(QString type, QString version, QString grade, QString level) {
    QFile file(QCoreApplication::applicationDirPath()+"/gup.xml");
    if(!file.open(QFile::ReadOnly))
        return;

    QDomDocument doc;
    if(!doc.setContent(&file))
    {
        file.close();
        return;
    }
    file.close();

    QDomElement root=doc.documentElement();
    QDomElement info=doc.createElement("Download");
    info.setAttribute("type", type);
    info.setAttribute("version", version);
    info.setAttribute("grade", grade);
    info.setAttribute("level", level);
    root.appendChild(info);

    if(!file.open(QFile::WriteOnly|QFile::Truncate)) //先读进来，再重写，如果不用truncate就是在后面追加内容，就无效了
        return;
    QTextStream out_stream(&file);
    doc.save(out_stream,4); //缩进4格
    file.close();
}

void Tool::unSetDownloadInfo(QString type, QString version, QString grade, QString level) {
    QFile file(QCoreApplication::applicationDirPath()+"/gup.xml");
    if(!file.open(QFile::ReadOnly))
        return;

    QDomDocument doc;
    if(!doc.setContent(&file))
    {
        file.close();
        return;
    }
    file.close();

    QDomElement root=doc.documentElement();
    QDomNodeList list=doc.elementsByTagName("Download");

    for(int i=0;i<list.count();i++)
    {
        QDomElement e=list.at(i).toElement();
        qDebug() << __FUNCTION__  << e.attribute("type");
        if(e.attribute("type")==type && e.attribute("version")==version &&
                e.attribute("grade")== grade && e.attribute("level")==level)
            root.removeChild(list.at(i));
    }

    if(!file.open(QFile::WriteOnly|QFile::Truncate))
        return;
    QTextStream out_stream(&file);
    doc.save(out_stream,4);
    file.close();
}

QList< QList<QString> > Tool::getDownloadInfo()
{
    QList< QList<QString> > download;

    QFile file(QCoreApplication::applicationDirPath()+"/gup.xml");
    if(!file.open(QFile::ReadOnly))
        return download;

    QDomDocument doc;
    if(!doc.setContent(&file))
    {
        file.close();
        return download;
    }
    file.close();

    QDomElement root=doc.documentElement(); //返回根节点
    QDomNode node=root.firstChild(); //获得第一个子节点
    while(!node.isNull())  //如果节点不空
    {
        if(node.isElement()) //如果节点是元素
        {
            QDomElement e=node.toElement();
            if (e.tagName() == "Download") {
                QList<QString> tmp;
                tmp.append(e.attribute("type"));
                tmp.append(e.attribute("version"));
                tmp.append(e.attribute("grade"));
                tmp.append(e.attribute("level"));
                download.append(tmp);
            }
        }
        node=node.nextSibling(); //下一个兄弟节点,nextSiblingElement()是下一个兄弟元素，都差不多
    }

    return download;
}

void Tool::updateXml(QString tag, QString value)
{
    QFile file(QCoreApplication::applicationDirPath()+"/gup.xml");
    if(!file.open(QFile::ReadOnly))
        return;

    QDomDocument doc;
    if(!doc.setContent(&file))
    {
        file.close();
        return;
    }
    file.close();

    QDomElement root=doc.documentElement(); //返回根节点
    qDebug() << __FUNCTION__ <<root.nodeName();
    QDomNode node=root.firstChild(); //获得第一个子节点
    while(!node.isNull())  //如果节点不空
    {
        if(node.isElement()) //如果节点是元素
        {
            QDomElement e=node.toElement(); //转换为元素，注意元素和节点是两个数据结构，其实差不多
            if (e.tagName() == tag) {
                QDomNode oldnode = e.firstChild();     //标签之间的内容作为节点的子节点出现，得到原来的子节点
                e.firstChild().setNodeValue(value);   //用提供的value值来设置子节点的内容
                QDomNode newnode = e.firstChild();     //值修改过后
                e.replaceChild(newnode,oldnode);
            }
        }
        node=node.nextSibling(); //下一个兄弟节点,nextSiblingElement()是下一个兄弟元素，都差不多
    }

    if(!file.open(QFile::WriteOnly|QFile::Truncate)) //先读进来，再重写，如果不用truncate就是在后面追加内容，就无效了
        return;
    //输出到文件
    QTextStream out_stream(&file);
    doc.save(out_stream,4); //缩进4格
    file.close();
}

void Tool::writeIni(QString key, QString value)
{
    QString file = QDesktopServices::storageLocation(QDesktopServices::DataLocation) + "\\dxpy.ini";
    QSettings *config = new QSettings(file, QSettings::IniFormat);

    config->setValue(key, value);
}

QString Tool::readIni(QString key)
{
    QString file = QDesktopServices::storageLocation(QDesktopServices::DataLocation) + "\\dxpy.ini";
    QSettings *config = new QSettings(file, QSettings::IniFormat);

    QString value = config->value(key).toString();

    return value;
}
