#include "video.h"
#include <QDebug>
#include <qmath.h>


Video::Video(QWidget *parent) : QWidget(parent)
{
    m_vlcInstance = NULL;
    m_media = NULL;
    m_mediaPlayer = NULL;
    m_bPlaying = false;
}

Video::~Video()
{

}

bool Video::loadVlc(const QString &filepath, void *video)
{
    const char* vlc_args[] = {"--ignore-config",
                              "--no-video-title-show",
                              "--network-caching=5000"};
    m_vlcInstance = libvlc_new(sizeof(vlc_args) / sizeof(vlc_args[0]), vlc_args);
    m_media = libvlc_media_new_path(m_vlcInstance, filepath.toUtf8().data());
    if(!m_media){
        freeVlc();
        return m_media;
    }
    m_mediaPlayer = libvlc_media_player_new_from_media(m_media);
    if(!m_mediaPlayer){
        freeVlc();
        return false;
    }
    libvlc_media_parse(m_media);
    libvlc_event_manager_t *em = libvlc_media_player_event_manager(m_mediaPlayer);
    libvlc_event_attach(em, libvlc_MediaPlayerTimeChanged, vlcEvents, this);
    libvlc_event_attach(em, libvlc_MediaPlayerEndReached, vlcEvents, this);
    libvlc_event_attach(em, libvlc_MediaPlayerStopped, vlcEvents, this);
    libvlc_event_attach(em, libvlc_MediaPlayerPlaying, vlcEvents, this);
    libvlc_event_attach(em, libvlc_MediaPlayerPaused, vlcEvents, this);

//    libvlc_media_player_set_hwnd(m_mediaPlayer, (void *)ui->label->winId());
    libvlc_video_set_mouse_input(m_mediaPlayer, false);
    libvlc_video_set_key_input(m_mediaPlayer, false);
    libvlc_media_player_set_hwnd(m_mediaPlayer, video);
    return true;
}

void Video::play()
{
    if(m_media && libvlc_Ended == libvlc_media_get_state(m_media)){
        stop();//vlc这个设定太奇怪了，当自然播放结束，重新播放还得主动调用一次stop();
    }
    if (m_mediaPlayer) {
        libvlc_media_player_play(m_mediaPlayer);
        m_bPlaying = true;
    }
}

void Video::pause()
{
    if(m_mediaPlayer && (bool)libvlc_media_player_is_playing(m_mediaPlayer)){
        m_bPlaying = false;
        libvlc_media_player_set_pause(m_mediaPlayer, 1);//暂停
    }
}

void Video::stop()
{
    qDebug() << __FUNCTION__  << "stop";
    m_bPlaying = false;
    if (m_mediaPlayer)
        libvlc_media_player_stop(m_mediaPlayer);
}

void Video::freeVlc()
{
    if(m_media){
        libvlc_media_release(m_media);
        m_media = NULL;
    }
    if(m_mediaPlayer){
        libvlc_media_player_stop(m_mediaPlayer);
        libvlc_media_player_release(m_mediaPlayer);
        m_mediaPlayer = NULL;
    }
    if(m_vlcInstance){
        libvlc_release(m_vlcInstance);
        m_vlcInstance = NULL;
    }
}

void Video::vlcEvents(const libvlc_event_t *ev, void *param)
{
    Video* _video = (Video*)param;
    switch (ev->type) {
    case libvlc_MediaPlayerTimeChanged:
        break;
    case libvlc_MediaPlayerEndReached://不能在回调里调用stop();否则会卡死
        qDebug() << __FUNCTION__  << "libvlc_MediaPlayerEndReached";
        _video->emitSignal();
        break;
    case libvlc_MediaPlayerStopped:
        qDebug() << __FUNCTION__  << "libvlc_MediaPlayerStopped";
        break;
    case libvlc_MediaPlayerPlaying:
        qDebug() << __FUNCTION__  << "libvlc_MediaPlayerPlay";
        break;
    case libvlc_MediaPlayerPaused:
        qDebug() << __FUNCTION__  << "libvlc_MediaPlayerPaused";
        break;
    }
}

void Video::emitSignal()
{
    emit endReached();
}

bool Video::getVideoState()
{
    bool ret = false;

    if (libvlc_Ended == libvlc_media_get_state(m_media) || libvlc_Paused == libvlc_media_get_state(m_media) ||
            libvlc_Stopped == libvlc_media_get_state(m_media)) {
        ret = false;
    } else {
        ret = true;
    }

    return ret;
}

void Video::mouseDoubleClickEvent(QMouseEvent *event)
{
    emit doubleClick();
}

QString Video::getTime()
{
    int time, time_h, time_m, time_s;
    QString time_str;

    if (!m_mediaPlayer)
        return "00:00";

    time = libvlc_media_player_get_time(m_mediaPlayer);

    time = qCeil(double(time)/1000);
    time_h = time/3600;
    time_m = (time%3600)/60;
    time_s = (time%60);
    time_str = QString("%1").arg(time_m, 2, 10, QChar('0')) + ":" + QString("%1").arg(time_s, 2, 10, QChar('0'));

//    time_str = QString::number(time_h,10) + ":" +
//            QString::number(time_m,10) + ":" + QString::number(time_s,10);

    return time_str;
}

QString Video::getLength()
{
    int time, time_h, time_m, time_s;
    QString time_str;

    if (!m_mediaPlayer)
        return "00:00";

    time = libvlc_media_player_get_length(m_mediaPlayer);

    time = qCeil(double(time)/1000);
    time_h = time/3600;
    time_m = (time%3600)/60;
    time_s = (time%60);

    time_str = QString("%1").arg(time_m, 2, 10, QChar('0')) + ":" + QString("%1").arg(time_s, 2, 10, QChar('0'));
//    time_str = QString::number(time_h,10) + ":" +
//            QString::number(time_m,10) + ":" + QString::number(time_s,10);

    return time_str;
}

int Video::getTimeInt()
{
    if (!m_mediaPlayer)
        return 0;

    return libvlc_media_player_get_time(m_mediaPlayer);
}

int Video::getLengthInt()
{
    if (!m_mediaPlayer)
        return 0;

    return libvlc_media_player_get_length(m_mediaPlayer);
}

void Video::setPosition(float pos)
{
    if (!m_mediaPlayer)
        return;

    libvlc_media_player_set_position(m_mediaPlayer, pos);
}

int Video::setVolume(int vol)
{
    if (!m_mediaPlayer)
        return -1;

    return libvlc_audio_set_volume(m_mediaPlayer, vol);
}

int Video::getVolume()
{
    if (!m_mediaPlayer)
        return 0;

    return libvlc_audio_get_volume(m_mediaPlayer);
}

/*Parameters
 p_mi
   media player
 status
  If status is true then mute, otherwise unmute
*/
void Video::setMute(int status)
{
     libvlc_audio_set_mute(m_mediaPlayer, status);
}

/*
 Returns
   the mute status (boolean) if defined, -1 if undefined/unapplicable
*/
int Video::getMute()
{
    return libvlc_audio_get_mute(m_mediaPlayer);
}
