#include "videopage.h"
#include "imagebutton.h"
#include <QLabel>
#include <QPixmap>
#include <QDebug>
#include <QApplication>
#include <QDesktopWidget>
#include <QFont>

VideoPage* VideoPage::videoPageInstance = NULL;

VideoPage::VideoPage(QWidget *parent) : QWidget(parent)
{
    middle = Middle::getMiddleInstance();
    video = new Video();
//    connect(video, SIGNAL(endReached()), this, SLOT(onVideoEndReached()));
}

VideoPage* VideoPage::getVideoPageInstance()
{
    if (videoPageInstance == NULL)
        videoPageInstance = new VideoPage();

    return videoPageInstance;
}

void VideoPage::setWidget(QWidget *videoP, QStackedWidget *stack, QWidget *root, MyLabel *video,
                          QSlider *slider, QLabel *timeLabel, QLabel *lenLabel, QSlider *_volSlider,
                          ImageButton *play, ImageButton *vol, ImageButton *full, QLabel *ctlRect)
{
    videoPage = videoP;
    stackWidget = stack;
    videoLabel = video;
    rootWidget = root;
    isFullScreen = false;

    videoBgLabel = new QLabel(videoPage);
    BgLabel = new QLabel(videoPage);
    topLabel = new QLabel(videoPage);
    leftLabel = new QLabel(videoPage);
    rightLabel = new QLabel(videoPage);
    btnBack = new ImageButton(videoPage);
    posSlider = slider;
    timer = new QTimer();
    curTimeLabel = timeLabel;
    lengthLabel = lenLabel;
    videoCtlTimer = new QTimer();
    volSlider = _volSlider;
    btnPlay = play;
    btnVol = vol;
    btnFull = full;
    videoCtlRect = ctlRect;

    posSlider->installEventFilter(this);
    volSlider->installEventFilter(this);

    connect(btnBack, SIGNAL(clicked()), this, SLOT(onBackClick()));
    connect(videoLabel, SIGNAL(LabelDoubleClicked()), this, SLOT(onVideoDoubleClick()));
//    connect(videoLabel, SIGNAL(LabelClicked()), this, SLOT(onVideoClick()));
    connect(videoLabel, SIGNAL(LabelMouseMove()), this, SLOT(onMouseMove()));
    connect(posSlider, SIGNAL(sliderMoved(int)), this, SLOT(changePosition(int)));
//    connect(posSlider, SIGNAL(sliderPressed()), this, SLOT(onPressed()));
    connect(timer, SIGNAL(timeout()), this, SLOT(updateInfo()));
    connect(videoCtlTimer, SIGNAL(timeout()), this, SLOT(hideVideoCtl()));

    connect(btnPlay, SIGNAL(clicked()), this, SLOT(onVideoClick()));
    connect(btnFull, SIGNAL(clicked()), this, SLOT(onVideoDoubleClick()));
    connect(btnVol, SIGNAL(clicked()), this, SLOT(onVolClick()));
    connect(volSlider, SIGNAL(sliderMoved(int)), this, SLOT(onVolSliderMoved(int)));
}

void VideoPage::setVideoPage(QString bgImage, QString topImage, QString leftImage,
                             QString rightImage, QString videoUrl, int videoSize,
                             ImageButton *btnP, QWidget *parent, QString type_)
{
    btnParent = btnP;
    parentPage = parent;
    QFont font;
    QFont fontVideo;
    QPalette paVideo;
    QString videoUrl_;
    type = type_;

    font.setFamily("黑体");
    font.setPixelSize(15);

    QPalette pa;
    pa.setColor(QPalette::WindowText,QColor(211,211,211));

    fontVideo.setFamily("楷体");
    fontVideo.setPixelSize(30);
    fontVideo.setLetterSpacing(QFont::AbsoluteSpacing,1);
    fontVideo.setBold(true);
    paVideo.setColor(QPalette::WindowText,QColor(2,173,237));

    // 设置背景图片
    BgLabel->setGeometry(0,0,1000,640);
    BgLabel->setStyleSheet(bgImage);

    // 设置顶部图片
    topLabel->setGeometry(365,42,237,29);
    topLabel->setStyleSheet(topImage);

    // 设置左部图片
    leftLabel->setGeometry(109,291,37,150);
    leftLabel->setStyleSheet(leftImage);

    // 设置右部图片
    rightLabel->setGeometry(900,291,30,111);
    rightLabel->setStyleSheet(rightImage);

    videoStop = false;
    initVideoCtl();
    showVideoCtl();

    videoLabel->setGeometry(stackWidget->x()+174,stackWidget->y()+116,640,465);
    videoLabel->setFont(fontVideo);
    videoLabel->setPalette(paVideo);
    videoLabel->setAlignment(Qt::AlignCenter);
    videoLabel->show();
    videoLabel->clear();
    if (videoUrl == "0") {
        videoLabel->setText("视频正在努力制作中，敬请期待！");
    }
//    video->loadVlc("c:\\chong2-encyp.mp4", (void*)videoLabel->winId());
//    video->loadVlc("http://www.cguoguo.org.cn/jfx-web/attachment/ggxzvideo/右偏旁阝.mp4", (void*)videoLabel->winId());
    // del
    if (videoUrl.contains("http", Qt::CaseInsensitive)) {
        videoUrl_ = videoUrl;
    } else {
        videoUrl_ = middle->getVideoPath(type, videoUrl, videoSize);
    }
    qDebug() << __FUNCTION__  << "videoUrl" << videoUrl_;
    video->loadVlc(videoUrl_, (void*)videoLabel->winId());
    video->setVolume(middle->getVolume());
    video->play();

    setVideoCtl();
    initVolCtl();

    // 设置返回按钮
    setBackButton();
    middle->download(videoUrl, videoSize);
}

void VideoPage::playVideos()
{
    QString videoUrl_, videoUrl;
    int videoSize;

    if (videoMap_.count() == 0)
        return;

    QMap<QString,int>::iterator it;
    it = videoMap_.begin();
    videoUrl = it.key();
    videoSize = it.value();
    videoMap_.remove(videoUrl);

    if (videoUrl == "0") {
        videoLabel->setText("视频正在努力制作中，敬请期待！");
    }

    videoUrl_ = middle->getVideoPath(type, videoUrl, videoSize);
    qDebug() << __FUNCTION__  << "videoUrl" << videoUrl_;
    video->stop();
    video->loadVlc(videoUrl_, (void*)videoLabel->winId());
    video->play();
    middle->download(videoUrl, videoSize);
}

void VideoPage::setVideoPage(QString bgImage, QString topImage, QString leftImage,
                             QString rightImage, QMap<QString, int> videoMap, ImageButton *btnP,
                             QWidget *parent, QString type_)
{
    btnParent = btnP;
    parentPage = parent;
    videoMap_ = videoMap;
    type = type_;

    QFont font;
    QFont fontVideo;
    QPalette paVideo;

    font.setFamily("黑体");
    font.setPixelSize(15);

    QPalette pa;
    pa.setColor(QPalette::WindowText,QColor(211,211,211));

    fontVideo.setFamily("楷体");
    fontVideo.setPixelSize(30);
    fontVideo.setLetterSpacing(QFont::AbsoluteSpacing,1);
    fontVideo.setBold(true);
    paVideo.setColor(QPalette::WindowText,QColor(2,173,237));

    // 设置背景图片
    BgLabel->setGeometry(0,0,1000,640);
    BgLabel->setStyleSheet(bgImage);

    // 设置顶部图片
    topLabel->setGeometry(365,42,237,29);
    topLabel->setStyleSheet(topImage);

    // 设置左部图片
    leftLabel->setGeometry(109,291,37,150);
    leftLabel->setStyleSheet(leftImage);

    // 设置右部图片
    rightLabel->setGeometry(900,291,30,111);
    rightLabel->setStyleSheet(rightImage);

    videoStop = false;
    initVideoCtl();
    showVideoCtl();

    videoLabel->setGeometry(stackWidget->x()+174,stackWidget->y()+116,640,465);
    videoLabel->setFont(fontVideo);
    videoLabel->setPalette(paVideo);
    videoLabel->setAlignment(Qt::AlignCenter);
    videoLabel->show();
    videoLabel->clear();

    if (videoMap_.count() == 0)
        return;

    video->setVolume(middle->getVolume());
    playVideos();

    setVideoCtl();
    initVolCtl();

    // 设置返回按钮
    setBackButton();
}

void VideoPage::setBackButton()
{
    btnBack->setGeometry(855,540,109,101);
    QPixmap backP1(":/nianji/yiji/fh_03.png");
    QPixmap backP2(":/nianji/yiji/fh_04.png");
    btnBack->setNormalImage(backP1);
    btnBack->setPressImage(backP2);
    btnBack->setCursor(QCursor(Qt::PointingHandCursor));
}

void VideoPage::onBackClick()
{
    timer->stop();
    if (btnParent)
        btnParent->blockSignals(false);

    video->stop();
    videoLabel->hide();
    hideVideoCtl();
    if (videoCtlTimer->isActive()) {
        videoCtlTimer->stop();
    }
    stackWidget->setCurrentWidget(parentPage);
    lengthLabel->clear();
    curTimeLabel->clear();
}

void VideoPage::onVideoDoubleClick()
{
    qDebug() << __FUNCTION__  << "video double click";
    if (isFullScreen) {
        isFullScreen = false;
        emit showNormal();

        videoLabel->setGeometry(stackWidget->x()+174,stackWidget->y()+116,640,465);
        videoLabel->show();

        showVideoCtl();
        posSlider->setGeometry(stackWidget->x()+179,stackWidget->y()+566,630,8);
        curTimeLabel->setGeometry(stackWidget->x()+221,stackWidget->y()+586,80,20);
        lengthLabel->setGeometry(stackWidget->x()+306,stackWidget->y()+586,100,20);
        btnPlay->setGeometry(stackWidget->x()+189,stackWidget->y()+586,18,20);
        btnVol->setGeometry(stackWidget->x()+721,stackWidget->y()+561,20,18);
        btnFull->setGeometry(stackWidget->x()+771,stackWidget->y()+586,22,16);
        volSlider->setGeometry(stackWidget->x()+746,stackWidget->y()+566,60,8);

        if (videoCtlTimer->isActive()) {
            videoCtlTimer->stop();
        }
    } else {
        isFullScreen = true;
        emit fullScreen();

        videoLabel->resize(rootWidget->width(),QApplication::desktop()->height());
        videoLabel->move(0,0);
        videoLabel->show();

        posSlider->setGeometry(10,QApplication::desktop()->height()-60,rootWidget->width()-20,8);
        curTimeLabel->setGeometry(60,QApplication::desktop()->height()-40,80,20);
        lengthLabel->setGeometry(145,QApplication::desktop()->height()-40,100,20);
        btnPlay->setGeometry(30,QApplication::desktop()->height()-40,18,20);
        btnVol->setGeometry(rootWidget->width()-120,QApplication::desktop()->height()-65,20,18);
        btnFull->setGeometry(rootWidget->width()-70,QApplication::desktop()->height()-40,22,16);
        volSlider->setGeometry(rootWidget->width()-90,QApplication::desktop()->height()-60,60,8);
        videoCtlRect->setGeometry(0,QApplication::desktop()->height()-65,rootWidget->width(),65);
        videoCtlRect->show();

        videoCtlRect->raise();
        posSlider->raise();
        curTimeLabel->raise();
        lengthLabel->raise();
        btnPlay->raise();
        btnVol->raise();
        btnFull->raise();
        volSlider->raise();

        if (videoCtlTimer->isActive()) {
            videoCtlTimer->stop();
        }
        videoCtlTimer->start(5000);
    }
}

void VideoPage::onVideoClick()
{
    if (videoStop) {
        video->stop();
        video->play();

        setPauseIcon();
        videoStop = false;
        return;
    }
    if (video->getVideoState()) {
        setPlayIcon();
        video->pause();
    } else {
        setPauseIcon();
        video->play();
    }
}

void VideoPage::changePosition(int pos)
{
    qDebug() << __FUNCTION__  << pos;
    float posf = (float)(pos)/100.0;
    video->setPosition(posf);
}

void VideoPage::onPressed()
{
    int pos = posSlider->value();
    qDebug() << __FUNCTION__  << pos;
    float posf = (float)(pos)/100.0;
    video->setPosition(posf);
}

void VideoPage::updateInfo()
{
//    qDebug() << __FUNCTION__  << "update";
//    qDebug() << __FUNCTION__  << video->getTime();
//    qDebug() << __FUNCTION__  << video->getLength();
    curTimeLabel->setText(video->getTime());
    lengthLabel->setText("/ " + video->getLength());

    if (video->getLengthInt() > 0 && (video->getLengthInt()-video->getTimeInt()) < 1000) {
        videoStop = true;
        if (videoMap_.count() == 0) {
            video->pause();
            setPlayIcon();
        } else {
            playVideos();
        }
    }

    if (video->getLengthInt() != 0) {
        int pos = video->getTimeInt()*100/video->getLengthInt();
//        qDebug() << __FUNCTION__  << pos;
        posSlider->setValue(pos);
    }
}

bool VideoPage::eventFilter(QObject *obj, QEvent *event)
{
    if(obj==posSlider)
    {
        if (event->type()==QEvent::MouseButtonPress)           //判断类型
        {
            QMouseEvent *mouseEvent = static_cast<QMouseEvent *>(event);
            if (mouseEvent->button() == Qt::LeftButton)	//判断左键
            {
               int dur = posSlider->maximum() - posSlider->minimum();
               int pos = posSlider->minimum() + dur * ((double)mouseEvent->x() / posSlider->width());
               if(pos != posSlider->sliderPosition())
                {
                   changePosition(pos);
                }
            }

        }
    } else if (obj==volSlider) {
        if (event->type()==QEvent::MouseButtonPress)           //判断类型
        {
            QMouseEvent *mouseEvent = static_cast<QMouseEvent *>(event);
            if (mouseEvent->button() == Qt::LeftButton)	//判断左键
            {
               int dur = volSlider->maximum() - volSlider->minimum();
               int pos = volSlider->minimum() + dur * ((double)mouseEvent->x() / volSlider->width());
               if(pos != volSlider->sliderPosition())
                {
                  volSlider->setValue(pos);
                  qDebug() << __FUNCTION__  << "vol value:" << pos;
                  video->setVolume(pos);
                  middle->setVolume(pos);
                }
            }

        }
    }
    return QObject::eventFilter(obj,event);
}

void VideoPage::resizeVideo()
{
    if (!videoLabel->isHidden()) {
        videoLabel->setGeometry(stackWidget->x()+174,stackWidget->y()+116,640,465);
        posSlider->setGeometry(stackWidget->x()+179,stackWidget->y()+566,630,8);
        curTimeLabel->setGeometry(stackWidget->x()+221,stackWidget->y()+586,80,20);
        lengthLabel->setGeometry(stackWidget->x()+306,stackWidget->y()+586,100,20);
        btnPlay->setGeometry(stackWidget->x()+189,stackWidget->y()+586,18,20);
        btnVol->setGeometry(stackWidget->x()+721,stackWidget->y()+561,20,18);
        btnFull->setGeometry(stackWidget->x()+771,stackWidget->y()+586,22,16);
        volSlider->setGeometry(stackWidget->x()+746,stackWidget->y()+566,60,8);
    }
}

void VideoPage::hideVideoCtl()
{
    posSlider->hide();
    curTimeLabel->hide();
    lengthLabel->hide();
    volSlider->hide();
    btnPlay->hide();
    btnVol->hide();
    btnFull->hide();
    videoBgLabel->hide();
    videoCtlRect->hide();
}

void VideoPage::showVideoCtl()
{
    posSlider->show();
    curTimeLabel->show();
    lengthLabel->show();
//    volSlider->show();
    btnPlay->show();
//    btnVol->show();
    btnFull->show();
    videoBgLabel->show();
}

//void VideoPage::updateVideoCtl()
//{
//    posSlider->hide();
//    curTimeLabel->hide();
//    lengthLabel->hide();
//}

void VideoPage::onMouseMove()
{
    if (isFullScreen) {
        showVideoCtl();
        videoCtlRect->show();

        if (videoCtlTimer->isActive()) {
            videoCtlTimer->stop();
        }
        videoCtlTimer->start(5000);
    }
}

void VideoPage::onVideoEndReached()
{
    video->stop();
    setPlayIcon();
}

void VideoPage::initVideoCtl()
{
    QFont font;
    font.setFamily("宋体");
    font.setPixelSize(20);

    QPalette pa;
    pa.setColor(QPalette::WindowText,QColor(211,211,211));

    // 设置控制栏背景
    //videoLabel->setGeometry(stackWidget->x()+174,stackWidget->y()+116,640,500);
    videoBgLabel->setGeometry(174,556,640,60);
    videoBgLabel->raise();
    videoBgLabel->hide();
    videoBgLabel->setStyleSheet("background-color: black");

    // 设置进度条
    posSlider->raise();
    posSlider->setGeometry(stackWidget->x()+179,stackWidget->y()+566,630,8);

    // 设置视频时间显示
    curTimeLabel->setGeometry(stackWidget->x()+221,stackWidget->y()+586,80,20);
    curTimeLabel->setFont(font);
    curTimeLabel->setPalette(pa);
    curTimeLabel->raise();
    curTimeLabel->setAlignment(Qt::AlignRight);
    lengthLabel->setGeometry(stackWidget->x()+306,stackWidget->y()+586,100,20);
    lengthLabel->setFont(font);
    lengthLabel->setPalette(pa);
    lengthLabel->raise();
    lengthLabel->setAlignment(Qt::AlignLeft);
    timer->start(1000);

    // 设置控制按钮图标
    QPixmap playP1(":/player/play01.png");
    QPixmap playP2(":/player/play02.png");
    btnPlay->setNormalImage(playP1);
    btnPlay->setPressImage(playP2);
    QPixmap volP1(":/player/vol01.png");
    QPixmap volP2(":/player/vol02.png");
    btnVol->setNormalImage(volP1);
    btnVol->setPressImage(volP2);
    QPixmap fullP1(":/player/full01.png");
    QPixmap fullP2(":/player/full02.png");
    btnFull->setNormalImage(fullP1);
    btnFull->setPressImage(fullP2);

    // 设置控制按钮坐标
    btnPlay->raise();
    btnPlay->setGeometry(stackWidget->x()+189,stackWidget->y()+586,18,20);
    btnVol->raise();
    btnVol->setGeometry(stackWidget->x()+721,stackWidget->y()+561,20,18);
    btnFull->raise();
    btnFull->setGeometry(stackWidget->x()+771,stackWidget->y()+586,22,16);

    // 设置声音控制
    volSlider->raise();
    volSlider->setGeometry(stackWidget->x()+746,stackWidget->y()+566,60,8);
}

void VideoPage::setVideoCtl()
{
    // 设置播放按钮
    if (video->getVideoState()) {
        setPauseIcon();
    } else {
        setPlayIcon();
    }

    // 设置声音
    int vol = video->getVolume();
    qDebug() << __FUNCTION__  << "get volume:" << vol;
    volSlider->setValue(vol);
}

void VideoPage::onVolClick()
{
    qDebug() << __FUNCTION__  << video->getMute();

    if (video->getMute()) { //当前状态为静音
        video->setMute(0);
        Common::muteStatus = false;

        QPixmap volP1(":/player/vol01.png");
        QPixmap volP2(":/player/vol02.png");
        btnVol->setNormalImage(volP1);
        btnVol->setPressImage(volP2);
        btnVol->repaint();
    } else {
        video->setMute(1);
        Common::muteStatus = true;

        QPixmap muteP1(":/player/mute01.png");
        QPixmap muteP2(":/player/mute02.png");
        btnVol->setNormalImage(muteP1);
        btnVol->setPressImage(muteP2);
        btnVol->repaint();
    }
}

void VideoPage::initVolCtl()
{
    qDebug() << __FUNCTION__  << Common::muteStatus;

    if (Common::muteStatus) { //当前状态为静音
        video->setMute(1);

        QPixmap muteP1(":/player/mute01.png");
        QPixmap muteP2(":/player/mute02.png");
        btnVol->setNormalImage(muteP1);
        btnVol->setPressImage(muteP2);
        btnVol->repaint();
    } else {
        video->setMute(0);

        QPixmap volP1(":/player/vol01.png");
        QPixmap volP2(":/player/vol02.png");
        btnVol->setNormalImage(volP1);
        btnVol->setPressImage(volP2);
        btnVol->repaint();
    }
}

void VideoPage::onVolSliderMoved(int pos)
{
    video->setVolume(pos);
    middle->setVolume(pos);
}

void VideoPage::setPlayIcon()
{
    QPixmap playP1(":/player/play01.png");
    QPixmap playP2(":/player/play02.png");
    btnPlay->setNormalImage(playP1);
    btnPlay->setPressImage(playP2);
    btnPlay->repaint();
}

void VideoPage::setPauseIcon()
{
    QPixmap playP1(":/player/pause01.png");
    QPixmap playP2(":/player/pause02.png");
    btnPlay->setNormalImage(playP1);
    btnPlay->setPressImage(playP2);
    btnPlay->repaint();
}
