#include "welcomepage.h"
#include "imagebutton.h"
#include <QLabel>
#include <QPainter>
#include <QPen>
#include <QFrame>
#include <QDebug>
#include <QCoreApplication>

WelcomePage* WelcomePage::welcomeInstance = NULL;

WelcomePage::WelcomePage(QObject *parent) : QObject(parent)
{
    middle = Middle::getMiddleInstance();
    course = CoursePage::getCourseInstance();
    tool = Tool::getToolInstance();
}

WelcomePage* WelcomePage::getWelcomeInstance()
{
    if (welcomeInstance == NULL)
        welcomeInstance = new WelcomePage();

    return welcomeInstance;
}

void WelcomePage::setWidget(QWidget *welcome, QStackedWidget *stack, QWidget *course, QMenuBar *bar)
{
    welcomePage = welcome;
    coursePage = course;
    stackWidget = stack;
    menuBar = bar;

    enterButton = new ImageButton(welcomePage);
}

void WelcomePage::setWelcomePage()
{
    QString welcomeImage;
    QPalette pa;
    pa.setColor(QPalette::WindowText,QColor(35,29,29));

    // 设置背景图片
    welcomeImage = middle->getOption("klxz_welcome");
//    QString bgurl = middle->getBaseUrl() + welcomeImage;
//    welcomePage->setStyleSheet("QWidget#welcomePage{image:url(:/bj_21.jpg)}");
    QLabel *backgroundLabel = new QLabel(welcomePage);
    backgroundLabel->lower();
    backgroundLabel->setGeometry(0,0,1000,640);
    QPixmap bgPix;\
    QString imageFile = middle->getImageFile(welcomeImage, "/welcome/");
    if (middle->getClientMode() && imageFile.length() > 0) {
        bgPix.load(imageFile);
    } else {
        bgPix.loadFromData(middle->getTrainImageData(welcomeImage));
    }
    bgPix = bgPix.scaled(QSize(1000,640),Qt::IgnoreAspectRatio);
    backgroundLabel->setPixmap(bgPix);

    // 版本号
    QLabel *version = new QLabel(welcomePage);
    QString clientVer = middle->getCurrentVersion();
    version->setText("当前版本: " + clientVer);
    version->setFont(QFont( "楷体_GB2312" , 13 , QFont::Normal));
    version->setGeometry(15,20,260,30);

    // 欢迎文字
    welText = new QLabel(welcomePage);
    wText = middle->getOption("klxz_welcometext");
    welText->setFont(QFont( "楷体_GB2312" , 13 , QFont::Normal));
    welText->setGeometry(480,20,350,30);

    QTimer *pTimer = new QTimer(this);
    connect(pTimer, SIGNAL(timeout()), this, SLOT(scrollCaption()));

    // 定时200毫秒
    pTimer->start(400);

//    // 客服电话
//    QLabel *telNum = new QLabel(welcomePage);
//    telNum->setText("客服电话: ");
//    telNum->setFont(QFont( "楷体_GB2312" , 20 , QFont::Normal));
//    telNum->setGeometry(570,20,1000,30);

//    QLabel *telNumText = new QLabel(welcomePage);
//    telNumText->setText(middle->getOption("klxz_service_telephone"));
//    telNumText->setFont(QFont( "楷体_GB2312" , 13 , QFont::Normal));
//    telNumText->setPalette(pa);
//    telNumText->setGeometry(483,20,300,30);

    // 版本设置框
    /*
    comBox = new QComboBox(welcomePage);
    comBox->setGeometry(850,20,140,30);
    comBox->setFont(QFont( "楷体_GB2312" , 13 , QFont::Normal));

    QList<QString> versions = middle->getVersions();

    QString userVer = middle->getUserVersion();
    qDebug() << __FUNCTION__  << userVer;
    if (userVer.length() > 0 && versions.contains(userVer)) {
        comBox->insertItem(0, userVer);
        versions.removeOne(userVer);
        for (int i=0; i<versions.count(); i++) {
            comBox->insertItem(i+1, versions.at(i));
        }
    } else if (versions.contains("人教版")) {
        comBox->insertItem(0, "人教版");
        versions.removeOne("人教版");
        for (int i=0; i<versions.count(); i++) {
            comBox->insertItem(i+1, versions.at(i));
        }
    } else {
        for (int i=0; i<versions.count(); i++) {
            comBox->insertItem(i, versions.at(i));
        }
    }
    */

    // 横线
    QFrame *line = new QFrame(welcomePage);
    line->setGeometry(0,50,1000,2);
    line->setStyleSheet("border-top: 1px solid rgb(176,176,176);");
    line->setFrameShape(QFrame::HLine);
    line->setFrameShadow(QFrame::Sunken);

    // 每天的欢迎图片
    QLabel *welImage = new QLabel(welcomePage);
    QString url;
    int week = middle->getWeekInfo();
    url.sprintf("image:url(:/week/%d.png)", week);
    welImage->setStyleSheet(url);
    welImage->setGeometry(676,450,256,37);

    // 点击进入 按钮
    enterButton->setGeometry(770,512,100,35);
    enterButton->setMinimumSize(100,35);
    QPixmap enterP1(":/shouye/djjr_01.png");
    QPixmap enterP2(":/shouye/djjr_02.png");
    enterButton->setNormalImage(enterP1);
    enterButton->setPressImage(enterP2);
    enterButton->setCursor(QCursor(Qt::PointingHandCursor));

    QObject::connect(enterButton, SIGNAL(clicked()), this, SLOT(onEnterClick()));
}

void WelcomePage::scrollCaption()
{
    static int nPos = 0;

    // 当截取的位置比字符串长时，从头开始
    if (nPos > wText.length())
        nPos = 0;

    welText->setText(wText.mid(nPos));
    nPos++;
}

void WelcomePage::onEnterClick()
{
    enterButton->blockSignals(true);
//    middle->setVersion(comBox->currentText());
    course->setCoursePage();
    stackWidget->setCurrentWidget(coursePage);
//    setMenu(comBox->currentText());
//    middle->setUserVersion(comBox->currentText());
    menuBar->show();
}

// 设置菜单栏
void WelcomePage::setMenu(QString version)
{
//    QList< QList<QString> > versionList = middle->getVersionList();
    QList<QString> nianji;
    QFont font;

    nianji.append("一年级");
    nianji.append("二年级");
    nianji.append("三年级");
    nianji.append("四年级");
    nianji.append("五年级");
    nianji.append("六年级");

    font.setFamily("黑体");
    font.setPixelSize(16);
    font.setLetterSpacing(QFont::AbsoluteSpacing,1);

    menuBar->setFont(font);
    QMenu *videoSource = menuBar->addMenu("视频下载");
    QMenu *download = videoSource->addMenu("下载视频到本机");
    QAction *cacheAction = videoSource->addAction("缓存播放视频");
    cacheAction->setCheckable(true);

//    for (int i=0; i<versionList.count(); i++) {
        QMenu *tmpMenu = download->addMenu(version);
//        QAction *tmpAction = tmpMenu->addAction("全部下载");
        QString ver = tmpMenu->title();

        QVariantMap data;
        data["type"] = "course";
        data["version"] = ver;
        data["grade"] = "";
        data["level"] = "";

//        tmpAction->setData(data);
//        tmpAction->setCheckable(true);
//        if (middle->getDownloadStatus("course", ver, "", "")) {
//            tmpAction->setChecked(true);
//            tmpAction->setDisabled(true);
//        }

        for (int i=0; i<nianji.count(); i++) {
            QMenu *nianjiMenu = tmpMenu->addMenu(nianji[i]);

            QVariantMap data;
            QString grade = QString::number((i+1), 10);
            data["type"] = "course";
            data["version"] = ver;
            data["grade"] = grade;
            data["level"] = "1";

            qDebug() << __FUNCTION__  << data;

            QAction *shangAction = nianjiMenu->addAction("上册");
            shangAction->setData(data);
            shangAction->setCheckable(true);
            if (middle->getDownloadStatus("course", ver, grade, "1")) {
                shangAction->setChecked(true);
//                shangAction->setDisabled(true);
            }

            QAction *xiaAction = nianjiMenu->addAction("下册");
            data["level"] = "2";
            xiaAction->setData(data);
            xiaAction->setCheckable(true);
            if (middle->getDownloadStatus("course", ver, grade, "2")) {
                xiaAction->setChecked(true);
//                xiaAction->setDisabled(true);
            }
        }
//    }

    QAction *xzzsAction = download->addAction("写字姿势");
    QAction *jbbhAction = download->addAction("基本笔画");
//    QAction *sfktAction = download->addAction("书法课堂");
    QAction *ppbsAction = download->addAction("偏旁部首");
//    QAction *sxzfAction = download->addAction("书写章法");

    xzzsAction->setCheckable(true);
    jbbhAction->setCheckable(true);
//    sfktAction->setCheckable(true);
    ppbsAction->setCheckable(true);
//    sxzfAction->setCheckable(true);

    if (middle->getDownloadStatus("xzzs", "", "", "")) {
        xzzsAction->setChecked(true);
//        xzzsAction->setDisabled(true);
    }
    if (middle->getDownloadStatus("jbbh", "", "", "")) {
        jbbhAction->setChecked(true);
//        jbbhAction->setDisabled(true);
    }
//    if (middle->getDownloadStatus("sfkt", "", "", ""))
//        sfktAction->setChecked(true);
    if (middle->getDownloadStatus("ppbs", "", "", "")) {
        ppbsAction->setChecked(true);
//        ppbsAction->setDisabled(true);
    }
    if (tool->readIni("Cache") == "true")
        cacheAction->setChecked(true);

    connect(download, SIGNAL(triggered(QAction*)), this, SLOT(download(QAction*)));
    connect(videoSource, SIGNAL(triggered(QAction*)), this, SLOT(setCacheStatus(QAction*)));
}

void WelcomePage::download(QAction *action)
{
    QString type, version, grade, level;
    QVariant data;

    data = action->data();

    if (data.isNull()) {
        type = middle->versionMap(action->text());
        qDebug() << __FUNCTION__  << type;

        if (action->isChecked()) {
            middle->setDownloadInfo(type, "", "", "");
//            action->setDisabled(true);
        }
        else
            middle->unSetDownloadInfo(type, "", "", "");
    } else {
        type = data.toMap()["type"].toString();
        version = data.toMap()["version"].toString();
        grade = data.toMap()["grade"].toString();
        level = data.toMap()["level"].toString();
        qDebug() << __FUNCTION__  << type << middle->versionMap(version) << grade << level;

        if (action->isChecked()) {
            middle->setDownloadInfo(type, version, grade, level);
//            action->setDisabled(true);
        }
        else
            middle->unSetDownloadInfo(type, version, grade, level);
    }

}

void WelcomePage::setCacheStatus(QAction *action)
{
    if (action->isChecked()) {
        tool->writeIni("Cache", "true");
    } else {
        tool->writeIni("Cache", "false");
    }
}
