#include "wordpage.h"
#include <QStackedWidget>
#include <QPixmap>
#include <QIcon>
#include <QPalette>
#include <QFont>
#include <QStringListModel>
#include <QStringList>
#include <QListView>
#include <QDebug>
#include <QApplication>
#include <QDesktopWidget>

WordPage* WordPage::wordPageInstance = NULL;
//extern bool muteStatus;

WordPage::WordPage(QObject *parent) : QObject(parent)
{
    video = new Video();
    middle = Middle::getMiddleInstance();
    sfxsPage = SfxsPage::getSfxsInstance();
    swjz = SwjzPage::getSwjzInstance();
    flashInstance = FlashPage::getFlashPageInstance();
    isFullScreen = false;
    flashIsFull = false;

//    connect(video, SIGNAL(endReached()), this, SLOT(onVideoEndReached()));
}

WordPage* WordPage::getWordPageInstance()
{
    if (wordPageInstance == NULL) {
        wordPageInstance = new WordPage();
    }

    return wordPageInstance;
}

WordPage::~WordPage()
{
}

void WordPage::setWidget(QWidget* word, QStackedWidget *stack, QWidget *next, QWidget *root,
                         MyLabel *video, QWidget *swjz, QSlider *Slider, QLabel *timeLabel,
                         QLabel *lenLabel, QSlider *_volSlider, ImageButton *play,
                         ImageButton *vol, ImageButton *full, QLabel *ctlRect, QWidget *flashP)
{
    wordPage = word;
    nextPage = next;
    stackWidget = stack;
    rootWidget = root;
    videoLabel = video;
    swjzPage = swjz;
    flashPage = flashP;

//    bgLabel = new QLabel(wordPage);
    videoBgLabel = new QLabel(wordPage);
    sjqpLabel = new QLabel(wordPage);
    bsLabel = new QLabel(wordPage);
    topLabel = new QLabel(wordPage);
    pinyinLabel = new QLabel(wordPage);
    bushouLabel = new QLabel(wordPage);
    jiegouLabel = new QLabel(wordPage);
    jiegouText = new QLabel(wordPage);
    jiegouRect = new QLabel(wordPage);

    bssfButton = new ImageButton(wordPage);
    mbsfButton = new ImageButton(wordPage);
    gbsfButton = new ImageButton(wordPage);
    sfxsButton = new ImageButton(wordPage);
    btnBack = new ImageButton(wordPage);

//    zuciView = new QListView(wordPage);

    scrollArea = new QScrollArea(wordPage);
//    zuciWidget = new QWidget(wordPage);

    posSlider = Slider;
    timer = new QTimer();
    videoCtlTimer = new QTimer();
    curTimeLabel = timeLabel;
    lengthLabel = lenLabel;
    volSlider = _volSlider;
    btnPlay = play;
    btnVol = vol;
    btnFull = full;
    videoCtlRect = ctlRect;

    posSlider->installEventFilter(this);
    volSlider->installEventFilter(this);

    connect(btnBack, SIGNAL(clicked()), this, SLOT(onBackClick()));
    connect(bssfButton, SIGNAL(clicked()), this, SLOT(bssfClick()));
    connect(gbsfButton, SIGNAL(clicked()), this, SLOT(gbsfClick()));
    connect(mbsfButton, SIGNAL(clicked()), this, SLOT(mbsfClick()));
    connect(sfxsButton, SIGNAL(clicked()), this, SLOT(sfxsClick()));

    connect(videoLabel, SIGNAL(LabelDoubleClicked()), this, SLOT(onVideoDoubleClick()));
//    connect(videoLabel, SIGNAL(LabelClicked()), this, SLOT(onVideoClick()));
    connect(videoLabel, SIGNAL(LabelMouseMove()), this, SLOT(onMouseMove()));

    connect(posSlider, SIGNAL(sliderMoved(int)), this, SLOT(changePosition(int)));
//    connect(posSlider, SIGNAL(sliderPressed()), this, SLOT(onPressed()));
    connect(timer, SIGNAL(timeout()), this, SLOT(updateInfo()));
    connect(videoCtlTimer, SIGNAL(timeout()), this, SLOT(hideVideoCtl()));

    connect(btnPlay, SIGNAL(clicked()), this, SLOT(onVideoClick()));
    connect(btnFull, SIGNAL(clicked()), this, SLOT(onVideoDoubleClick()));
    connect(btnVol, SIGNAL(clicked()), this, SLOT(onVolClick()));
    connect(volSlider, SIGNAL(sliderMoved(int)), this, SLOT(onVolSliderMoved(int)));
}

void WordPage::setWordPage(QWidget *parent, QString word_, QListWidget *listWidgetP, QString grade_, QString level_)
{
    if (wordPage == NULL)
        return;
    listWidgetParent = listWidgetP;
    parentPage = parent;
    word = word_;
    grade = grade_;
    level = level_;

    flash = new QAxWidget(wordPage);
    flash->installEventFilter(this);
    flash->setGeometry(333,140,360,360);
    bool ret = flash->setControl(QString::fromUtf8("{d27cdb6e-ae6d-11cf-96b8-444553540000}"));

    if (!ret)
        middle->errorMBox("未检测到Adobe Flash Player,请下载安装");

    QList<QVariant> wordInfo = middle->getWordInfo(word);
    if (wordInfo.count() == 0)
        return;
    QString pinyin = wordInfo[0].toMap()["pinyin"].toString();
    QString bushou = wordInfo[0].toMap()["bushou"].toString();
    QString zuci = wordInfo[0].toMap()["zuci"].toString();
    QString jiegou = wordInfo[0].toMap()["structure"].toString();
    QString zxjg = wordInfo[0].toMap()["zixingjiegou"].toString();
    QString flashUrl = wordInfo[0].toMap()["rd_url"].toString();
    int flashSize = wordInfo[0].toMap()["rd_url_size"].toInt();
    bsVideo = wordInfo[0].toMap()["bs_video_url"].toString();
    ybVideo = wordInfo[0].toMap()["gb_video_url"].toString();
    mbVideo = wordInfo[0].toMap()["mb_video_url"].toString();
    bsVideoSize = wordInfo[0].toMap()["bs_video_size"].toInt();
    ybVideoSize = wordInfo[0].toMap()["gb_video_size"].toInt();
    mbVideoSize = wordInfo[0].toMap()["mb_video_size"].toInt();
    // 说文解字
    swjzFlashUrl = wordInfo[0].toMap()["rd_url"].toString();
    swjzFlashSize = wordInfo[0].toMap()["rd_url_size"].toInt();

    if (grade.length() > 0 && grade.toInt() < 3) {
        ybVideo = wordInfo[0].toMap()["qb_video_url"].toString();
        ybVideoSize = wordInfo[0].toMap()["qb_video_size"].toInt();
    } else if (ybVideo == "0") {
        ybVideo = wordInfo[0].toMap()["qb_video_url"].toString();
        ybVideoSize = wordInfo[0].toMap()["qb_video_size"].toInt();
    }

    wordPage->setStyleSheet("QWidget#WordPage{image:url(:/nianji/erji/bj_04.jpg)}");
//    bgLabel->setGeometry(0,0,1000,700);
//    bgLabel->lower();
//    QPixmap pix;
//    pix.loadFromData(middle->getImageData2("http://www.cguoguo.org.cn/jfx-web/attachment/aggxz/bjpic/bj_04.jpg"));
//    bgLabel->setPixmap(pix);

    topLabel->setGeometry(393,11,224,85);
    topLabel->setStyleSheet("image:url(:/nianji/erji/nj_bt01.png)");
    bsLabel->setGeometry(373,11,290,87);
    bsLabel->hide();
    setVideo();
    setFlash(flashUrl, flashSize);
    setButton();
    setInfo(pinyin, bushou, zuci, zxjg, jiegou);
}

void WordPage::setFlash(QString flashUrl, int flashSize)
{
//    bool ret;
    // flash设置
//    flash = new QAxWidget(wordPage);
//    flash->lower();
//    flash->setGeometry(333,170,360,360);
//    ret = flash->setControl(QString::fromUtf8("{d27cdb6e-ae6d-11cf-96b8-444553540000}"));

//    if (!ret)
//        middle->errorMBox("未检测到Adobe Flash Player,请下载安装");

    flash->dynamicCall("LoadMovie(long,string)",0,middle->getVideoPath("ggxz", flashUrl, flashSize));
    flash->dynamicCall("Play()");
    middle->download(flashUrl, flashSize);
}

void WordPage::setVideo()
{
    QFont font;
    QPalette pa;

    pa.setColor(QPalette::WindowText,QColor(2,73,237));

    font.setFamily("楷体");
    font.setPixelSize(30);
    font.setLetterSpacing(QFont::AbsoluteSpacing,1);
    font.setBold(true);

    // 视频设置
    videoStop = false;
    videoLabel->hide();
    videoLabel->setGeometry(stackWidget->x()+333,stackWidget->y()+160,360,315);
    videoLabel->setStyleSheet("background-color: black");
    videoLabel->setFont(font);
    videoLabel->setPalette(pa);
    videoLabel->setAlignment(Qt::AlignCenter);

    hideVideoCtl();
    initVideoCtl();
}

void WordPage::setButton()
{
    // 板书示范
    bssfButton->setGeometry(768,197,166,48);
    bssfButton->setMinimumSize(120,35);
    QPixmap bssfP1(":/nianji/erji/nj_bssf01.png");
    QPixmap bssfP2(":/nianji/erji/nj_bssf02.png");
    bssfButton->setNormalImage(bssfP1);
    bssfButton->setPressImage(bssfP2);
    bssfButton->setImageArc(10);
    bssfButton->setCursor(QCursor(Qt::PointingHandCursor));

    // 钢笔示范
    gbsfButton->setGeometry(768,262,166,48);
    gbsfButton->setMinimumSize(120,35);
    QPixmap gbsfP1(":/nianji/erji/nj_gbsf01.png");
    QPixmap gbsfP2(":/nianji/erji/nj_gbsf02.png");
    gbsfButton->setNormalImage(gbsfP1);
    gbsfButton->setPressImage(gbsfP2);
    gbsfButton->setImageArc(10);
    gbsfButton->setCursor(QCursor(Qt::PointingHandCursor));

    // 毛笔示范
    mbsfButton->setGeometry(768,330,166,48);
    mbsfButton->setMinimumSize(120,35);
    QPixmap mbsfP1(":/nianji/erji/nj_mbsf01.png");
    QPixmap mbsfP2(":/nianji/erji/nj_mbsf02.png");
    mbsfButton->setNormalImage(mbsfP1);
    mbsfButton->setPressImage(mbsfP2);
    mbsfButton->setImageArc(10);
    mbsfButton->setCursor(QCursor(Qt::PointingHandCursor));

    // 书法欣赏
    sfxsButton->setGeometry(768,392,166,48);
    QPixmap sfxsP1(":/nianji/erji/nj_swjz01.png");
    QPixmap sfxsP2(":/nianji/erji/nj_swjz02.png");
    sfxsButton->setNormalImage(sfxsP1);
    sfxsButton->setPressImage(sfxsP2);
    sfxsButton->setCursor(QCursor(Qt::PointingHandCursor));

    // 返回按钮
    btnBack->setGeometry(769,528,109,101);
    QPixmap backP1(":/nianji/yiji/fh_03.png");
    QPixmap backP2(":/nianji/yiji/fh_04.png");
    btnBack->setNormalImage(backP1);
    btnBack->setPressImage(backP2);
    btnBack->setCursor(QCursor(Qt::PointingHandCursor));
}

void WordPage::setInfo(QString pinyi, QString bushou, QString zuci,QString zxjg, QString jiegou)
{
    QFont fontPinyin;
    QFont font;
    QFont fontJiegou;
    QFont fontBushou;
    QPalette pa, paSjqp;

    pa.setColor(QPalette::WindowText,QColor(38,151,209));
    paSjqp.setColor(QPalette::WindowText,QColor(251,151,4));

    fontPinyin.setFamily("HaiCi Dict");
    fontPinyin.setPixelSize(35);
    fontPinyin.setLetterSpacing(QFont::AbsoluteSpacing,1);
    fontPinyin.setBold(true);

    font.setFamily("楷体");
    font.setPixelSize(30);
    font.setLetterSpacing(QFont::AbsoluteSpacing,1);
//    font.setBold(true);

    fontJiegou.setFamily("楷体");
    fontJiegou.setPixelSize(28);
    fontJiegou.setLetterSpacing(QFont::AbsoluteSpacing,1);
//    fontJiegou.setBold(true);

    fontBushou.setFamily("楷体");
    fontBushou.setPixelSize(45);
    fontBushou.setLetterSpacing(QFont::AbsoluteSpacing,1);
    fontBushou.setBold(true);

    sjqpLabel->setGeometry(700, 250, 50, 200);
    sjqpLabel->setAlignment(Qt::AlignTop);
    QString sjqp = "双击全屏";
    sjqpLabel->setText(sjqp.split("", QString::SkipEmptyParts).join("\n"));
    sjqpLabel->setFont(font);
    sjqpLabel->setPalette(paSjqp);

    pinyinLabel->setText(pinyi);
    pinyinLabel->setGeometry(333,95,360,50);
    pinyinLabel->setFont(fontPinyin);
    pinyinLabel->setPalette(pa);
    pinyinLabel->setAlignment(Qt::AlignCenter);

    bushouLabel->setText(bushou);
    bushouLabel->setGeometry(170,230,50,50);
    bushouLabel->setFont(fontBushou);
    bushouLabel->setAlignment(Qt::AlignCenter);
//    bushouLabel->setStyleSheet("background-color:red");

    // 字形结构
    QPalette paYellow;
    paYellow.setColor(QPalette::WindowText,QColor(251,111,5));
    jiegouText->setText(zxjg);
    jiegouText->setGeometry(80,550,250,50);
    jiegouText->setAlignment(Qt::AlignRight);
    jiegouText->setFont(font);
    jiegouText->setPalette(pa);

    jiegouLabel->setText(jiegou);
    jiegouLabel->setGeometry(340,543,350,50);
    jiegouLabel->setFont(fontJiegou);
    jiegouLabel->setAlignment(Qt::AlignCenter);

    jiegouRect->setGeometry(340,540,350,53);
    jiegouRect->setStyleSheet("border-width: 1px; border-style: solid; border-radius:8px; border-color: rgb(255, 170, 0);");

    // 组词
    QFont fontZuci;
    fontZuci.setFamily("HaiCi Dict");
    fontZuci.setPixelSize(30);
    fontZuci.setLetterSpacing(QFont::AbsoluteSpacing,1);
    QStringList zuciList;
    zuciList = zuci.split(",");

    QWidget *zuciWidget = new QWidget();
    QVBoxLayout *vlayout = new QVBoxLayout();
    vlayout->setSizeConstraint(QVBoxLayout::SetMinAndMaxSize);
    vlayout->setSpacing(1);
    for (int i=0; i<zuciList.count(); i++) {
        QLabel *label = new QLabel();
        label->setAlignment(Qt::AlignCenter);
        label->setText(zuciList[i]);
        label->setFont(fontZuci);
        vlayout->addWidget(label);
    }

    zuciWidget->setGeometry(121,340,139,150);
    zuciWidget->setLayout(vlayout);

    scrollArea->setGeometry(120,340,146,150);
    scrollArea->setFrameShape(QFrame::NoFrame);
    scrollArea->setStyleSheet("background-color:white");
    scrollArea->setWidget(zuciWidget);
//    scrollArea->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    scrollArea->horizontalScrollBar()->setStyleSheet("QScrollBar:horizontal{height:6px;background:transparent;background-color:rgb(248, 248, 248);margin:0px,0px,0px,0px;padding-left:10px;padding-right:10px;}"
                       "QScrollBar::handle:horizontal{height:6px;background:lightgray ;border-radius:5px;min-height:20px;}"
                       "QScrollBar::handle:horizontal:hover{height:6px;background:gray;border-radius:5px;min-height:20px;}"
                       "QScrollBar::add-line:horizontal{height:10px;width:10px;border-image:url(:/button/images/button/down.png);subcontrol-position:bottom;}"
                       "QScrollBar::sub-line:horizontal{height:10px;width:10px;border-image:url(:/button/images/button/up.png);subcontrol-position:top;}"
                       "QScrollBar::add-line:horizontal:hover{height:10px;width:10px;border-image:url(:/button/images/button/down_mouseDown.png);subcontrol-position:bottom;}"
                       "QScrollBar::sub-line:horizontal:hover{height:10px;width:10px;border-image:url(:/button/images/button/up_mouseDown.png);subcontrol-position:top;}"
                       "QScrollBar::add-page:horizontal,QScrollBar::sub-page:vertical{background:transparent;border-radius:5px;}");
//                                                     );

    scrollArea->verticalScrollBar()->setStyleSheet("QScrollBar:vertical{width:6px;background:transparent;background-color:rgb(248, 248, 248);margin:0px,0px,0px,0px;padding-top:10px;padding-bottom:10px;}"
                       "QScrollBar::handle:vertical{width:6px;background:lightgray ;border-radius:5px;min-height:20px;}"
                       "QScrollBar::handle:vertical:hover{width:6px;background:gray;border-radius:5px;min-height:20px;}"
                       "QScrollBar::add-line:vertical{height:10px;width:10px;border-image:url(:/button/images/button/down.png);subcontrol-position:bottom;}"
                       "QScrollBar::sub-line:vertical{height:10px;width:10px;border-image:url(:/button/images/button/up.png);subcontrol-position:top;}"
                       "QScrollBar::add-line:vertical:hover{height:10px;width:10px;border-image:url(:/button/images/button/down_mouseDown.png);subcontrol-position:bottom;}"
                       "QScrollBar::sub-line:vertical:hover{height:10px;width:10px;border-image:url(:/button/images/button/up_mouseDown.png);subcontrol-position:top;}"
                       "QScrollBar::add-page:vertical,QScrollBar::sub-page:vertical{background:transparent;border-radius:5px;}");

//    ItemDelegate *newItemDelegate = new ItemDelegate();
//    QStringListModel *zuciModel = new QStringListModel(zuciList);
////    zuciView->setItemDelegate(newItemDelegate);
//    zuciView->setModel(zuciModel);
//    zuciView->setSpacing(3);
//    zuciView->setGeometry(145,372,115,150);
//    zuciView->setEditTriggers(QAbstractItemView::NoEditTriggers);
//    zuciView->setFocusPolicy(Qt::NoFocus);
//    // 去边框
//    zuciView->setFrameStyle(0);
//    // 设置样式
//    zuciView->setStyleSheet("QListView{font-size: 20px; font-family:宋体}"
//                            "QListView::item:selected:!active{background-color: transparent; color:black}"
//                            "QListView::item:selected:active{background-color: transparent; color:black}"
//                            "QListView{background-color:transparent}");
}

void WordPage::bssfClick()
{
    QString videoUrl = middle->getVideoPath("ggxz", bsVideo, bsVideoSize);

//    bool flashPlaying = flash->dynamicCall("Playing").toBool();
    if (flash) {
        flash->dynamicCall("Rewind()");
        flash->hide();
    }

    videoLabel->show();
    showVideoCtl();
    video->stop();
    qDebug() << __FUNCTION__  << "bssf vdiroUrl" << videoUrl;
    videoLabel->clear();
    if (bsVideo == "0") {
        videoLabel->setText("视频正在努力制作中，敬请期待！");
    }
//    video->loadVlc("c:\\chong2-encyp.mp4", (void*)videoLabel->winId());
    video->loadVlc(videoUrl, (void*)videoLabel->winId());
    video->setVolume(middle->getVolume());
    video->play();
    setPauseIcon();
    topLabel->hide();
    bsLabel->setStyleSheet("image:url(:/nianji/erji/nj_bt03.png)");
    bsLabel->show();
    setVideoCtl();
    initVolCtl();

    middle->download(bsVideo, bsVideoSize);
}

void WordPage::gbsfClick()
{
    QString videoUrl = middle->getVideoPath("ggxz", ybVideo,ybVideoSize);

//    bool flashPlaying = flash->dynamicCall("Playing").toBool();
    if (flash) {
        flash->dynamicCall("Rewind()");
        flash->hide();
    }

    videoLabel->show();
    showVideoCtl();
    video->stop();
    qDebug() << __FUNCTION__  << videoUrl;
    videoLabel->clear();
    if (ybVideo == "0") {
        videoLabel->setText("视频正在努力制作中，敬请期待！");
    }
//    video->loadVlc("c:\\work\\test2.wmv", (void*)videoLabel->winId());
    video->loadVlc(videoUrl, (void*)videoLabel->winId());
    video->setVolume(middle->getVolume());
    initVolCtl();
    video->play();
    setPauseIcon();
    topLabel->hide();
    bsLabel->setStyleSheet("image:url(:/nianji/erji/nj_bt04.png)");
    bsLabel->show();
    setVideoCtl();
    initVolCtl();
    middle->download(ybVideo, ybVideoSize);
}

void WordPage::mbsfClick()
{
    QString videoUrl = middle->getVideoPath("ggxz", mbVideo, mbVideoSize);

//    bool flashPlaying = flash->dynamicCall("Playing").toBool();
    if (flash) {
        flash->dynamicCall("Rewind()");
        flash->hide();
    }

    videoLabel->show();
    showVideoCtl();
    video->stop();
    qDebug() << __FUNCTION__  << videoUrl;
    qDebug() << __FUNCTION__  << "mbVideo" << mbVideo;
    videoLabel->clear();
    if (mbVideo == "0") {
        videoLabel->setText("视频正在努力制作中，敬请期待！");
    }
 //    video->loadVlc("c:\\work\\test2.wmv", (void*)videoLabel->winId());
    video->loadVlc(videoUrl, (void*)videoLabel->winId());
    video->setVolume(middle->getVolume());
    initVolCtl();
    video->play();
    setPauseIcon();
    topLabel->hide();
    bsLabel->setStyleSheet("image:url(:/nianji/erji/nj_bt02.png)");
    bsLabel->show();
    setVideoCtl();
    initVolCtl();
    middle->download(mbVideo, mbVideoSize);
}

void WordPage::sfxsClick()
{
    bool show = false;
    sfxsButton->blockSignals(true);
    if (!videoLabel->isHidden()) {
        video->stop();
        videoLabel->hide();
        hideVideoCtl();
        show = true;
    }

    bool flashPlaying = flash->dynamicCall("Playing").toBool();
    if (flash) {
        flash->dynamicCall("Rewind()");
        flash->hide();
    }

    swjz->setSwjzPage(wordPage, word, sfxsButton, videoLabel, videoBgLabel, show, flash, flashPlaying);
    stackWidget->setCurrentWidget(swjzPage);

    // 改为播放flash
//    flashInstance->setFlashPage("image:url(:/xzzs/bj_11.jpg)", swjzFlashUrl, swjzFlashSize, sfxsButton, wordPage, flash);
//    stackWidget->setCurrentWidget(flashPage);
}

void WordPage::onBackClick()
{
    timer->stop();
    listWidgetParent->blockSignals(false);
    listWidgetParent->clearSelection();
    if (flash) {
        flash->dynamicCall("Rewind()");
        delete flash;
        flash = NULL;
    }
    video->stop();
    videoLabel->hide();
    stackWidget->setCurrentWidget(parentPage);
    topLabel->show();
    bsLabel->hide();
    hideVideoCtl();
    if (videoCtlTimer->isActive()) {
        videoCtlTimer->stop();
    }
    lengthLabel->clear();
    curTimeLabel->clear();
}

void WordPage::onVideoDoubleClick()
{
    qDebug() << __FUNCTION__  << "video double click";
    if (isFullScreen) {
        isFullScreen = false;
        emit showNormal();

        videoLabel->setGeometry(stackWidget->x()+333,stackWidget->y()+160,360,315);
        videoLabel->show();

        showVideoCtl();
        posSlider->setGeometry(stackWidget->x()+338,stackWidget->y()+460,350,8);
        curTimeLabel->setGeometry(stackWidget->x()+380,stackWidget->y()+480,80,20);
        lengthLabel->setGeometry(stackWidget->x()+465,stackWidget->y()+480,100,20);
        btnPlay->setGeometry(stackWidget->x()+348,stackWidget->y()+480,18,20);
        btnVol->setGeometry(stackWidget->x()+600,stackWidget->y()+455,20,18);
        btnFull->setGeometry(stackWidget->x()+650,stackWidget->y()+480,22,16);
        volSlider->setGeometry(stackWidget->x()+625,stackWidget->y()+460,60,8);

        if (videoCtlTimer->isActive()) {
            videoCtlTimer->stop();
        }
    } else {
        emit fullScreen();
        isFullScreen = true;
        videoLabel->resize(rootWidget->width(),QApplication::desktop()->height());
        videoLabel->move(0,0);
        videoLabel->show();

        posSlider->setGeometry(10,QApplication::desktop()->height()-60,rootWidget->width()-20,8);
        curTimeLabel->setGeometry(60,QApplication::desktop()->height()-40,80,20);
        lengthLabel->setGeometry(145,QApplication::desktop()->height()-40,100,20);
        btnPlay->setGeometry(30,QApplication::desktop()->height()-40,18,20);
        btnVol->setGeometry(rootWidget->width()-120,QApplication::desktop()->height()-65,20,18);
        btnFull->setGeometry(rootWidget->width()-70,QApplication::desktop()->height()-40,22,16);
        volSlider->setGeometry(rootWidget->width()-90,QApplication::desktop()->height()-60,60,8);
        videoCtlRect->setGeometry(0,QApplication::desktop()->height()-65,rootWidget->width(),65);
        videoCtlRect->show();

        videoCtlRect->raise();

        posSlider->raise();
        curTimeLabel->raise();
        lengthLabel->raise();
        btnPlay->raise();
        btnVol->raise();
        btnFull->raise();
        volSlider->raise();

        if (videoCtlTimer->isActive()) {
            videoCtlTimer->stop();
        }
        videoCtlTimer->start(5000);
    }
}

void WordPage::resizeVideo()
{
    if (!videoLabel->isHidden()) {
        videoLabel->setGeometry(stackWidget->x()+333,stackWidget->y()+160,360,315);

        posSlider->setGeometry(stackWidget->x()+338,stackWidget->y()+460,350,8);
        curTimeLabel->setGeometry(stackWidget->x()+380,stackWidget->y()+480,80,20);
        lengthLabel->setGeometry(stackWidget->x()+465,stackWidget->y()+480,100,20);
        btnPlay->setGeometry(stackWidget->x()+348,stackWidget->y()+480,18,20);
        btnVol->setGeometry(stackWidget->x()+600,stackWidget->y()+455,20,18);
        btnFull->setGeometry(stackWidget->x()+650,stackWidget->y()+480,22,16);
        volSlider->setGeometry(stackWidget->x()+625,stackWidget->y()+460,60,8);
    }
}

void WordPage::onVideoClick()
{
    if (videoStop) {
        video->stop();
        video->play();

        setPauseIcon();
        videoStop = false;
        return;
    }
    if (video->getVideoState()) {
        setPlayIcon();
        video->pause();
    } else {
        setPauseIcon();
        video->play();
    }
}

void WordPage::onMouseMove()
{
    if (isFullScreen) {
        showVideoCtl();
        videoCtlRect->show();

        if (videoCtlTimer->isActive()) {
            videoCtlTimer->stop();
        }
        videoCtlTimer->start(5000);
    }
}

//void WordPage::updateVideoCtl()
//{
//    posSlider->hide();
//    curTimeLabel->hide();
//    lengthLabel->hide();
//    volSlider->hide();
//    btnPlay->hide();
//    btnVol->hide();
//    btnFull->hide();
//}

void WordPage::changePosition(int pos)
{
//    qDebug() << __FUNCTION__  << pos;
    float posf = (float)(pos)/100.0;
    video->setPosition(posf);
}

void WordPage::onPressed()
{
    int pos = posSlider->value();
//    qDebug() << __FUNCTION__  << pos;
    float posf = (float)(pos)/100.0;
    video->setPosition(posf);
}

void WordPage::updateInfo()
{
    curTimeLabel->setText(video->getTime());
    lengthLabel->setText("/ " + video->getLength());

    if (video->getLengthInt() > 0 && (video->getLengthInt()-video->getTimeInt()) < 1100) {
        videoStop = true;
        video->pause();
        setPlayIcon();
    }

    if (video->getLengthInt() != 0) {
        int pos = video->getTimeInt()*100/video->getLengthInt();
        posSlider->setValue(pos);
    }
}

bool WordPage::eventFilter(QObject *obj, QEvent *event)
{
    if(obj==posSlider)
    {
        if (event->type()==QEvent::MouseButtonPress)           //判断类型
        {
            QMouseEvent *mouseEvent = static_cast<QMouseEvent *>(event);
            if (mouseEvent->button() == Qt::LeftButton)	//判断左键
            {
               int dur = posSlider->maximum() - posSlider->minimum();
               int pos = posSlider->minimum() + dur * ((double)mouseEvent->x() / posSlider->width());
               if(pos != posSlider->sliderPosition())
                {
                   changePosition(pos);
                }
            }

        }
    }

    if (obj==volSlider) {
        if (event->type()==QEvent::MouseButtonPress)           //判断类型
        {
            QMouseEvent *mouseEvent = static_cast<QMouseEvent *>(event);
            if (mouseEvent->button() == Qt::LeftButton)	//判断左键
            {
               int dur = volSlider->maximum() - volSlider->minimum();
               int pos = volSlider->minimum() + dur * ((double)mouseEvent->x() / volSlider->width());
               if(pos != volSlider->sliderPosition())
                {
                  volSlider->setValue(pos);
                  qDebug() << __FUNCTION__  << "vol value:" << pos;
                  video->setVolume(pos);
                  middle->setVolume(pos);
                }
            }

        }
    }

    if(obj==flash)
    {
        if (event->type()==QEvent::MouseButtonDblClick)           //判断类型
        {
            QMouseEvent *mouseEvent = static_cast<QMouseEvent *>(event);
            if (mouseEvent->button() == Qt::LeftButton)	//判断左键
            {
               onDoubleClick();
            }
        }
    }
    return QObject::eventFilter(obj,event);
}

void WordPage::hideVideoCtl()
{
    posSlider->hide();
    curTimeLabel->hide();
    lengthLabel->hide();
    volSlider->hide();
    btnPlay->hide();
    btnVol->hide();
    btnFull->hide();
    videoBgLabel->hide();
    videoCtlRect->hide();
}

void WordPage::showVideoCtl()
{
    posSlider->show();
    curTimeLabel->show();
    lengthLabel->show();
//    volSlider->show();
    btnPlay->show();
//    btnVol->show();
    btnFull->show();
    videoBgLabel->show();
}

void WordPage::onVideoEndReached()
{
    video->stop();
    setPlayIcon();
}

void WordPage::initVideoCtl()
{
    QFont font;
    font.setFamily("宋体");
    font.setPixelSize(20);

    QPalette pa;
    pa.setColor(QPalette::WindowText,QColor(211,211,211));

    // 设置控制栏背景
    videoBgLabel->setGeometry(333,450,360,60);
    videoBgLabel->raise();
    videoBgLabel->hide();
    videoBgLabel->setStyleSheet("background-color: black");

    // 设置进度条
    posSlider->hide();
    posSlider->raise();
    posSlider->setGeometry(stackWidget->x()+338,stackWidget->y()+460,350,8);

    // 设置视频时间显示
    curTimeLabel->setGeometry(stackWidget->x()+380,stackWidget->y()+480,80,20);
    curTimeLabel->setFont(font);
    curTimeLabel->setPalette(pa);
    curTimeLabel->hide();
    curTimeLabel->raise();
    curTimeLabel->setAlignment(Qt::AlignRight);
    lengthLabel->setGeometry(stackWidget->x()+465,stackWidget->y()+480,100,20);
    lengthLabel->setFont(font);
    lengthLabel->setPalette(pa);
    lengthLabel->hide();
    lengthLabel->raise();
    lengthLabel->setAlignment(Qt::AlignLeft);
    timer->start(1000);

    // 设置控制按钮图标
    QPixmap playP1(":/player/play01.png");
    QPixmap playP2(":/player/play02.png");
    btnPlay->setNormalImage(playP1);
    btnPlay->setPressImage(playP2);
    QPixmap volP1(":/player/vol01.png");
    QPixmap volP2(":/player/vol02.png");
    btnVol->setNormalImage(volP1);
    btnVol->setPressImage(volP2);
    QPixmap fullP1(":/player/full01.png");
    QPixmap fullP2(":/player/full02.png");
    btnFull->setNormalImage(fullP1);
    btnFull->setPressImage(fullP2);

    // 设置控制按钮坐标
    btnPlay->raise();
    btnPlay->setGeometry(stackWidget->x()+348,stackWidget->y()+480,18,20);
    btnVol->raise();
    btnVol->setGeometry(stackWidget->x()+600,stackWidget->y()+455,20,18);
    btnFull->raise();
    btnFull->setGeometry(stackWidget->x()+650,stackWidget->y()+480,22,16);

    // 设置声音控制
    volSlider->raise();
    volSlider->setGeometry(stackWidget->x()+625,stackWidget->y()+460,60,8);
}

void WordPage::setVideoCtl()
{
    // 设置播放按钮
    if (video->getVideoState()) {
        setPauseIcon();
    } else {
        setPlayIcon();
    }

    // 设置声音
    int vol = video->getVolume();
    qDebug() << __FUNCTION__  << "get volume:" << vol;
    volSlider->setValue(vol);
}

void WordPage::onVolClick()
{
    qDebug() << __FUNCTION__  << video->getMute();

    if (video->getMute()) { //当前状态为静音
        video->setMute(0);
        Common::muteStatus = false;

        QPixmap volP1(":/player/vol01.png");
        QPixmap volP2(":/player/vol02.png");
        btnVol->setNormalImage(volP1);
        btnVol->setPressImage(volP2);
        btnVol->repaint();
    } else {
        video->setMute(1);
        Common::muteStatus = true;

        QPixmap muteP1(":/player/mute01.png");
        QPixmap muteP2(":/player/mute02.png");
        btnVol->setNormalImage(muteP1);
        btnVol->setPressImage(muteP2);
        btnVol->repaint();
    }
}

void WordPage::initVolCtl()
{
    qDebug() << __FUNCTION__  << Common::muteStatus;

    if (Common::muteStatus) { //当前状态为静音
        video->setMute(1);

        QPixmap muteP1(":/player/mute01.png");
        QPixmap muteP2(":/player/mute02.png");
        btnVol->setNormalImage(muteP1);
        btnVol->setPressImage(muteP2);
        btnVol->repaint();
    } else {
        video->setMute(0);

        QPixmap volP1(":/player/vol01.png");
        QPixmap volP2(":/player/vol02.png");
        btnVol->setNormalImage(volP1);
        btnVol->setPressImage(volP2);
        btnVol->repaint();
    }
}

void WordPage::onVolSliderMoved(int pos)
{
    video->setVolume(pos);
    middle->setVolume(pos);
}

void WordPage::setPlayIcon()
{
    QPixmap playP1(":/player/play01.png");
    QPixmap playP2(":/player/play02.png");
    btnPlay->setNormalImage(playP1);
    btnPlay->setPressImage(playP2);
    btnPlay->repaint();
}

void WordPage::setPauseIcon()
{
    QPixmap playP1(":/player/pause01.png");
    QPixmap playP2(":/player/pause02.png");
    btnPlay->setNormalImage(playP1);
    btnPlay->setPressImage(playP2);
    btnPlay->repaint();
}

void WordPage::onDoubleClick()
{
    if (flashIsFull) {
        flashIsFull = false;
        emit showNormal();

        flash->setParent(rootWidget,0);
        flash->setWindowFlags(Qt::FramelessWindowHint);
        flash->setGeometry(stackWidget->x()+333,stackWidget->y()+140,360,360);
        flash->show();
    } else {
        flashIsFull = true;
        emit fullScreen();
        qDebug() << "flash 全屏" << rootWidget->width() << QApplication::desktop()->height();
        flash->setParent(0,0);
        flash->showFullScreen();
    }
}
